/*
  Name: Aegis.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine Subsystem Unification
*/

/* License (ZLIB-license variation)

- This software is provided 'as-is', without any express or implied warranty.
  In no event will the author(s) be held liable for any damages arising from the
  use of this software or software based on it. 

- Permission is granted to anyone to use this software for any purpose, including
  commercial applications, and to alter it and redistribute it freely, subject to
  the following restrictions:
   - The origin of this software must not be misrepresented; you must not claim
     that you wrote the original software. If you use this software in a product,
     an acknowledgment in the product is required. 
   - Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software. Redistribution of the modified
     sources should conform GPL habbits.
   - Additionals (add-ons) to the library and/or modifications should be submited
     to the authors of original software. 
   - This notice may not be removed or altered from any source distribution.

- The author and maintainer of the original software is:
  Jan Ringo, http://Tringi.Mx-3.cz, mail to: Tringi 'at' Mx-3 'dot' cz

*/


#ifndef _AEGIS_HPP
#define _AEGIS_HPP

namespace Aegis {
    // Forward global definition of namespace
};    

#ifdef USING_AEGIS
using namespace Aegis;
#endif

#include "Attribs.hpp" // Attribute compatibility
#include "A/aA.hpp" // Artifical Inteligence - path finding, neural networks, ...
#include "C/aC.hpp" // Control - Mouse, Keyboard, ... abstraction classes
#include "F/aF.hpp" // File Formats - aBitmap, aTexMap, ...
#include "G/aG.hpp" // Graphics - OpenGL levels of abtraction
#include "I/aI.hpp" // Interface - set of classes for Graphics User Interface 
#include "M/aM.hpp" // Math - all kind of computing
#include "P/aP.hpp" // Physical modeling - collisions and other
#include "S/aS.hpp" // Sound - sound mixer and processing
#include "W/aW.hpp" // Window - support classes for Window under MS Windows
#include "X/aX.hpp" // eXecution - Game Core processing classes

#endif

/*
  Acknowledgements:
  -----------------

  Parts of Aegis Engine are based on other sources, here is the list:

    SSE and 3Dnow! assmbler routines
        - Iain Nicholson [ijnicholson@users.sourceforge.net]
    Some vector manipulations
        - provided by Marek Mizanin [zanir@szm.sk]

  The project uses these libraries:        
    
    Windows KERNEL32, USER32, GDI32, WINMM, and other libraries
        - provided as DLL by Microsoft with OS Windows
        
    OpenGL implementation
        - by Microsoft
    
    ZLIB compression library (1.2.1)
        - Jean-loup Gailly and Mark Adler
        - [jloup@gzip.org / madler@alumni.caltech.edu]
    
   All company and product names are subjects of copyright or trademarks
   and are properties of their respective owners.

*/

