/*
  Name: aC_DiscDrive.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - C (Control) Subsystem - Optical Drive handler
*/

#include "aC_DiscDrive.hpp"
#include <winioctl.h>
#include <dbt.h>
#include <stdio.h>

#ifndef FILE_READ_ONLY_VOLUME
#define FILE_READ_ONLY_VOLUME           0x00080000  
#endif

__fastcall Aegis::C::DiscDrive::DiscDrive(char ltr) {
    DWORD drives_mask = GetLogicalDrives();
    
    if (ltr) {
        if (drives_mask & (1 << (ltr - 'A')))
            this->drive = ltr - 'A';
        else
            this->drive = 0;
            
    } else {
        for (int i = 2; i < 'Z'-'A'; ++i, drives_mask >>= 1)
            if (drives_mask & 1) {
                char sz[4] = " :\\";
                sz[0] = 'A'+i;
                
                if (GetDriveType(sz) == DRIVE_CDROM) {
                    this->drive = i;
                    break; // got it
                };
            };
    };
    
    return;
};    

__fastcall Aegis::C::DiscDrive::~DiscDrive() {

    return;
};

bool Aegis::C::DiscDrive::ProcessMessage( HWND hwnd, UINT msg,
                                          WPARAM wParam, LPARAM lParam ) {
    if (!this->drive) return false;

    switch (msg) {
        case WM_DEVICECHANGE: {
            switch (wParam) {
                case DBT_DEVICEARRIVAL: {
                    DEV_BROADCAST_VOLUME *vol = (DEV_BROADCAST_VOLUME*) lParam;
                    if (vol->dbcv_devicetype != DBT_DEVTYP_VOLUME ) return false;
                    if (!(vol->dbcv_flags & DBTF_MEDIA)) return false;
                    
                    if (vol->dbcv_unitmask & (1 << this->drive) ) this->Update();
                } return true;
                case DBT_DEVICEREMOVECOMPLETE: {
                    DEV_BROADCAST_VOLUME *vol = (DEV_BROADCAST_VOLUME*) lParam;
                    if (vol->dbcv_devicetype != DBT_DEVTYP_VOLUME ) return false;
                    if (!(vol->dbcv_flags & DBTF_MEDIA)) return false;
                    
                    if (vol->dbcv_unitmask & (1 << this->drive) ) this->Update();
                } return true;
            };    
        } break;
    };    
    return false;
};    

void Aegis::C::DiscDrive::Update() {
    if (!this->drive) return;
    
    SetLastError(0);
    SetErrorMode( SEM_FAILCRITICALERRORS );

    char path[4] = " :\\";
    path[0] = this->drive + 'A';
    
    DWORD flags = 0;
    GetVolumeInformation( path, this->name, 256,
                          &this->serial, NULL, &flags,
                          filesystem, 256 );
    
    if (GetLastError() == ERROR_NOT_READY) {
        if (this->is) {
            this->is = false;
            this->RemoveHandler( this->drive + 'A' );
        };
            
        this->name[0] = 0;
        this->filesystem[0] = 0;
        this->serial = 0;
        this->free_space = 0;
        
        this->read_only = false;
        this->case_sensitive = false;
        this->case_preserved = false;
        this->unicode_system = false;
            
    } else {
        this->read_only = flags & FILE_READ_ONLY_VOLUME;
        this->case_sensitive = flags & FS_CASE_SENSITIVE;
        this->case_preserved = flags & FS_CASE_IS_PRESERVED;
        this->unicode_system = flags & FS_UNICODE_STORED_ON_DISK;
        
        GetDiskFreeSpaceEx( this->name, (ULARGE_INTEGER*) &this->free_space,
                            NULL, NULL );

        if (!this->is) {
            this->is = true;
            this->ArriveHandler( this->drive + 'A' );
        };
    };        

    SetErrorMode(0);
    
    return;
};

int Aegis::C::DiscDrive::ArriveHandler(char letter) { return 0; };
int Aegis::C::DiscDrive::RemoveHandler(char letter) { return 0; };

int Aegis::C::AssemblyDrives( Aegis::C::DiscDrive **dda, DWORD max ) {
    if (!dda || !max) return 0;
    
    DWORD drives_mask = GetLogicalDrives();
    int n = 0;
    
    for (int i = 2; i < 'Z'-'A'; ++i, drives_mask >>= 1)
        if (drives_mask & 1) {
            char sz[4] = " :\\";
            sz[0] = 'A'+i;
            
            if (GetDriveType(sz) == DRIVE_CDROM) {
                dda[n++] = new Aegis::C::DiscDrive(sz[0]);
                if (n == max) break;
            };
        };
    
    return n;
};


