/*
  Name: aC_DiscDrive.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - C (Control) Subsystem - Optical Drive handler
*/

#ifndef _AC_DISCDRIVE_HPP
#define _AC_DISCDRIVE_HPP

#include <windows.h>

namespace Aegis {
    namespace C {
        
        class DiscDrive {
            private:
                unsigned int drive;
                
                // current data
                bool  is;
                bool  read_only;

                char  name[256];
                char  filesystem[256];

                __int64 free_space;
                DWORD serial;
                
                bool  case_sensitive;
                bool  case_preserved;
                bool  unicode_system;
                
                // some internals
                void Update();
            
            protected:
                virtual int ArriveHandler( char letter );
                virtual int RemoveHandler( char letter );
                
            public:
                __fastcall DiscDrive(char letter = 0); // to watch over ('A'...'Z')
                __fastcall ~DiscDrive();
                
                bool ProcessMessage( HWND hwnd, UINT msg,
                                     WPARAM wParam, LPARAM lParam );

                inline char GetLetter() const { return this->drive + 'A'; };
                inline bool IsInserted() const { return this->is; };
                inline bool IsCaseSensitive() const{return this->case_sensitive;};
                inline bool IsUnicode() const { return this->unicode_system; };

                inline __int64 FreeSpace () const { return this->free_space; };
                inline DWORD SerialNumber () const { return this->serial; };

                inline const char* Label() const { return this->name; };
                inline const char* FileSystem() const { return this->filesystem; };
                
        };
        
        // support management functions
        // this creates array of at least max drives and returns the number
       int AssemblyDrives( Aegis::C::DiscDrive**, DWORD max );
    };
};

#endif
            
