/*
  Name: aC_Joystick.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - C (Control) Subsystem - Joystick abstraction
*/

#ifndef _AC_JOYSTICK_HPP
#define _AC_JOYSTICK_HPP

#include <windows.h>
#include "../M/M2/aM2_Normal.hpp"

namespace Aegis {
    namespace C {
        
        class Joystick {
            private:
                unsigned int joystick;
                
                JOYCAPS joycaps;
                JOYINFOEX joyinfo;
                
                // button counters
                unsigned int imm_on_table[32];
                unsigned int inc_table[32];
                unsigned int time_table[32];
                unsigned int uptime_table[32];
                
                // settings
                float deadzone;
                float fullzone;

                // internals
                bool quit;
                CRITICAL_SECTION cs;
                HANDLE hThread;
                static DWORD WINAPI PollingThread(LPVOID);
            private:
                __fastcall float Adjust( float axis );

            protected:
                __fastcall virtual int MovementHandler( unsigned int axis );
                __fastcall virtual int POVHandler();
                __fastcall virtual int UpDownHandler( unsigned int button,
                                                      bool down );
                __fastcall virtual int OnOffHandler( unsigned int button,
                                                     bool down );
                
            public:
                __fastcall Joystick(unsigned int joy = 0);
                __fastcall ~Joystick();
                
                __fastcall bool DeadZone( float dz ); // 0.0 ... 0.5
                __fastcall bool FullZone( float fz ); // 0.5 ... 1.0
                
                // joystick states
                __fastcall float GetX(); // -1.0 ... 1.0
                __fastcall float GetY();
                __fastcall float GetZ();
                __fastcall float GetR();
                __fastcall float GetU();
                __fastcall float GetV();

                __fastcall float GetPOV();
                inline bool IsPOVCentered() {
                    if (this->joyinfo.dwPOV == JOY_POVCENTERED) return true;
                                                           else return false;
                };
                
                inline Aegis::M2::Normal GetAxis() {
                    Aegis::M2::Normal n( this->GetX(), this->GetY(), this->GetZ() );
                    return n.Normalized();
                };    
                inline Aegis::M2::Normal GetAxis1() { return this->GetAxis(); };
                inline Aegis::M2::Normal GetAxis2() {
                    Aegis::M2::Normal n( this->GetR(), this->GetU(), this->GetV() );
                    return n.Normalized();
                };    
                
                // button states
                inline bool IsDown( unsigned int button ) { 
                    return this->joyinfo.dwButtons & (1 << button);
                };
                inline bool IsOn( unsigned int button ) {
                    if (button > 31) return false; // 31 is last buton
                    return this->imm_on_table[ button ];
                };
                inline void TurnOn( unsigned int button ) {
                    if (button < 32)
                        this->imm_on_table[ button ] = true;
                };
                inline void TurnOff( unsigned int button ) {
                    if (button < 32)
                        this->imm_on_table[ button ] = false;
                };
                
                // button counters
                __fastcall unsigned int GetHits( unsigned int button,
                                                 unsigned int max = 0xFFFFFFFF );
                __fastcall unsigned int GetTime( unsigned int button,
                                                 unsigned int max = 0xFFFFFFFF );
                __fastcall unsigned int GetUpTime( unsigned int button,
                                                   unsigned int max = 0xFFFFFFFF );
                
        };
        
    };
};

#endif
    
