/*
  Name: aC_Keyboard.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - C (Control) Subsystem - Keyboard handler
*/

#include "aC_Keyboard.hpp"

__fastcall Aegis::C::Keyboard::Keyboard(bool accurate) {
    InitializeCriticalSection( &this->cs );
    this->cs.SpinCount = 4000; // TODO: FIXME: Is this correct ???
    
    ZeroMemory( this->imm_down_table, sizeof(unsigned int) * 0xFF );
    ZeroMemory( this->imm_on_table, sizeof(unsigned int) * 0xFF );
    ZeroMemory( this->inc_table, sizeof(unsigned int) * 0xFF );
    ZeroMemory( this->time_table, sizeof(unsigned int) * 0xFF );
    ZeroMemory( this->uptime_table, sizeof(unsigned int) * 0xFF );
    
    if (accurate) 
        this->perf_timer
            = QueryPerformanceFrequency((LARGE_INTEGER *)
                                        &(this->perf_timer_freq));
    else
        this->perf_timer = false;

    this->quit = false;
    DWORD dw = 0;
    this->hThread = CreateThread( NULL, 65536, this->ThreadCounter, this, 0, &dw);

    return;
};    

__fastcall Aegis::C::Keyboard::~Keyboard() {
    this->quit = true;
    WaitForSingleObject( this->hThread, INFINITE );
    
    DeleteCriticalSection( &this->cs );
    return;
};    

bool Aegis::C::Keyboard::ProcessMessage( HWND hwnd, UINT msg,
                                         WPARAM wParam, LPARAM lParam ) {
    switch(msg) {
        case WM_KEYDOWN:
            if (wParam >= 0xFF) return false;

            if (!this->imm_down_table[ wParam ]) { // no repeated
                this->imm_on_table[ wParam ] = !this->imm_on_table[ wParam ];
                this->imm_down_table[ wParam ] = true;
                
                EnterCriticalSection( &this->cs ); // is this really needed?
                ++this->inc_table[ wParam ];
                LeaveCriticalSection( &this->cs );

                // handlers
                this->UpDownHandler( wParam, true );
                this->OnOffHandler( wParam, this->imm_on_table[ wParam ] );

                // this is here because handlers must know the uptime
                EnterCriticalSection( &this->cs ); // is this really needed?
                this->uptime_table[ wParam ] = 0;
                LeaveCriticalSection( &this->cs );
            };
        return true;
        case WM_KEYUP:
            if (wParam >= 0xFF) return false;

            this->imm_down_table[ wParam ] = false;
            // handler
            this->UpDownHandler( wParam, false );
        return true;
    };
    return false;
};

DWORD WINAPI Aegis::C::Keyboard::ThreadCounter(LPVOID lpvoid) {
    Aegis::C::Keyboard *This = (Aegis::C::Keyboard *)lpvoid;
    __int64 temp, temp2;

    if (This->perf_timer)
        while(!This->quit) {
            QueryPerformanceCounter((LARGE_INTEGER *) &(temp) );
            Sleep(20);
            QueryPerformanceCounter((LARGE_INTEGER *) &(temp2) );
            
            temp2 -= temp;
            temp2 *= 1000ll;
            temp2 /= This->perf_timer_freq;
    
            EnterCriticalSection( &This->cs ); // is CS really needed?
            for (int i = 0; i < 0xFF; ++i)
                if (This->imm_down_table[i]) {
                    This->time_table[i] += temp2;
                } else {
                    This->uptime_table[i] += temp2;
                };    
            LeaveCriticalSection( &This->cs );
        } 
    else
        while(!This->quit) {
            temp = GetTickCount();
            Sleep(50);
            temp = GetTickCount() - temp;
    
            EnterCriticalSection( &This->cs ); // is CS really needed?
            for (int i = 0; i < 0xFF; ++i)
                if (This->imm_down_table[i]) {
                    This->time_table[i] += temp2;
                } else {
                    This->uptime_table[i] += temp2;
                };    
            LeaveCriticalSection( &This->cs );
        }
    
    return 0;
};    

__fastcall bool Aegis::C::Keyboard::IsDown( unsigned char vk ) {
    if (vk >= 0xA0 && vk <= 0xA5) // VK_Lxxx and VK_Rxxx
        return GetKeyState(vk) & 0x8000; // 0x80000000
    else
        return this->imm_down_table[ vk ];
};    

__fastcall bool Aegis::C::Keyboard::IsOn( unsigned char vk ) {
    if ((vk >= 0xA0 && vk <= 0xA5) || 
        (vk == VK_NUMLOCK || vk == VK_SCROLL || vk == VK_CAPITAL))
            return GetKeyState(vk) & 0x0001;
    else
        return this->imm_on_table[ vk ];
};    

__fastcall unsigned int Aegis::C::Keyboard::GetKeyHits( unsigned char vk,
                                                        unsigned int max ) {
    EnterCriticalSection( &this->cs );

    int n;
    if (!max) {
        n = this->inc_table[ vk ];
    } else {    
        n = ( this->inc_table[ vk ] <? max );
        this->inc_table[ vk ] -= n;
    };
    
    LeaveCriticalSection( &this->cs );
    return n;
};

__fastcall unsigned int Aegis::C::Keyboard::GetKeyTime( unsigned char vk,
                                                        unsigned int max ) {
    EnterCriticalSection( &this->cs );
    
    int n;
    if (!max) {
        n = this->time_table[ vk ];
    } else {    
        n = ( this->time_table[ vk ] <? max );
        this->time_table[ vk ] -= n;
    };
    
    LeaveCriticalSection( &this->cs );
    return n;
};

__fastcall unsigned int Aegis::C::Keyboard::GetUpTime( unsigned char vk,
                                                       unsigned int max ) {
    EnterCriticalSection( &this->cs );
    
    int n;
    if (!max) {
        n = this->uptime_table[ vk ];
    } else {    
        n = ( this->uptime_table[ vk ] <? max );
        this->uptime_table[ vk ] -= n;
    };
    
    LeaveCriticalSection( &this->cs );
    return n;
};


__fastcall int Aegis::C::Keyboard::UpDownHandler( unsigned int vk, bool down ) {
    return 0;
};    
__fastcall int Aegis::C::Keyboard::OnOffHandler( unsigned int vk, bool on ) {
    return 0;
};    

