/*
  Name: aC_Keyboard.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - C (Control) Subsystem - Keyboard handler
*/

#ifndef _AC_KEYBOARD_HPP
#define _AC_KEYBOARD_HPP

#include <windows.h>

namespace Aegis {
    namespace C {
        
        class Keyboard {
            private:
                CRITICAL_SECTION cs;
                HANDLE hThread; // time counter thread
                
                unsigned int imm_down_table[0xFF];
                unsigned int imm_on_table[0xFF];
                unsigned int inc_table[0xFF];

                unsigned int time_table[0xFF];
                unsigned int uptime_table[0xFF];
                
                bool quit;
                bool perf_timer; 
                __int64 perf_timer_freq;
                
                static DWORD WINAPI ThreadCounter(LPVOID);
                
            protected:
                // handlers do not distinguish left an right keys
                __fastcall virtual int UpDownHandler( unsigned int vk, bool down );
                __fastcall virtual int OnOffHandler( unsigned int vk, bool on );
                
            public:
                __fastcall Keyboard(bool accurate = true);
                __fastcall ~Keyboard();

                bool ProcessMessage( HWND hwnd, UINT msg,
                                     WPARAM wParam, LPARAM lParam );
                
                // current state (these only distinguish LEFT and RIGHT keys)
                // vk is VK_xxx or '0' ... '9' or 'A' ... 'Z'
                __fastcall bool IsDown( unsigned char vk );
                __fastcall bool IsOn  ( unsigned char vk );

                // static asynchro functions for global use
                static inline bool IsKeyDownNow( unsigned char vk ) {
                    return GetAsyncKeyState( vk ) & 0x8000;
                };
                static inline bool IsKeyOnNow( unsigned char vk ) {
                    return GetAsyncKeyState( vk ) & 0x0001;
                };
                
                // retrieves number of keypress and decrements it to
                // (CURRENT-max)<?0. Number of key presses is returned
                // according to max.
                __fastcall unsigned int GetKeyHits( unsigned char vk,
                                                    unsigned int max=0xFFFFFFFF );

                // returns number of millisecond (up to max) the key have been
                // held down and sets this number (CURRENT-max)<?0
                __fastcall unsigned int GetKeyTime( unsigned char vk,
                                                    unsigned int max=0xFFFFFFFF );
                __fastcall unsigned int GetUpTime( unsigned char vk,
                                                   unsigned int max=0xFFFFFFFF );
        };
    };
};

#endif

