/*
  Name: aC_Mouse.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - C (Control) Subsystem - Mouse handler
  
  Not as effective as it could be.
*/

#include "aC_Mouse.hpp"

#ifndef WM_XBUTTONDOWN
#define WM_XBUTTONDOWN                  0x020B
#define WM_XBUTTONUP                    0x020C
#define WM_XBUTTONDBLCLK                0x020D
#endif

#ifndef MK_XBUTTON1
#define MK_XBUTTON1         0x0020
#define MK_XBUTTON2         0x0040
#endif 

__fastcall Aegis::C::Mouse::Mouse(bool accurate) {
    InitializeCriticalSection( &this->cs );
    this->cs.SpinCount = 4000; // TODO: FIXME: Is this correct ???
    
    ZeroMemory( this->imm_down_table, sizeof(unsigned int) * 5 );
    ZeroMemory( this->imm_on_table, sizeof(unsigned int) * 5 );
    ZeroMemory( this->inc_table, sizeof(unsigned int) * 5 );
    ZeroMemory( this->time_table, sizeof(unsigned int) * 5 );
    ZeroMemory( this->uptime_table, sizeof(unsigned int) * 5 );
    
    if (accurate) 
        this->perf_timer
            = QueryPerformanceFrequency((LARGE_INTEGER *)
                                        &(this->perf_timer_freq));
    else
        this->perf_timer = false;

    this->quit = false;
    DWORD dw = 0;
    this->hThread = CreateThread( NULL, 65536, this->ThreadCounter, this, 0, &dw);

    this->x = 0;
    this->y = 0;

    return;
};    

__fastcall Aegis::C::Mouse::~Mouse() {
    this->quit = true;
    WaitForSingleObject( this->hThread, INFINITE );
    
    DeleteCriticalSection( &this->cs );
    return;
};    

bool Aegis::C::Mouse::ProcessMessage( HWND hwnd, UINT msg,
                                      WPARAM wParam, LPARAM lParam ) {
    switch(msg) {
        case WM_MOUSEMOVE: {
            POINT pt;
            GetCursorPos( &pt );
            
            this->x = pt.x;
            this->y = pt.y;
            
            // handler
            this->MovementHandler();
        } return true;
        
        case WM_LBUTTONDOWN:
            this->imm_on_table[0] = !this->imm_on_table[0];
            this->imm_down_table[0] = true;

            EnterCriticalSection( &this->cs ); // is this really needed?
            ++this->inc_table[0];
            LeaveCriticalSection( &this->cs );
            
            // handlers
            this->UpDownHandler( 0, true );
            this->OnOffHandler( 0, this->imm_on_table[ wParam ] );

            // this is here because handlers must know the uptime
            EnterCriticalSection( &this->cs ); // is this really needed?
            this->uptime_table[0] = 0;
            LeaveCriticalSection( &this->cs );
        return true;
        case WM_LBUTTONUP:
            this->imm_down_table[0] = false;
            this->UpDownHandler( 0, false );
        return true;
        
        case WM_RBUTTONDOWN:
            this->imm_on_table[1] = !this->imm_on_table[1];
            this->imm_down_table[1] = true;

            EnterCriticalSection( &this->cs ); // is this really needed?
            ++this->inc_table[1];
            LeaveCriticalSection( &this->cs );
            
            // handlers
            this->UpDownHandler( 1, true );
            this->OnOffHandler( 1, this->imm_on_table[ wParam ] );

            EnterCriticalSection( &this->cs ); // is this really needed?
            this->uptime_table[1] = 0;
            LeaveCriticalSection( &this->cs );
        return true;
        case WM_RBUTTONUP:
            this->imm_down_table[1] = false;
            this->UpDownHandler( 1, false );
        return true;
        
        case WM_MBUTTONDOWN:
            this->imm_on_table[2] = !this->imm_on_table[2];
            this->imm_down_table[2] = true;

            EnterCriticalSection( &this->cs ); // is this really needed?
            ++this->inc_table[2];
            LeaveCriticalSection( &this->cs );
            
            // handlers
            this->UpDownHandler( 2, true );
            this->OnOffHandler( 2, this->imm_on_table[ wParam ] );

            EnterCriticalSection( &this->cs ); // is this really needed?
            this->uptime_table[2] = 0;
            LeaveCriticalSection( &this->cs );
        return true;
        case WM_MBUTTONUP:
            this->imm_down_table[2] = false;
            this->UpDownHandler( 2, false );
        return true;
        
        case WM_XBUTTONDOWN: {
            int x = 3 + !!(wParam & MK_XBUTTON2);
            
            this->imm_on_table[x] = !this->imm_on_table[x];
            this->imm_down_table[x] = true;

            EnterCriticalSection( &this->cs ); // is this really needed?
            ++this->inc_table[x];
            LeaveCriticalSection( &this->cs );
            
            // handlers
            this->UpDownHandler( x, true );
            this->OnOffHandler( x, this->imm_on_table[ wParam ] );

            EnterCriticalSection( &this->cs ); // is this really needed?
            this->uptime_table[x] = 0;
            LeaveCriticalSection( &this->cs );
        } return true;
        case WM_XBUTTONUP: {
            int x = 3 + !!(wParam & MK_XBUTTON2);
            
            this->imm_down_table[x] = false;
            this->UpDownHandler( x, false );
        } return true;
        
    };
    return false;
};

DWORD WINAPI Aegis::C::Mouse::ThreadCounter(LPVOID lpvoid) {
    Aegis::C::Mouse *This = (Aegis::C::Mouse *)lpvoid;
    __int64 temp, temp2;

    if (This->perf_timer)
        while(!This->quit) {
            QueryPerformanceCounter((LARGE_INTEGER *) &(temp) );
            Sleep(20);
            QueryPerformanceCounter((LARGE_INTEGER *) &(temp2) );
            
            temp2 -= temp;
            temp2 *= 1000ll;
            temp2 /= This->perf_timer_freq;
    
            EnterCriticalSection( &This->cs ); // is CS really needed?
            for (int i = 0; i < 5; ++i) {
                if (This->imm_down_table[i]) {
                    This->time_table[i] += temp2;
                } else {
                    This->uptime_table[i] += temp2;
                };    
            };
            LeaveCriticalSection( &This->cs );
            
        } 
    else
        while(!This->quit) {
            temp = GetTickCount();
            Sleep(50);
            temp = GetTickCount() - temp;
    
            EnterCriticalSection( &This->cs );
            for (int i = 0; i < 5; ++i)
                if (This->imm_down_table[i]) {
                    This->time_table[i] += temp;
                } else {
                    This->uptime_table[i] += temp;
                };    
            LeaveCriticalSection( &This->cs );
        }
    
    return 0;
};    

__fastcall bool Aegis::C::Mouse::IsDown( unsigned int button ) {
    if (button >= 5) return false;
    return this->imm_down_table[ button ];
};    

__fastcall bool Aegis::C::Mouse::IsOn( unsigned int button ) {
    if (button >= 5) return false;
    return this->imm_on_table[ button ];
};

__fastcall void Aegis::C::Mouse::TurnOn( unsigned int button ) {
    if (button < 5)
        this->imm_on_table[ button ] = true;
};

__fastcall void Aegis::C::Mouse::TurnOff( unsigned int button ) {
    if (button < 5)
        this->imm_on_table[ button ] = false;
};

__fastcall bool Aegis::C::Mouse::Center( HWND hwnd ) {
    if (!hwnd) {
        int mx = GetSystemMetrics(SM_CXSCREEN);
        int my = GetSystemMetrics(SM_CYSCREEN);
        
        SetCursorPos( mx >> 1, my >> 1 );
    } else {
        if (GetForegroundWindow() != hwnd) return false;
        
        RECT r;
        GetWindowRect( hwnd, &r );
        
        SetCursorPos( r.left + ((r.right-r.left) >> 1),
                      r.top + ((r.bottom-r.top) >> 1) );
    };        
    return true;
};

__fastcall int Aegis::C::Mouse::GetX( HWND hwnd ) {
    if (!hwnd) return this->x;
    
    POINT pt = { this->x, this->y };
    ScreenToClient( hwnd, &pt );
    
    return pt.x;
};

__fastcall int Aegis::C::Mouse::GetY( HWND hwnd ) {
    if (!hwnd) return this->y;
    
    POINT pt = { this->x, this->y };
    ScreenToClient( hwnd, &pt );
    
    return pt.y;
};

__fastcall float Aegis::C::Mouse::GetOffsetX( HWND hwnd ) {
    int max = 0;
    if (!hwnd) {
        max = GetSystemMetrics(SM_CXSCREEN);
    } else {
        RECT r;
        GetClientRect( hwnd, &r );
        max = r.right;
    };
    
    if (this->GetX(hwnd) == 0.0f) return -1.0f;
    if (this->GetX(hwnd) == max-1) return 1.0f;

    return -1.0f * (float)(max - (this->GetX(hwnd) << 1)) / (float)(max);
};

__fastcall float Aegis::C::Mouse::GetOffsetY( HWND hwnd ) {
    int max = 0;
    if (!hwnd) {
        max = GetSystemMetrics(SM_CYSCREEN);
    } else {
        RECT r;
        GetClientRect( hwnd, &r );
        max = r.bottom;
    };
    
    if (this->GetY(hwnd) == 0.0f) return -1.0f;
    if (this->GetY(hwnd) == max-1) return 1.0f;

    return -1.0f * (float)(max - (this->GetY(hwnd) << 1)) / (float)(max);
};


__fastcall unsigned int Aegis::C::Mouse::GetHits( unsigned int button,
                                                  unsigned int max ) {
    EnterCriticalSection( &this->cs );

    int n;
    if (!max) {
        n = this->inc_table[ button ];
    } else {    
        n = ( this->inc_table[ button ] <? max );
        this->inc_table[ button ] -= n;
    };
    
    LeaveCriticalSection( &this->cs );
    return n;
};

__fastcall unsigned int Aegis::C::Mouse::GetTime( unsigned int button,
                                                  unsigned int max ) {
    EnterCriticalSection( &this->cs );
    
    int n;
    if (!max) {
        n = this->time_table[ button ];
    } else {    
        n = ( this->time_table[ button ] <? max );
        this->time_table[ button ] -= n;
    };
    
    LeaveCriticalSection( &this->cs );
    return n;
};

__fastcall unsigned int Aegis::C::Mouse::GetUpTime( unsigned int button,
                                                    unsigned int max ) {
    EnterCriticalSection( &this->cs );
    
    int n;
    if (!max) {
        n = this->uptime_table[ button ];
    } else {    
        n = ( this->uptime_table[ button ] <? max );
        this->uptime_table[ button ] -= n;
    };
    
    LeaveCriticalSection( &this->cs );
    return n;
};


__fastcall int Aegis::C::Mouse::UpDownHandler( unsigned int button, bool down ) {
    return 0;
};
__fastcall int Aegis::C::Mouse::OnOffHandler( unsigned int button, bool on ) { 
    return 0;
};
__fastcall int Aegis::C::Mouse::MovementHandler() { 
    return 0; 
};

