/*
  Name: aC_Mouse.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - C (Control) Subsystem - Mouse handler
*/

#ifndef _AC_MOUSE_HPP
#define _AC_MOUSE_HPP

#include <windows.h>

namespace Aegis {
    namespace C {
        
        class Mouse {
            private:
                CRITICAL_SECTION cs;
                HANDLE hThread; // time counter thread
                
                int x;
                int y;
                
                unsigned int imm_down_table[5];
                unsigned int imm_on_table[5];
                unsigned int inc_table[5];
                unsigned int time_table[5];
                unsigned int uptime_table[5]; // may overflow after 49,7 days :-)
                
                bool quit;
                bool perf_timer; 
                __int64 perf_timer_freq;
                
                static DWORD WINAPI ThreadCounter(LPVOID);
                
            protected:
                __fastcall virtual int MovementHandler();
                __fastcall virtual int UpDownHandler( unsigned int button,bool dn );
                __fastcall virtual int OnOffHandler( unsigned int button,bool on );
                
            public:
                __fastcall Mouse(bool accurate = true);
                __fastcall ~Mouse();

                bool ProcessMessage( HWND hwnd, UINT msg,
                                     WPARAM wParam, LPARAM lParam );
                __fastcall bool Center( HWND hwnd = NULL ); 
                
                // button can be
                //  0 - left
                //  1 - right
                //  2 - middle
                //  3 - x1
                //  4 - x2
                __fastcall bool IsDown( unsigned int button );
                __fastcall bool IsOn  ( unsigned int button );
                __fastcall void TurnOn( unsigned int button );
                __fastcall void TurnOff( unsigned int button );
                
                
                __fastcall int GetX( HWND hwnd = NULL ); // relative HWND
                __fastcall int GetY( HWND hwnd = NULL );
                // is GetOffsetN right name???
                __fastcall float GetOffsetX( HWND hwnd = NULL ); // -1.0 to 1.0
                __fastcall float GetOffsetY( HWND hwnd = NULL ); // -1.0 to 1.0
                
                __fastcall unsigned int GetHits( unsigned int button,
                                                 unsigned int max = 0xFFFFFFFF );
                __fastcall unsigned int GetTime( unsigned int button,
                                                 unsigned int max = 0xFFFFFFFF );
                __fastcall unsigned int GetUpTime( unsigned int button,
                                                   unsigned int max = 0xFFFFFFFF );
                
        };
    };
};

#endif

