/*
  Name: aF_Bitmap.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - F (File Formats) - Bitmap (aBitmap) base class  
*/

#ifndef _AF_BITMAP_HPP
#define _AF_BITMAP_HPP

#include <windows.h>
#include "../M/M2/aM2_RGBA.hpp"

namespace Aegis {
    namespace F {
        
        class Bitmap {
            private:
                char  magic[8];
                DWORD sizex;
                DWORD sizey;
                
                DWORD compression; // compression
                Aegis::M2::RGBA border; // border color, 0 - disabled
                DWORD p3; // reserved
                DWORD p4; // reserved

            protected:
                Aegis::M2::RGBA *rgba;
                
            public:
                __fastcall Bitmap();
                __fastcall Bitmap( long x, long y );
                __fastcall Bitmap( const Aegis::F::Bitmap& );
                __fastcall ~Bitmap();
                
                inline int Width() const { return sizex; };
                inline int Height() const { return sizey; };
                inline int GetFormat() const { return 0x8058; /*GL_RGBA8*/ };

                inline const Aegis::M2::RGBA* GetPointer() const { return rgba; };
                inline Aegis::M2::RGBA* Pointer() { return rgba; };

                
                __fastcall int Load( char *name ); 
                // Returns 0 if successfull
                // Error codes:
                // -1 - Invalid file format
                // -2 - Header read error (truncated file?)
                // -3 - Not enough memory
                // -5 - Data read error (truncated file?)
                // -6 - Damaged data
                // -7 - Damaged data
                // -8 - Unknown error in decompression
                // -9 - Unknown compression type
                // All positive return values are from GetLastError();
                
                __fastcall int Save( char *name, int cmp );
                // Returns 0 if successfull
                // Error codes:
                // -1 - Invalid compression specified
                // -3 - Not enough memory
                // -5 - Data write error (not enough space?)
                // -6 - Unexpected not enough memory (shouldn't happen)
                // -8 - Unknown error in compression
                // -9 - Unknown compression type
                // All positive return values are from GetLastError();
                // CMP can be one of these values
                //  0 - Use no compression
                //  1 - Compress directly by ZLIB
                //  2 - Optimize data and compress with ZLIB - unimplemented yet
        
                __fastcall int LoadBMP( HDC hDC, char *name );
                __fastcall int LoadHBITMAP( HDC hDC, HBITMAP hBitmap );
                __fastcall int LoadResource( HDC hDC, LPCTSTR rsrc );
                
                __fastcall int Draw( HDC hDC, int x, int y, float alpha );
                __fastcall int Blit( HDC hDC, int x, int y);
                
                __fastcall int SetPixel( unsigned int x, unsigned int y,
                                         int r, int g, int b, int a );
                __fastcall int SetPixel( unsigned int x, unsigned int y,
                                         DWORD color );

                __fastcall DWORD GetPixel( unsigned int x, unsigned int y ) const;
                __fastcall int GetPixelRed( unsigned int x, unsigned int y ) const;
                __fastcall int GetPixelGreen( unsigned int x, unsigned int y ) const;
                __fastcall int GetPixelBlue( unsigned int x, unsigned int y ) const;
                __fastcall int GetPixelAlpha( unsigned int x, unsigned int y ) const;
                __fastcall DWORD GetPixel( float x, float y, float size ) const;
                
        };
        
    };
};


#endif

