/*
  Name: aF_LinMap.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - F (File Formats) - LinMap (aLinMap) base class  
*/

#ifndef _AF_LINMAP_HPP
#define _AF_LINMAP_HPP

#include <windows.h>
#include "../M/M2/aM2_RGBA.hpp"

namespace Aegis {
    namespace F {
        
        class LinMap {
            private:
                char  magic[8];
                DWORD size;
                DWORD p0; // reserved
                
                DWORD compression; // compression
                Aegis::M2::RGBA border; // border color, 0 - disabled
                DWORD p3; // reserved
                DWORD p4; // reserved

            protected:
                Aegis::M2::RGBA *rgba;
                
            public:
                LinMap();
                LinMap( long s );
                ~LinMap();
                
                inline int Width() const { return size; };
                inline int GetFormat() const { return 0x8058; /*GL_RGBA8*/ };

                inline const void* GetPointer() const { return rgba; };
                inline void* Pointer() { return rgba; };

                
                __fastcall int Load( char *name ); 
                // Returns 0 if successfull
                // Error codes:
                // -1 - Invalid file format
                // -2 - Header read error (truncated file?)
                // -3 - Not enough memory
                // -5 - Data read error (truncated file?)
                // -6 - Damaged data
                // -7 - Damaged data
                // -8 - Unknown error in decompression
                // -9 - Unknown compression type
                // All positive return values are from GetLastError();
                
                __fastcall int Save( char *name, int cmp );
                // Returns 0 if successfull
                // Error codes:
                // -1 - Invalid compression specified
                // -3 - Not enough memory
                // -5 - Data write error (not enough space?)
                // -6 - Unexpected not enough memory (shouldn't happen)
                // -8 - Unknown error in compression
                // -9 - Unknown compression type
                // All positive return values are from GetLastError();
                // CMP can be one of these values
                //  0 - Use no compression
                //  1 - Compress directly by ZLIB
        
                __fastcall int LoadResource( HDC hDC, LPCTSTR rsrc );
                
                __fastcall int Draw( HDC hDC, int x, int y, float alpha );
                __fastcall int Blit( HDC hDC, int x, int y);
                
                __fastcall int SetPixel( unsigned int offset,
                                         int r, int g, int b, int a );
                __fastcall int SetPixel( unsigned int offset, DWORD color );

                __fastcall DWORD GetPixel( unsigned int offset ) const;
                __fastcall int GetPixelRed( unsigned int offset ) const;
                __fastcall int GetPixelGreen( unsigned int offset ) const;
                __fastcall int GetPixelBlue( unsigned int offset ) const;
                __fastcall int GetPixelApha( unsigned int offset ) const;
                __fastcall DWORD GetPixel( float offset, float size ) const;
                
        };
        
    };
};


#endif

