/*
  Name: aF_TexMap.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - F (File Formats) - TexMap (aTexMap) base class  
*/

#ifndef _AF_TEXMAP_HPP
#define _AF_TEXMAP_HPP

#include <windows.h>
#include "aF_Bitmap.hpp"
#include "../M/M2/aM2_RGBA.hpp"

namespace Aegis {
    namespace F {
        
        class TexMap {
            private:
                char  magic[8];
                unsigned char size; // number of mipmaps

                unsigned char r0a; // reserved
                unsigned char r0b; // reserved
                unsigned char r0c; // reserved

                Aegis::M2::RGBA base; // base color (size-th mipmap)
                
                unsigned int r1; // reserved
                unsigned int r2; // reserved
                
                unsigned int data_length;
                unsigned int tags_length;
                
            protected:
                signed char *red;
                signed char *green;
                signed char *blue;
                signed char *alpha;
                
            public:    
                __fastcall TexMap();
                inline TexMap( const char *filename ) {
                    if (this->Load(filename))
                        this->size = 0;
                    return;
                };
                __fastcall TexMap( Aegis::F::TexMap& );
                __fastcall TexMap( Aegis::F::Bitmap& );
                __fastcall ~TexMap();


                __fastcall const signed char *
                           GetPointer( int c, int mipmap ) const;
                           
                inline const signed char * GetPointerRed( int mipmap ) const {
                    return GetPointer( 0, mipmap );
                };    
                inline const signed char * GetPointerGreen( int mipmap ) const {
                    return GetPointer( 1, mipmap );
                };    
                inline const signed char * GetPointerBlue( int mipmap ) const {
                    return GetPointer( 2, mipmap );
                };    
                inline const signed char * GetPointerAlpha( int mipmap ) const {
                    return GetPointer( 3, mipmap );
                };    

                inline int Width(int mipmap) const {
                    if (mipmap < 0 || mipmap > this->size) return 0;
                    return (1 << (this->size - mipmap));
                };
                inline int Height(int mipmap) const {
                    return this->Width(mipmap);
                };
                inline int GetFormat() const { return 0x8058; /*GL_RGBA8*/ };

                __fastcall Aegis::F::Bitmap* MakeBitmap() const;

                __fastcall int Load( const char *name ); 
                // Returns 0 if successfull
                // Error codes:
                // -1 - Invalid file format
                // -2 - Header read error (truncated file?)
                // -3 - Not enough memory
                // -5 - Data read error (truncated file?)
                // -6 - Damaged data
                // -7 - Damaged data
                // -8 - Unknown error in decompression
                // All positive return values are from GetLastError();
                
                __fastcall int Save( const char *name );
                // Returns 0 if successfull
                // Error codes:
                // -1 - Invalid compression specified
                // -3 - Not enough memory
                // -5 - Data write error (not enough space?)
                // -6 - Unexpected not enough memory (shouldn't happen)
                // -8 - Unknown error in compression
                // -9 - Unknown compression type
                // All positive return values are from GetLastError();
                // Uses frequency preprocessed compression
                
                __fastcall DWORD GetPixel(unsigned int x,unsigned int y) const;
                __fastcall int GetPixelRed(unsigned int x,unsigned int y) const;
                __fastcall int GetPixelGreen(unsigned int x,unsigned int y) const;
                __fastcall int GetPixelBlue(unsigned int x,unsigned int y) const;
                __fastcall int GetPixelApha(unsigned int x,unsigned int y) const;

                __fastcall int SetPixel( unsigned int x, unsigned int y,
                                         int mipmap, Aegis::M2::RGBA clr );

                
        };
    };
};

#endif

