/*
  Name: aF_aFile.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - F (File Formats) - aXXX files base class
*/

#ifndef _AF_AFILE_HPP
#define _AF_AFILE_HPP

#include <stdio.h>

namespace Aegis {
    namespace F {
        
        class aFile {
            private:
                char magic[8];
                
            protected:
                char header[24];
                void *data;
                
            protected:
                // used by this object, redefined by derived classes
                virtual const char *GetTypeString();
                
                // De: return false if unknown compression format
                virtual bool Decompress( const void *source, unsigned int len );
                // Co: return false if invalid cmp format set
                //     return new[]-allocated data through target ptpointer
                virtual bool Compress( const void *source, unsigned int len,
                                       void **target, int compression );

                // global internal use
                __fastcall int LoadData( const void *data, unsigned int len );

                // access to data
                void Read( unsigned int offset, unsigned int size,
                           void *target );
                void Write( unsigned int offset, unsigned int size,
                            const void *source );
                            
                // TODO: some inlines for common data types access here
                   
            public:
                __fastcall aFile();
                __fastcall ~aFile();
                
                __fastcall int Save( const char *filename, int compression = 0 );
                __fastcall int Load( const char *filename );
                
        };
        
    };
};

#endif

