/*
  Name: aG1_BlendControl.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G1 - Blend Control and Cache
*/

#include "aG1_BlendControl.hpp"

__fastcall Aegis::G1::BlendControlObject::BlendControlObject(
                        const Aegis::G1::Renderer & ren) : renderer(ren) {

    this->enabled = false;
    this->mode = GL_FUNC_ADD_EXT;
    this->src_mode = GL_SRC_ALPHA;
    this->dst_mode = GL_ONE_MINUS_SRC_ALPHA;
    
//    this->st.reserve(4096);
    
    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
    glDisable( GL_BLEND );
    
    return;
};    

__fastcall void Aegis::G1::BlendControlObject::Enable( bool en ) {
    if (this->enabled == en) return;

    this->enabled = en;
    if (en)
        glEnable( GL_BLEND );
    else
        glDisable( GL_BLEND );
    
    return;
};

__fastcall void Aegis::G1::BlendControlObject::SetMode( int m ) {
    if (this->mode == m) return;
    this->mode = m;
    
    if (this->renderer.Ext->Is( Aegis::G1::BlendMinMax ))
        if (m == GL_FUNC_ADD_EXT || m == GL_MIN_EXT || m == GL_MAX_EXT) {
            this->renderer.Ext->glBlendEquationEXT( m );
            return;
        };    

    if (this->renderer.Ext->Is( Aegis::G1::BlendSubstract ))
        if (m == GL_FUNC_SUBTRACT_EXT || m == GL_FUNC_REVERSE_SUBTRACT_EXT) {
            this->renderer.Ext->glBlendEquationEXT( m );
            return;
        };    
    
    return;
};

__fastcall void Aegis::G1::BlendControlObject::SetFunc( int source,
                                                        int destination ) {
    if (this->src_mode == source && this->dst_mode == destination) return;
    
    this->src_mode = source;
    this->dst_mode = destination;
    
    glBlendFunc( source, destination );
    
    return;
};

__fastcall void Aegis::G1::BlendControlObject::Blend() {
    this->SetMode( GL_FUNC_ADD_EXT );

    // TODO: FIXME: Is this bad or does MultiFace use it in wrong way???
    if (this->renderer.Ext->Is( Aegis::G1::BlendSquare ))
        this->SetFunc( GL_SRC_COLOR, GL_ONE_MINUS_SRC_COLOR );
    else
        this->SetFunc( GL_ONE, GL_ONE_MINUS_SRC_COLOR );
    
    return;
};


__fastcall void Aegis::G1::BlendControlObject::Push() {
    this->st.push(this->dst_mode);
    this->st.push(this->src_mode);
    this->st.push(this->mode);
    this->st.push(this->enabled);
    
    return;
};    

__fastcall void Aegis::G1::BlendControlObject::Pop() {
    if (this->st.empty()) return;
    int m, s, d, e;
    
    e = this->st.top(); this->st.pop();
    m = this->st.top(); this->st.pop();
    s = this->st.top(); this->st.pop();
    d = this->st.top(); this->st.pop();
    
    this->SetMode(m);
    this->SetFunc(s,d);
    this->Enable(e);
    
    return;
};

