/*
  Name: aG1_BlendControl.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G1 - Blend Control and Cache
*/

#ifndef _AG1_BLENDCONTROL_HPP
#define _AG1_BLENDCONTROL_HPP

#include <windows.h>
#include <gl/gl.h>
#include <stack>

namespace Aegis {
    namespace G1 {
        // definition
        class BlendControlObject;
    };
};

#include "aG1_Renderer.hpp"

namespace Aegis {
    namespace G1 {
        
        class BlendControlObject {
            private:
                const Aegis::G1::Renderer & renderer; // owner
                
                int enabled;
                int mode; // GL_FUNC_ADD_EXT, GL_FUNC_SUBSTRACT_EXT,
                          // GL_FUNC_REVERSE_SUBTRACT_EXT, GL_MIN_EXT, GL_MAX_EXT
                int src_mode;
                int dst_mode;
                
                std::stack<int> st;
                
            public:
                __fastcall BlendControlObject( const Aegis::G1::Renderer & );
                
                __fastcall void Enable( bool en = true );
                inline void Disable() { this->Enable(false); };
                
                __fastcall void SetMode( int mode );
                __fastcall void SetFunc( int source, int destination );
                
                inline void Default() {
                    this->SetMode( GL_FUNC_ADD_EXT );
                    this->SetFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
                    
                    return;
                };
                inline void Modulate() {
                    this->SetMode( GL_FUNC_ADD_EXT );
                    this->SetFunc( GL_DST_COLOR, GL_ZERO ); // is this correct
//                    this->SetFunc( GL_ZERO, GL_SRC_COLOR );

                    return;
                };
                inline void Add() {
                    this->SetMode( GL_FUNC_ADD_EXT );
                    this->SetFunc( GL_ONE, GL_ONE );
                    
                    return;
                };
                inline void Replace() {
                    this->SetMode( GL_FUNC_ADD_EXT );
                    this->SetFunc( GL_ONE, GL_ZERO );
                    
                    return;
                };
                inline void Decal() {
//                    this->Replace();
                    
                    // TODO: FIXME: What exactly is this supposed to do?
                    this->SetMode( GL_FUNC_ADD_EXT );
                    this->SetFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
                    
                    return;
                };
                inline void Void() {
                    this->SetMode( GL_FUNC_ADD_EXT );
                    this->SetFunc( GL_ZERO, GL_ONE );
                    
                    return;
                };
                __fastcall void Blend(); // this cannot be inline
                
                
                __fastcall void Push();
                __fastcall void Pop();
                
                
        };
        
    };
};

#endif
            
