/*
  Name: aG1_Extensions.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G1 - OpenGL Extensions
*/

#ifndef _AG1_EXTENSIONS_HPP
#define _AG1_EXTENSIONS_HPP

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>

#include "aG1_GLExt.h"
#include <vector>

namespace Aegis {
    namespace G1 {
        enum {
            Multitexture = 1,                                                             
            PointParameters = 11,
            PointSprite = 12,
            TextureCubeMap = 21,
            TextureAnisotropicFilter = 22,
            TextureGenerateMipmap = 23,
            TextureCompression = 24,
            
            TextureEnvAdd = 31,
            TextureEnvCombine = 32,
            TextureEnvDot3 = 33,

            VertexProgram = 41,
            VertexBufferObject = 42,
            
            BlendColor = 51,
            BlendMinMax = 52,
            BlendSubstract = 53,
            BlendSquare = 54,
            CompiledVertexArray = 61,
            DrawRangeElements = 71,
            FogCoord = 81,
            PackedPixels = 91,
            BGRA = 92,
            ARGB = 93,
            SecondaryColor = 101,
            SeparateSpecularColor = 102,
            
            HighestExtensionIdentifier = 128 // should be aligned at 8
        };    
        
        // declaration
        class Extension {
            private:
                char *szGlExtensions;
                std::vector<int> bExtFlag; // int for measurable faster access
                std::vector<int> bExtMask; // int for measurable faster access
                   
                PROC wglGetProcAddress(LPCSTR sz) {
                    PROC p = ::wglGetProcAddress(sz);
//                    printf("%s\t\t = %08x\n",sz,p);
                    return p;
                };    
            
            public:
                Extension();
                ~Extension();
                
                __fastcall bool Is(const int id) const;
                __fastcall bool Is(const char *name) const;
                __fastcall bool Mask(const int id, bool disable = true);

                // Multitexture extension
                PFNGLACTIVETEXTUREARBPROC        glActiveTextureARB;
                PFNGLCLIENTACTIVETEXTUREARBPROC  glClientActiveTextureARB;
                PFNGLMULTITEXCOORD1DARBPROC      glMultiTexCoord1dARB ;      
                PFNGLMULTITEXCOORD1DVARBPROC     glMultiTexCoord1dvARB;
                PFNGLMULTITEXCOORD1FARBPROC      glMultiTexCoord1fARB ;         
                PFNGLMULTITEXCOORD1FVARBPROC     glMultiTexCoord1fvARB;
                PFNGLMULTITEXCOORD1IARBPROC      glMultiTexCoord1iARB ;
                PFNGLMULTITEXCOORD1IVARBPROC     glMultiTexCoord1ivARB;
                PFNGLMULTITEXCOORD1SARBPROC      glMultiTexCoord1sARB ;
                PFNGLMULTITEXCOORD1SVARBPROC     glMultiTexCoord1svARB;
                PFNGLMULTITEXCOORD2DARBPROC      glMultiTexCoord2dARB ;
                PFNGLMULTITEXCOORD2DVARBPROC     glMultiTexCoord2dvARB;
                PFNGLMULTITEXCOORD2FARBPROC      glMultiTexCoord2fARB ;
                PFNGLMULTITEXCOORD2FVARBPROC     glMultiTexCoord2fvARB;
                PFNGLMULTITEXCOORD2IARBPROC      glMultiTexCoord2iARB ;
                PFNGLMULTITEXCOORD2IVARBPROC     glMultiTexCoord2ivARB;
                PFNGLMULTITEXCOORD2SARBPROC      glMultiTexCoord2sARB ;
                PFNGLMULTITEXCOORD2SVARBPROC     glMultiTexCoord2svARB;
                PFNGLMULTITEXCOORD3DARBPROC      glMultiTexCoord3dARB ;
                PFNGLMULTITEXCOORD3DVARBPROC     glMultiTexCoord3dvARB;
                PFNGLMULTITEXCOORD3FARBPROC      glMultiTexCoord3fARB ;
                PFNGLMULTITEXCOORD3FVARBPROC     glMultiTexCoord3fvARB;
                PFNGLMULTITEXCOORD3IARBPROC      glMultiTexCoord3iARB ;
                PFNGLMULTITEXCOORD3IVARBPROC     glMultiTexCoord3ivARB;
                PFNGLMULTITEXCOORD3SARBPROC      glMultiTexCoord3sARB ;
                PFNGLMULTITEXCOORD3SVARBPROC     glMultiTexCoord3svARB;
                PFNGLMULTITEXCOORD4DARBPROC      glMultiTexCoord4dARB ;
                PFNGLMULTITEXCOORD4DVARBPROC     glMultiTexCoord4dvARB;
                PFNGLMULTITEXCOORD4FARBPROC      glMultiTexCoord4fARB ;
                PFNGLMULTITEXCOORD4FVARBPROC     glMultiTexCoord4fvARB;
                PFNGLMULTITEXCOORD4IARBPROC      glMultiTexCoord4iARB ;
                PFNGLMULTITEXCOORD4IVARBPROC     glMultiTexCoord4ivARB;
                PFNGLMULTITEXCOORD4SARBPROC      glMultiTexCoord4sARB ;
                PFNGLMULTITEXCOORD4SVARBPROC     glMultiTexCoord4svARB;
    
                PFNGLBLENDCOLOREXTPROC           glBlendColorEXT;
    
                PFNGLBLENDEQUATIONEXTPROC        glBlendEquationEXT;
    
                PFNGLLOCKARRAYSEXTPROC           glLockArraysEXT;
                PFNGLUNLOCKARRAYSEXTPROC         glUnlockArraysEXT;
    
                PFNGLDRAWRANGEELEMENTSEXTPROC    glDrawRangeElementsEXT;
    
                PFNGLFOGCOORDFEXTPROC            glFogCoordfEXT;
                PFNGLFOGCOORDFVEXTPROC           glFogCoordfvEXT;
                PFNGLFOGCOORDDEXTPROC            glFogCoorddEXT;
                PFNGLFOGCOORDDVEXTPROC           glFogCoorddvEXT;
                PFNGLFOGCOORDPOINTEREXTPROC      glFogCoordPointerEXT;
    
                PFNGLSECONDARYCOLOR3BEXTPROC      glSecondaryColor3bEXT;
                PFNGLSECONDARYCOLOR3BVEXTPROC     glSecondaryColor3bvEXT;
                PFNGLSECONDARYCOLOR3DEXTPROC      glSecondaryColor3dEXT ;
                PFNGLSECONDARYCOLOR3DVEXTPROC     glSecondaryColor3dvEXT;
                PFNGLSECONDARYCOLOR3FEXTPROC      glSecondaryColor3fEXT ;
                PFNGLSECONDARYCOLOR3FVEXTPROC     glSecondaryColor3fvEXT;
                PFNGLSECONDARYCOLOR3IEXTPROC      glSecondaryColor3iEXT ;
                PFNGLSECONDARYCOLOR3IVEXTPROC     glSecondaryColor3ivEXT;
                PFNGLSECONDARYCOLOR3SEXTPROC      glSecondaryColor3sEXT ;
                PFNGLSECONDARYCOLOR3SVEXTPROC     glSecondaryColor3svEXT;
                PFNGLSECONDARYCOLOR3UBEXTPROC     glSecondaryColor3ubEXT;
                PFNGLSECONDARYCOLOR3UBVEXTPROC    glSecondaryColor3ubvEXT;
                PFNGLSECONDARYCOLOR3UIEXTPROC     glSecondaryColor3uiEXT ;
                PFNGLSECONDARYCOLOR3UIVEXTPROC    glSecondaryColor3uivEXT;
                PFNGLSECONDARYCOLOR3USEXTPROC     glSecondaryColor3usEXT ;
                PFNGLSECONDARYCOLOR3USVEXTPROC    glSecondaryColor3usvEXT;
                PFNGLSECONDARYCOLORPOINTEREXTPROC glSecondaryColorPointerEXT;
                
                PFNGLCOMPRESSEDTEXIMAGE3DARBPROC     glCompressedTexImage3DARB;
                PFNGLCOMPRESSEDTEXIMAGE2DARBPROC     glCompressedTexImage2DARB;
                PFNGLCOMPRESSEDTEXIMAGE1DARBPROC     glCompressedTexImage1DARB;
                PFNGLCOMPRESSEDTEXSUBIMAGE3DARBPROC  glCompressedTexSubImage3DARB;
                PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC  glCompressedTexSubImage2DARB;
                PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC  glCompressedTexSubImage1DARB;
                PFNGLGETCOMPRESSEDTEXIMAGEARBPROC    glGetCompressedTexImageARB;
                
                PFNGLVERTEXATTRIB1SARBPROC              glVertexAttrib1sARB;
                PFNGLVERTEXATTRIB1FARBPROC              glVertexAttrib1fARB;
                PFNGLVERTEXATTRIB1DARBPROC              glVertexAttrib1dARB;
                PFNGLVERTEXATTRIB2SARBPROC              glVertexAttrib2sARB;
                PFNGLVERTEXATTRIB2FARBPROC              glVertexAttrib2fARB;
                PFNGLVERTEXATTRIB2DARBPROC              glVertexAttrib2dARB;
                PFNGLVERTEXATTRIB3SARBPROC              glVertexAttrib3sARB;
                PFNGLVERTEXATTRIB3FARBPROC              glVertexAttrib3fARB;
                PFNGLVERTEXATTRIB3DARBPROC              glVertexAttrib3dARB;
                PFNGLVERTEXATTRIB4SARBPROC              glVertexAttrib4sARB;
                PFNGLVERTEXATTRIB4FARBPROC              glVertexAttrib4fARB;
                PFNGLVERTEXATTRIB4DARBPROC              glVertexAttrib4dARB;
                PFNGLVERTEXATTRIB4NUBARBPROC            glVertexAttrib4NubARB;
                PFNGLVERTEXATTRIB1SVARBPROC             glVertexAttrib1svARB;
                PFNGLVERTEXATTRIB1FVARBPROC             glVertexAttrib1fvARB;
                PFNGLVERTEXATTRIB1DVARBPROC             glVertexAttrib1dvARB;
                PFNGLVERTEXATTRIB2SVARBPROC             glVertexAttrib2svARB;
                PFNGLVERTEXATTRIB2FVARBPROC             glVertexAttrib2fvARB;
                PFNGLVERTEXATTRIB2DVARBPROC             glVertexAttrib2dvARB;
                PFNGLVERTEXATTRIB3SVARBPROC             glVertexAttrib3svARB;
                PFNGLVERTEXATTRIB3FVARBPROC             glVertexAttrib3fvARB;
                PFNGLVERTEXATTRIB3DVARBPROC             glVertexAttrib3dvARB;
                PFNGLVERTEXATTRIB4BVARBPROC             glVertexAttrib4bvARB;
                PFNGLVERTEXATTRIB4SVARBPROC             glVertexAttrib4svARB;
                PFNGLVERTEXATTRIB4IVARBPROC             glVertexAttrib4ivARB;
                PFNGLVERTEXATTRIB4UBVARBPROC            glVertexAttrib4ubvARB;
                PFNGLVERTEXATTRIB4USVARBPROC            glVertexAttrib4usvARB;
                PFNGLVERTEXATTRIB4UIVARBPROC            glVertexAttrib4uivARB;
                PFNGLVERTEXATTRIB4FVARBPROC             glVertexAttrib4fvARB;
                PFNGLVERTEXATTRIB4DVARBPROC             glVertexAttrib4dvARB;
                PFNGLVERTEXATTRIB4NBVARBPROC            glVertexAttrib4NbvARB;
                PFNGLVERTEXATTRIB4NSVARBPROC            glVertexAttrib4NsvARB;
                PFNGLVERTEXATTRIB4NIVARBPROC            glVertexAttrib4NivARB;
                PFNGLVERTEXATTRIB4NUBVARBPROC           glVertexAttrib4NubvARB;
                PFNGLVERTEXATTRIB4NUSVARBPROC           glVertexAttrib4NusvARB;
                PFNGLVERTEXATTRIB4NUIVARBPROC           glVertexAttrib4NuivARB;
                PFNGLVERTEXATTRIBPOINTERARBPROC         glVertexAttribPointerARB;
                PFNGLENABLEVERTEXATTRIBARRAYARBPROC     glEnableVertexAttribArrayARB;
                PFNGLDISABLEVERTEXATTRIBARRAYARBPROC    glDisableVertexAttribArrayARB;
                PFNGLPROGRAMSTRINGARBPROC               glProgramStringARB;
                PFNGLBINDPROGRAMARBPROC                 glBindProgramARB;
                PFNGLDELETEPROGRAMSARBPROC              glDeleteProgramsARB;
                PFNGLGENPROGRAMSARBPROC                 glGenProgramsARB;
                PFNGLPROGRAMENVPARAMETER4DARBPROC       glProgramEnvParameter4dARB;
                PFNGLPROGRAMENVPARAMETER4DVARBPROC      glProgramEnvParameter4dvARB;
                PFNGLPROGRAMENVPARAMETER4FARBPROC       glProgramEnvParameter4fARB;
                PFNGLPROGRAMENVPARAMETER4FVARBPROC      glProgramEnvParameter4fvARB;
                PFNGLPROGRAMLOCALPARAMETER4DARBPROC     glProgramLocalParameter4dARB;
                PFNGLPROGRAMLOCALPARAMETER4DVARBPROC    glProgramLocalParameter4dvARB;
                PFNGLPROGRAMLOCALPARAMETER4FARBPROC     glProgramLocalParameter4fARB;
                PFNGLPROGRAMLOCALPARAMETER4FVARBPROC    glProgramLocalParameter4fvARB;
                PFNGLGETPROGRAMENVPARAMETERDVARBPROC    glGetProgramEnvParameterdvARB;
                PFNGLGETPROGRAMENVPARAMETERFVARBPROC    glGetProgramEnvParameterfvARB;
                PFNGLGETPROGRAMLOCALPARAMETERDVARBPROC  glGetProgramLocalParameterdvARB;
                PFNGLGETPROGRAMLOCALPARAMETERFVARBPROC  glGetProgramLocalParameterfvARB;
                PFNGLGETPROGRAMIVARBPROC                glGetProgramivARB;
                PFNGLGETPROGRAMSTRINGARBPROC            glGetProgramStringARB;
                PFNGLGETVERTEXATTRIBDVARBPROC           glGetVertexAttribdvARB;
                PFNGLGETVERTEXATTRIBFVARBPROC           glGetVertexAttribfvARB;
                PFNGLGETVERTEXATTRIBIVARBPROC           glGetVertexAttribivARB;
                PFNGLGETVERTEXATTRIBPOINTERVARBPROC     glGetVertexAttribPointervARB;
                PFNGLISPROGRAMARBPROC                   glIsProgramARB;                       
                
                PFNGLPOINTPARAMETERFARBPROC             glPointParameterfARB;
                PFNGLPOINTPARAMETERFVARBPROC            glPointParameterfvARB;
                
                PFNGLBINDBUFFERARBPROC                  glBindBufferARB;
                PFNGLDELETEBUFFERSARBPROC               glDeleteBuffersARB;
                PFNGLGENBUFFERSARBPROC                  glGenBuffersARB;
                PFNGLISBUFFERARBPROC                    glIsBufferARB;
                PFNGLBUFFERDATAARBPROC                  glBufferDataARB;
                PFNGLBUFFERSUBDATAARBPROC               glBufferSubDataARB;
                PFNGLGETBUFFERSUBDATAARBPROC            glGetBufferSubDataARB;
                PFNGLMAPBUFFERARBPROC                   glMapBufferARB;
                PFNGLUNMAPBUFFERARBPROC                 glUnmapBufferARB;
                PFNGLGETBUFFERPARAMETERIVARBPROC        glGetBufferParameterivARB;
                PFNGLGETBUFFERPOINTERVARBPROC           glGetBufferPointervARB;

        };  
            
            
    };      
};          
            
#endif      
            
            
            
            
            
            
            
