/*
  Name: aG1_FogControl.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G1 - Fog Control and Cache
*/

#ifndef _AG1_FOG_CONTROL_HPP
#define _AG1_FOG_CONTROL_HPP

#include <windows.h>
#include <gl/gl.h>
#include <math.h>
#include <stack>

namespace Aegis {
    namespace G1 {
        // definition
        class FogControlObject;
    };
};

#include "../../Attribs.hpp"
#include "aG1_Renderer.hpp"
#include "../../M/M2/aM2_Color.hpp"

namespace Aegis {
    namespace G1 {
        
        class FogControlObject {
            private:
                const Aegis::G1::Renderer & renderer; // owner
                
                bool constant_fog;
                bool parametric_fog;
                
                bool parametric_capability;

                // some caching
                bool emulate_parametric;
                bool glfog_param_set;
                bool glfog_enabled;

                // direct glFog caching
                int   glfog_mode; // GL_LINEAR, GL_EXP or GL_EXP2
                float glfog_density;
                float glfog_start;
                float glfog_end;
                Aegis::M2::Color glfog_color;
                
                std::stack<int> mode_stack;
                std::stack<float> fparam_stack;
                std::stack<Aegis::M2::Color> color_stack;

            private:
                __fastcall void UpdateEmulateParametricFlag();
                __fastcall void UpdateFogMode();
                
            public:
                __fastcall FogControlObject( const Aegis::G1::Renderer &,
                                             bool want_emulated_parametric = false
                                            );
                
                inline bool EmulateParametric() const {
                    return this->emulate_parametric;
                };
                
                inline void Disable() { return this->Enable(false); };
                inline void ParametricDisable() { 
                    return this->ParametricEnable(false);
                    return;
                };
                __fastcall void Enable( bool en = true );
                __fastcall void ParametricEnable( bool en = true );
                
                __fastcall void SetMode( int mode ); // GL_LINEAR, GL_EXP, GLEXP2
                __fastcall void SetDensity( float density );
                __fastcall void SetLinearStart( float start );
                __fastcall void SetLinearEnd( float end );
                __fastcall void SetColor( Aegis::M2::Color clr );
                
                inline Aegis::M2::Color GetColor() const {
                    return this->glfog_color;
                };
                inline float ComputeFakeParametricAlpha( float fc ) __pure {
                    float f = 0.0;
                    if (this->glfog_mode == GL_LINEAR)
                        f = (this->glfog_end - fc)
                          / (this->glfog_end - this->glfog_start);
                    if (this->glfog_mode == GL_EXP)
                        f = expf( -this->glfog_density * fc );
                    if (this->glfog_mode == GL_EXP2)
                        f = expf( (-this->glfog_density * fc)
                                * (-this->glfog_density * fc) );

                    f = 1.0f <? (0.0f >? f); // clamp to 0.0 ... 1.0
                    return 1.0f - f;
                };    
                
                __fastcall void Push();
                __fastcall void Pop();
        };
        
    };
};

#endif
        
