/*
  Name: aG1_FrameControl.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G1 - FrameBuffer Control
*/

#include "aG1_FrameControl.hpp"

__fastcall Aegis::G1::FrameControlObject::
           FrameControlObject( Aegis::G1::Renderer & ren ) : renderer(ren) {
    this->maskDepth = true;
    this->maskColor = true;
    
//    this->mode_stack.reserve(5120);
//    this->float_stack.reserve(4096);
//    this->mask_stack.reserve(2048);
    
    this->Depth();
    this->Viewport(1,1);
    
    return;
};    

__fastcall void Aegis::G1::FrameControlObject::Depth(float _near, float _far) {
    this->z_near = _near;
    this->z_far = _far;
    
    return;
};

__fastcall void Aegis::G1::FrameControlObject::Viewport(int w, int h) {
    this->vpx = w;
    this->vpy = h;
    this->aspect = (float)w / (float)h;
    
    return;
};

__fastcall void Aegis::G1::FrameControlObject::Set2D() {
    this->Set2D( this->vpx, this->vpy );
    return;
};

__fastcall void Aegis::G1::FrameControlObject::Set2D(int _vx, int _vy) {
    this->perspective = false;
    this->vx = _vx;
    this->vy = _vy;
    
/*    printf("2D: %d %d, VIRT: %d %d, Z: -1.0 ... 1.0\n",
            this->vpx, this->vpy, this->vx, this->vy );//*/

    this->renderer.Orthogonal( 0, 0, this->vpx-1, this->vpy-1,
                               0, this->vx-1, this->vy-1, 0,
                               -1.0, 1.0 );
    // TODO: think if wouldn't be better to have there z_fear and z_far

    return;
};

__fastcall void Aegis::G1::FrameControlObject::Set3D(float _angle) {
    this->perspective = true;
    this->angle = _angle;
    
/*    printf("3D: %d %d, ANGLE: %3.2f, ASPECT: %3.2f, Z: %3.2f ... %3.2f\n",
            this->vpx, this->vpy,this->angle, this->aspect,
            this->z_near, this->z_far);//*/
    
    this->renderer.Frustum( 0, 0, this->vpx-1, this->vpy-1,
                            this->angle, this->aspect,
                            this->z_near, this->z_far );//*/
//    glLoadIdentity();
//    gluPerspective( this->angle, this->aspect, this->z_near, this->z_far );
    
    return;
};


__fastcall void Aegis::G1::FrameControlObject::DepthMask(bool b) {
    if (this->maskDepth == b) return;
    this->maskDepth = b;
    
    glDepthMask(b);
    
    return;
};
    
__fastcall void Aegis::G1::FrameControlObject::ColorMask(bool b) {
    if (this->maskColor == b) return;
    this->maskColor = b;
    
    glColorMask(b,b,b,b);
    
    return;
};


__fastcall void Aegis::G1::FrameControlObject::Push() {
    mask_stack.push( this->maskColor );
    mask_stack.push( this->maskDepth );
    
    return;
};

__fastcall void Aegis::G1::FrameControlObject::Pop() {
    int c, d;
    d = mask_stack.top(); mask_stack.pop();
    c = mask_stack.top(); mask_stack.pop();
    
    this->ColorMask(c);
    this->DepthMask(c);
    
    return;
};



