/*
  Name: aG1_FrameControl.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G1 - FrameBuffer Control
*/

#ifndef _AG1_FRAME_CONTROL_HPP
#define _AG1_FRAME_CONTROL_HPP

#include <windows.h>
#include <gl/gl.h>
#include <math.h>
#include <stack>

namespace Aegis {
    namespace G1 {
        // definition
        class FrameControlObject;
    };
};

#include "aG1_Renderer.hpp"

namespace Aegis {
    namespace G1 {

        class FrameControlObject {
            private:
                Aegis::G1::Renderer & renderer; // owner
                
                int perspective;
                int vpx; // real
                int vpy;
                int vx; // virtual
                int vy;
                
                float aspect;
                float angle;
                
                float z_near;
                float z_far;
                
                int maskDepth;
                int maskColor;
                
                std::stack<int> mode_stack;
                std::stack<float> float_stack;

                std::stack<int> mask_stack;
                
            public:
                __fastcall FrameControlObject( Aegis::G1::Renderer & );
                
                __fastcall void Depth(float _near = 1.0f, float _far = 100.f);
                __fastcall void Viewport(int w, int h);
                __fastcall void Set2D();
                __fastcall void Set2D(int vx, int vy);
                __fastcall void Set3D(float angle = 45.0f);
                
                inline int GetViewportX() { return vpx; };
                inline int GetViewportY() { return vpy; };
                inline int GetScreenX() { return vx; };
                inline int GetScreenY() { return vy; };
                
                __fastcall void DepthMask(bool b);
                __fastcall void ColorMask(bool b);
                
                __fastcall void Push();
                __fastcall void Pop();
                
        };
        
    };
};

#endif

