/*
  Name: aG1_FrustumCull.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G1 - Frustum Culling
*/

#include "aG1_FrustumCull.hpp"
#include <math.h>

__fastcall Aegis::G1::Frustum::Frustum() {
    this->ReCalculateFrustum();
    return;
};

__fastcall Aegis::G1::Frustum::~Frustum() {
    return;
};

__fastcall void Aegis::G1::Frustum::NormalizePlane(int side) {
    float magnitude = sqrtf( m_Frustum[side][0] * m_Frustum[side][0] + 
					 	     m_Frustum[side][1] * m_Frustum[side][1] + 
							 m_Frustum[side][2] * m_Frustum[side][2] );
	m_Frustum[side][0] /= magnitude;
	m_Frustum[side][1] /= magnitude;
	m_Frustum[side][2] /= magnitude;
	m_Frustum[side][3] /= magnitude; 
    
    return;
};


__fastcall void Aegis::G1::Frustum::ReCalculateFrustum() {
	float   proj[16];
	float   modl[16];
	float   clip[16];

	glGetFloatv( GL_PROJECTION_MATRIX, proj );
	glGetFloatv( GL_MODELVIEW_MATRIX, modl );

	clip[ 0] = modl[ 0] * proj[ 0] + modl[ 1] * proj[ 4] + modl[ 2] * proj[ 8] + modl[ 3] * proj[12];
	clip[ 1] = modl[ 0] * proj[ 1] + modl[ 1] * proj[ 5] + modl[ 2] * proj[ 9] + modl[ 3] * proj[13];
	clip[ 2] = modl[ 0] * proj[ 2] + modl[ 1] * proj[ 6] + modl[ 2] * proj[10] + modl[ 3] * proj[14];
	clip[ 3] = modl[ 0] * proj[ 3] + modl[ 1] * proj[ 7] + modl[ 2] * proj[11] + modl[ 3] * proj[15];

	clip[ 4] = modl[ 4] * proj[ 0] + modl[ 5] * proj[ 4] + modl[ 6] * proj[ 8] + modl[ 7] * proj[12];
	clip[ 5] = modl[ 4] * proj[ 1] + modl[ 5] * proj[ 5] + modl[ 6] * proj[ 9] + modl[ 7] * proj[13];
	clip[ 6] = modl[ 4] * proj[ 2] + modl[ 5] * proj[ 6] + modl[ 6] * proj[10] + modl[ 7] * proj[14];
	clip[ 7] = modl[ 4] * proj[ 3] + modl[ 5] * proj[ 7] + modl[ 6] * proj[11] + modl[ 7] * proj[15];

	clip[ 8] = modl[ 8] * proj[ 0] + modl[ 9] * proj[ 4] + modl[10] * proj[ 8] + modl[11] * proj[12];
	clip[ 9] = modl[ 8] * proj[ 1] + modl[ 9] * proj[ 5] + modl[10] * proj[ 9] + modl[11] * proj[13];
	clip[10] = modl[ 8] * proj[ 2] + modl[ 9] * proj[ 6] + modl[10] * proj[10] + modl[11] * proj[14];
	clip[11] = modl[ 8] * proj[ 3] + modl[ 9] * proj[ 7] + modl[10] * proj[11] + modl[11] * proj[15];

	clip[12] = modl[12] * proj[ 0] + modl[13] * proj[ 4] + modl[14] * proj[ 8] + modl[15] * proj[12];
	clip[13] = modl[12] * proj[ 1] + modl[13] * proj[ 5] + modl[14] * proj[ 9] + modl[15] * proj[13];
	clip[14] = modl[12] * proj[ 2] + modl[13] * proj[ 6] + modl[14] * proj[10] + modl[15] * proj[14];
	clip[15] = modl[12] * proj[ 3] + modl[13] * proj[ 7] + modl[14] * proj[11] + modl[15] * proj[15];
	
	m_Frustum[0][0] = clip[ 3] - clip[ 0];
	m_Frustum[0][1] = clip[ 7] - clip[ 4];
	m_Frustum[0][2] = clip[11] - clip[ 8];
	m_Frustum[0][3] = clip[15] - clip[12];

	this->NormalizePlane(0);

	m_Frustum[1][0] = clip[ 3] + clip[ 0];
	m_Frustum[1][1] = clip[ 7] + clip[ 4];
	m_Frustum[1][2] = clip[11] + clip[ 8];
	m_Frustum[1][3] = clip[15] + clip[12];

	this->NormalizePlane(1);

	m_Frustum[2][0] = clip[ 3] + clip[ 1];
	m_Frustum[2][1] = clip[ 7] + clip[ 5];
	m_Frustum[2][2] = clip[11] + clip[ 9];
	m_Frustum[2][3] = clip[15] + clip[13];

	this->NormalizePlane(2);

	m_Frustum[3][0] = clip[ 3] - clip[ 1];
	m_Frustum[3][1] = clip[ 7] - clip[ 5];
	m_Frustum[3][2] = clip[11] - clip[ 9];
	m_Frustum[3][3] = clip[15] - clip[13];

	this->NormalizePlane(3);

	m_Frustum[4][0] = clip[ 3] - clip[ 2];
	m_Frustum[4][1] = clip[ 7] - clip[ 6];
	m_Frustum[4][2] = clip[11] - clip[10];
	m_Frustum[4][3] = clip[15] - clip[14];

	this->NormalizePlane(4);

	m_Frustum[5][0] = clip[ 3] + clip[ 2];
	m_Frustum[5][1] = clip[ 7] + clip[ 6];
	m_Frustum[5][2] = clip[11] + clip[10];
	m_Frustum[5][3] = clip[15] + clip[14];

	this->NormalizePlane(5);
	return;
};

__fastcall bool Aegis::G1::Frustum::PointInFrustum( const Aegis::M2::Vertex& v ) {
	for(int i = 0; i < 6; ++i ) {
		if( m_Frustum[i][0] * v.x + m_Frustum[i][1] * v.y + 
            m_Frustum[i][2] * v.z + m_Frustum[i][3] <= 0.0f){
			return false;
		}
	}
	return true;
};

__fastcall bool Aegis::G1::Frustum::SphereInFrustum( const Aegis::M2::Vertex & v,
                                                     float radius ) {
	for(int i = 0; i < 6; ++i ){
		if( m_Frustum[i][0] * v.x + m_Frustum[i][1] * v.y + 
            m_Frustum[i][2] * v.z + m_Frustum[i][3] <= -radius ){
			return false;
		}
	}
	return true;
};

__fastcall bool Aegis::G1::Frustum::CubeInFrustum( const Aegis::M2::Vertex & v,
                                                   float size ) {
	for(int i = 0; i < 6; ++i ){
		if (m_Frustum[i][0] * (v.x - size) + m_Frustum[i][1] * (v.y - size) + m_Frustum[i][2] * (v.z - size) + m_Frustum[i][3] > 0.0f) continue;
		if (m_Frustum[i][0] * (v.x + size) + m_Frustum[i][1] * (v.y - size) + m_Frustum[i][2] * (v.z - size) + m_Frustum[i][3] > 0.0f) continue;
		if (m_Frustum[i][0] * (v.x - size) + m_Frustum[i][1] * (v.y + size) + m_Frustum[i][2] * (v.z - size) + m_Frustum[i][3] > 0.0f) continue;
		if (m_Frustum[i][0] * (v.x + size) + m_Frustum[i][1] * (v.y + size) + m_Frustum[i][2] * (v.z - size) + m_Frustum[i][3] > 0.0f) continue;
		if (m_Frustum[i][0] * (v.x - size) + m_Frustum[i][1] * (v.y - size) + m_Frustum[i][2] * (v.z + size) + m_Frustum[i][3] > 0.0f) continue;
		if (m_Frustum[i][0] * (v.x + size) + m_Frustum[i][1] * (v.y - size) + m_Frustum[i][2] * (v.z + size) + m_Frustum[i][3] > 0.0f) continue;
		if (m_Frustum[i][0] * (v.x - size) + m_Frustum[i][1] * (v.y + size) + m_Frustum[i][2] * (v.z + size) + m_Frustum[i][3] > 0.0f) continue;
		if (m_Frustum[i][0] * (v.x + size) + m_Frustum[i][1] * (v.y + size) + m_Frustum[i][2] * (v.z + size) + m_Frustum[i][3] > 0.0f) continue;
		return false;
	}
    return true;
};
__fastcall bool Aegis::G1::Frustum::FaceInFrustum( const Aegis::M2::Vertex& v1,
                                                   const Aegis::M2::Vertex& v2,
                                                   const Aegis::M2::Vertex& v3 ) {
    Aegis::M2::Vertex mid;

    mid.x = (v1.x + v2.x + v3.x) / 3.0f;
    mid.y = (v1.y + v2.y + v3.y) / 3.0f;
    mid.z = (v1.z + v2.z + v3.z) / 3.0f;

    float dis = Aegis::M2::Distance( v1, mid );
    return SphereInFrustum(mid, dis);
};

__fastcall bool Aegis::G1::Frustum::RectInFrustum( const Aegis::M2::Vertex v[4] ) {
    float x = 0.0f, y = 0.0f, z = 0.0f;
    for (int i = 0; i < 4; ++i) { x += v[i].x; y += v[i].y; z += v[i].z; };

    x /= 4.0f, y /= 4.0f, z /= 4.0f;

    float sizex = fabs(x - v[0].x);
    float sizey = fabs(y - v[0].y);
    float sizez = fabs(z - v[0].z);    
    
	for(int i = 0; i < 6; ++i){
		if(m_Frustum[i][0] * (x - sizex) + m_Frustum[i][1] * (y - sizey) + m_Frustum[i][2] * (z - sizez) + m_Frustum[i][3] > 0.0f) continue;
		if(m_Frustum[i][0] * (x + sizex) + m_Frustum[i][1] * (y - sizey) + m_Frustum[i][2] * (z - sizez) + m_Frustum[i][3] > 0.0f) continue;
		if(m_Frustum[i][0] * (x - sizex) + m_Frustum[i][1] * (y + sizey) + m_Frustum[i][2] * (z - sizez) + m_Frustum[i][3] > 0.0f) continue;
		if(m_Frustum[i][0] * (x + sizex) + m_Frustum[i][1] * (y + sizey) + m_Frustum[i][2] * (z - sizez) + m_Frustum[i][3] > 0.0f) continue;
		if(m_Frustum[i][0] * (x - sizex) + m_Frustum[i][1] * (y - sizey) + m_Frustum[i][2] * (z + sizez) + m_Frustum[i][3] > 0.0f) continue;
		if(m_Frustum[i][0] * (x + sizex) + m_Frustum[i][1] * (y - sizey) + m_Frustum[i][2] * (z + sizez) + m_Frustum[i][3] > 0.0f) continue;
		if(m_Frustum[i][0] * (x - sizex) + m_Frustum[i][1] * (y + sizey) + m_Frustum[i][2] * (z + sizez) + m_Frustum[i][3] > 0.0f) continue;
		if(m_Frustum[i][0] * (x + sizex) + m_Frustum[i][1] * (y + sizey) + m_Frustum[i][2] * (z + sizez) + m_Frustum[i][3] > 0.0f) continue;
		return false;
	}

	return true;
};


