/*
  Name: aG1_LightControl.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G1 - Light Control
*/

#ifndef _AG1_LIGHT_CONTROL_HPP
#define _AG1_LIGHT_CONTROL_HPP

#include <windows.h>
#include <gl/gl.h>
#include <math.h>
#include <stack>

namespace Aegis {
    namespace G1 {
        // definition
        class LightControlObject;
    };
};

#include "aG1_Renderer.hpp"
#include "../../M/M2/aM2_Color.hpp"
#include "../../M/M2/aM2_Vertex.hpp"
#include "../../M/M2/aM2_Normal.hpp"

namespace Aegis {
    namespace G1 {

        class LightControlObject {
            private:
                const Aegis::G1::Renderer & renderer; // owner

                // constants
                int  n_lights;
                
                // caches
                bool lighting;
                
                struct LightState {
                    bool enabled;
                    bool positional;
                    
                    Aegis::M2::Color ambient;
                    Aegis::M2::Color diffuse;
                    Aegis::M2::Color specular;

                    Aegis::M2::Vertex position; // for positional
                    Aegis::M2::Normal direction; // for directional

                    Aegis::M2::Normal spot_direction;
                    float exponent;
                    float cutoff;

                    float attenuation_quadratic;
                    float attenuation_linear;
                    float attenuation_constant;
                } *light;
                
                // light model
                bool local_viewer;
                bool double_sided;
                Aegis::M2::Color global_ambient;
                

            public:
                __fastcall LightControlObject( const Aegis::G1::Renderer & );
                __fastcall ~LightControlObject();
                
                __fastcall void Enable( bool en = true );
                inline void Disable() { this->Enable(false); };

                __fastcall void EnableLight( int light, bool en = true );
                inline void DisableLight(int l) { this->EnableLight(l,false); };

                __fastcall void DisableAll();

                inline unsigned int GetNumLights() { return this->n_lights; };

                __fastcall void LocalViewer( bool );
                __fastcall void DoubleSided( bool );
                __fastcall void SetGlobalAmbient( const Aegis::M2::Color & amb );

                __fastcall void SetPositional( int light, Aegis::M2::Vertex pos );
                __fastcall void SetDirectional( int light, Aegis::M2::Normal dir );
                
                __fastcall void SetAmbient(int light, const Aegis::M2::Color& c);
                __fastcall void SetDiffuse(int light, const Aegis::M2::Color& c);
                __fastcall void SetSpecular(int light, const Aegis::M2::Color& c);
                
                inline void SetColor(int light, const Aegis::M2::Color& c) {
                    this->SetDiffuse(light, c);
                    return;
                };

                __fastcall void SetSpotDirection( int light,
                                                  const Aegis::M2::Normal & dir);
                __fastcall void SetSpotExponent( int light, float exponent );
                __fastcall void SetSpotCutOff( int light, float cutoff );
                __fastcall void SetAttenuation( int light, float constant,
                                                float linear = 0.0f,
                                                float quadratic = 0.0f );

                inline void SetOmni( int light ) {
                    // TODO: Assure positional light here
                    this->SetSpotCutOff( light, 180.0f );
                };
                
        };
        
    };
};

#endif

