/*
  Name: aG1_MaterialControl.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G1 - Material Cache
  
  Well, this file is REAL candidate to unit-at-time common subexpression
  elimination optimizations. I've never copied and pasted so many lines :-)))
*/

#include "aG1_MaterialControl.hpp"

__fastcall Aegis::G1::MaterialControlObject::
           MaterialControlObject( const Aegis::G1::Renderer & rn ) 
           : renderer(rn) {
    
    this->ClearCache();
//    this->fparam_stack.reserve(2048);
//    this->color_stack.reserve(8192);
    
    return;
};

__fastcall void Aegis::G1::MaterialControlObject::ClearCache() {
    this->shininess = 0.0f;
    this->back_shininess = 0.0f;

    this->ambient  = Aegis::M2::Color(-1.0f,-1.0f,-1.0f-1.0f);
    this->diffuse  = Aegis::M2::Color(-1.0f,-1.0f,-1.0f-1.0f);
    this->specular = Aegis::M2::Color(-1.0f,-1.0f,-1.0f-1.0f);
    this->emission = Aegis::M2::Color(-1.0f,-1.0f,-1.0f-1.0f);

    this->back_ambient  = Aegis::M2::Color(-1.0f,-1.0f,-1.0f-1.0f);
    this->back_diffuse  = Aegis::M2::Color(-1.0f,-1.0f,-1.0f-1.0f);
    this->back_specular = Aegis::M2::Color(-1.0f,-1.0f,-1.0f-1.0f);
    this->back_emission = Aegis::M2::Color(-1.0f,-1.0f,-1.0f-1.0f);

    return;
};

__fastcall void 
Aegis::G1::MaterialControlObject::ApplyFrontAmbient( const Aegis::M2::Color& clr ){
    if (this->ambient != clr) {
        this->ambient = clr;
        glMaterialfv( GL_FRONT, GL_AMBIENT,  clr.c );
    };
    return;
};
    
__fastcall void
Aegis::G1::MaterialControlObject::ApplyFrontDiffuse( const Aegis::M2::Color& clr ){
    if (this->diffuse != clr) {
        this->diffuse = clr;
        glMaterialfv( GL_FRONT, GL_DIFFUSE,  clr.c );
    };
    return;
};

__fastcall void 
Aegis::G1::MaterialControlObject::ApplyFrontSpecular( const Aegis::M2::Color& clr){
    if (this->specular != clr) {
        this->specular = clr;
        glMaterialfv( GL_FRONT, GL_SPECULAR,  clr.c );
    };
    return;
};

__fastcall void 
Aegis::G1::MaterialControlObject::ApplyFrontEmmision( const Aegis::M2::Color& clr){
    if (this->emission != clr) {
        this->emission = clr;
        glMaterialfv( GL_FRONT, GL_EMISSION,  clr.c );
    };
    return;
};

__fastcall void 
Aegis::G1::MaterialControlObject::ApplyFrontShine( float intensity ) {
    if (this->shininess != intensity) {
        this->shininess = intensity;
        glMaterialf( GL_FRONT, GL_SHININESS,  intensity );
    };
    return;
};


__fastcall void 
Aegis::G1::MaterialControlObject::ApplyBackAmbient( const Aegis::M2::Color& clr ){
    if (this->back_ambient != clr) {
        this->back_ambient = clr;
        glMaterialfv( GL_BACK, GL_AMBIENT,  back_ambient.c );
    };
    return;
};

__fastcall void 
Aegis::G1::MaterialControlObject::ApplyBackDiffuse( const Aegis::M2::Color& clr ){
    if (this->back_diffuse != clr) {
        this->back_diffuse = clr;
        glMaterialfv( GL_BACK, GL_DIFFUSE,  clr.c );
    };
    return;
};

__fastcall void 
Aegis::G1::MaterialControlObject::ApplyBackSpecular( const Aegis::M2::Color& clr ){
    if (this->back_specular != clr) {
        this->back_specular = clr;
        glMaterialfv( GL_BACK, GL_SPECULAR,  clr.c );
    };
    return;
};

__fastcall void 
Aegis::G1::MaterialControlObject::ApplyBackEmmision( const Aegis::M2::Color& clr ){
    if (this->back_emission != clr) {
        this->back_emission = clr;
        glMaterialfv( GL_BACK, GL_EMISSION,  clr.c );
    };
    return;
};

__fastcall void 
Aegis::G1::MaterialControlObject::ApplyBackShine( float intensity ) {
    if (this->back_shininess != intensity) {
        this->back_shininess = intensity;
        glMaterialf( GL_BACK, GL_SHININESS,  intensity );
    };
    return;
};

void Aegis::G1::MaterialControlObject::Apply( const Aegis::M2::Color& a,
                                              const Aegis::M2::Color& d,
                                              const Aegis::M2::Color& s,
                                              const Aegis::M2::Color& e,
                                              float sh ) {
    if (this->ambient != a) {
        this->ambient = a;
        glMaterialfv( GL_FRONT, GL_AMBIENT,  a.c );
    };
    if (this->diffuse != d) {
        this->diffuse = d;
        glMaterialfv( GL_FRONT, GL_DIFFUSE,  d.c );
    };
    if (this->specular != s) {
        this->specular = s;
        glMaterialfv( GL_FRONT, GL_SPECULAR, s.c );
    };
    if (this->emission != e) {
        this->emission = e;
        glMaterialfv( GL_FRONT, GL_EMISSION, e.c );
    };
    if (this->back_shininess != sh) {
        this->back_shininess = sh;
        glMaterialf ( GL_FRONT, GL_SHININESS,sh );
    };

    return;
};    

void Aegis::G1::MaterialControlObject::Apply( const Aegis::M2::Color& a,
                                              const Aegis::M2::Color& d,
                                              const Aegis::M2::Color& s,
                                              const Aegis::M2::Color& e,
                                              float sh,
                                              const Aegis::M2::Color& ba,
                                              const Aegis::M2::Color& bd,
                                              const Aegis::M2::Color& bs,
                                              const Aegis::M2::Color& be,
                                              float bsh ) {
    if (this->ambient != a) {
        this->ambient = a;
        glMaterialfv( GL_FRONT, GL_AMBIENT,  a.c );
    };
    if (this->diffuse != d) {
        this->diffuse = d;
        glMaterialfv( GL_FRONT, GL_DIFFUSE,  d.c );
    };
    if (this->specular != s) {
        this->specular = s;
        glMaterialfv( GL_FRONT, GL_SPECULAR, s.c );
    };
    if (this->emission != e) {
        this->emission = e;
        glMaterialfv( GL_FRONT, GL_EMISSION, e.c );
    };
    if (this->shininess != sh) {
        this->shininess = sh;
        glMaterialf ( GL_FRONT, GL_SHININESS,sh );
    };

    // back side
    if (this->back_ambient != ba) {
        this->back_ambient = ba;
        glMaterialfv( GL_BACK, GL_AMBIENT,  ba.c );
    };
    if (this->back_diffuse != bd) {
        this->back_diffuse = bd;
        glMaterialfv( GL_BACK, GL_DIFFUSE,  bd.c );
    };
    if (this->back_specular != bs) {
        this->back_specular = bs;
        glMaterialfv( GL_BACK, GL_SPECULAR, bs.c );
    };
    if (this->back_emission != be) {
        this->back_emission = be;
        glMaterialfv( GL_BACK, GL_EMISSION, be.c );
    };
    if (this->back_shininess != bsh) {
        this->back_shininess = bsh;
        glMaterialf ( GL_BACK, GL_SHININESS,bsh );
    };

    return;
};


__fastcall void Aegis::G1::MaterialControlObject::Push() {
    this->fparam_stack.push( this->back_shininess );
    this->fparam_stack.push( this->shininess );
    
    this->color_stack.push( this->back_emission );
    this->color_stack.push( this->back_specular );
    this->color_stack.push( this->back_diffuse );
    this->color_stack.push( this->back_ambient );
    
    this->color_stack.push( this->emission );
    this->color_stack.push( this->specular );
    this->color_stack.push( this->diffuse );
    this->color_stack.push( this->ambient );

    return;
};    

__fastcall void Aegis::G1::MaterialControlObject::Pop() {
    this->ApplyFrontShine(this->fparam_stack.top()); this->fparam_stack.pop();
    this->ApplyBackShine(this->fparam_stack.top()); this->fparam_stack.pop();

    this->ApplyFrontAmbient(this->color_stack.top()); this->color_stack.pop();
    this->ApplyFrontDiffuse(this->color_stack.top()); this->color_stack.pop();
    this->ApplyFrontSpecular(this->color_stack.top()); this->color_stack.pop();
    this->ApplyFrontEmmision(this->color_stack.top()); this->color_stack.pop();

    this->ApplyBackAmbient(this->color_stack.top()); this->color_stack.pop();
    this->ApplyBackDiffuse(this->color_stack.top()); this->color_stack.pop();
    this->ApplyBackSpecular(this->color_stack.top()); this->color_stack.pop();
    this->ApplyBackEmmision(this->color_stack.top()); this->color_stack.pop();
    
    return;
};    



