/*
  Name: aG1_MaterialControl.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G1 - Material Cache
*/

#ifndef _AG1_MATERIAL_CONTROL_HPP
#define _AG1_MATERIAL_CONTROL_HPP

#include <windows.h>
#include <gl/gl.h>
#include <stack>

namespace Aegis {
    namespace G1 {
        // declaration
        class MaterialControlObject;
    };
};

#include "../G1/aG1_Renderer.hpp"
#include "../../M/M2/aM2_Color.hpp"

namespace Aegis {
    namespace G1 {
        
        // definition
        class MaterialControlObject {
            private:
                const Aegis::G1::Renderer & renderer; // owner

                float shininess; // 0.0f - 128.0f
                float back_shininess; // 0.0f - 128.0f
                float something_unused;

                Aegis::M2::Color ambient;
                Aegis::M2::Color diffuse;
                Aegis::M2::Color specular;
                Aegis::M2::Color emission;
    
                Aegis::M2::Color back_ambient;
                Aegis::M2::Color back_diffuse;
                Aegis::M2::Color back_specular;
                Aegis::M2::Color back_emission;
                

                std::stack<float> fparam_stack;
                std::stack<Aegis::M2::Color> color_stack;
                
            public:
                __fastcall MaterialControlObject( const Aegis::G1::Renderer & );
                
                __fastcall void ClearCache(); // clears caches
                
                __fastcall void ApplyFrontAmbient( const Aegis::M2::Color& );
                __fastcall void ApplyFrontDiffuse( const Aegis::M2::Color& );
                __fastcall void ApplyFrontSpecular( const Aegis::M2::Color& );
                __fastcall void ApplyFrontEmmision( const Aegis::M2::Color& );
                __fastcall void ApplyFrontShine( float intensity );

                __fastcall void ApplyBackAmbient( const Aegis::M2::Color& );
                __fastcall void ApplyBackDiffuse( const Aegis::M2::Color& );
                __fastcall void ApplyBackSpecular( const Aegis::M2::Color& );
                __fastcall void ApplyBackEmmision( const Aegis::M2::Color& );
                __fastcall void ApplyBackShine( float intensity );
                            
                void Apply( const Aegis::M2::Color&, const Aegis::M2::Color&,
                            const Aegis::M2::Color&, const Aegis::M2::Color&,
                            float ); // only front
                // ambient, diffuse, specular, emission and shinniness
                            
                void Apply( const Aegis::M2::Color&, const Aegis::M2::Color&,
                            const Aegis::M2::Color&, const Aegis::M2::Color&,
                            float,
                            const Aegis::M2::Color&, const Aegis::M2::Color&,
                            const Aegis::M2::Color&, const Aegis::M2::Color&,
                            float ); // both sides

                __fastcall void Push();
                __fastcall void Pop();
        };
        
    };
};

#endif

