/*
  Name: aG1_Renderer.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G1 - Renderer Class
*/

#include "aG1_Renderer.hpp"

#include <math.h>

Aegis::G1::Renderer::Renderer(HDC hDC, bool accuratefps, bool gdisafe) {
    this->paused = true;
    this->quit = false;
    this->time = 0;
    
    this->szRenderer = "Aegis G (OpenGL)";
    this->szVendor = "n.v.t. Mx-3";
    this->glVersion = 1.0;
    
    this->hDC = hDC;
    if (accuratefps)
        this->perf_timer
            = QueryPerformanceFrequency((LARGE_INTEGER *)
                                        &(this->perf_timer_freq));
    else
        this->perf_timer = false;
    
    for (int i=0; i<32; ++i) this->fps_compute_array[i] = 0.0f;
    this->fps_compute_array_ptr = 0;
    
    // initialize paralel processing
    InitializeCriticalSection(&this->criticalSection);
    this->hCopyEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    this->hRenderedPauseEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
    this->hEvaluatorPauseEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
    this->hRendererInitialized = CreateEvent(NULL, TRUE, FALSE, NULL);

    DWORD dw = 0;
    this->hThreadEvaluator = CreateThread( NULL, 0, this->ThreadEvaluator,
                                           this, CREATE_SUSPENDED, &dw);
    this->hThreadRenderer = CreateThread( NULL, 0, this->ThreadRenderer,
                                          this, CREATE_SUSPENDED, &dw);

//    SetThreadPriority( this->hThreadEvaluator, THREAD_PRIORITY_BELOW_NORMAL );
//    SetThreadPriority( this->hThreadRenderer, THREAD_PRIORITY_BELOW_NORMAL );

    // start OpenGL
    int iFormat;
    PIXELFORMATDESCRIPTOR pfd; ZeroMemory (&pfd, sizeof (pfd));
    
    pfd.nSize = sizeof (pfd);
    pfd.nVersion = 1;

    if (gdisafe) // not guaranteed
        pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL |
                      PFD_SWAP_COPY | PFD_DOUBLEBUFFER;
    else    
        pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL |
                      PFD_DOUBLEBUFFER | PFD_SWAP_EXCHANGE |
                      PFD_GENERIC_ACCELERATED;
                  
    pfd.iPixelType = PFD_TYPE_RGBA;
	pfd.iLayerType = PFD_MAIN_PLANE;
    
	pfd.cColorBits = 32; // 9 MB VRAM if 1024*768 (back, front, depth, stencil)
	pfd.cDepthBits = 24;
    pfd.cStencilBits = 8;
    
	pfd.cRedBits = 8;
	pfd.cGreenBits = 8;
	pfd.cBlueBits = 8;
	pfd.cAlphaBits = 8;

	pfd.cRedShift = 16;
	pfd.cGreenShift = 8;
	pfd.cBlueShift = 0;
	pfd.cAlphaShift = 24;

	pfd.cAccumBits = 32;
	pfd.cAccumRedBits = 8;
	pfd.cAccumGreenBits = 8;
	pfd.cAccumBlueBits = 8;
	pfd.cAccumAlphaBits = 8;
	
    iFormat = ChoosePixelFormat (hDC, &pfd);
    SetPixelFormat (hDC, iFormat, &pfd);
    
    this->hRC = wglCreateContext( hDC );

    ResumeThread( hThreadRenderer );
    ResumeThread( hThreadEvaluator );

    // wait for renderer to init
    WaitForSingleObject( this->hRendererInitialized, INFINITE );

    // and wait to be unpaused
    return;
};

Aegis::G1::Renderer::~Renderer() {
    // wait for threads to finish
    this->quit = true;

    if (this->paused) this->Pause(false);

    WaitForSingleObject( this->hThreadRenderer, INFINITE );
    WaitForSingleObject( this->hThreadEvaluator, INFINITE );
    DeleteCriticalSection( &this->criticalSection );
    
    // destroy OpenGL
    this->DeactivateThisThread();
    wglDeleteContext(this->hRC);
    
    delete this->Ext;
    delete this->Frame;
    delete this->Textures;
    delete this->Materials;
    delete this->Lights;
    delete this->Fog;
    delete this->Blend;
    
    return;
};

void Aegis::G1::Renderer::Pause(bool p) {
    // this order should be the best (aviods rendering the same frames again)
    if (p) {
        ResetEvent(this->hEvaluatorPauseEvent);
        ResetEvent(this->hRenderedPauseEvent);
    } else {
        SetEvent(this->hRenderedPauseEvent);
        SetEvent(this->hEvaluatorPauseEvent);
    };
    this->paused = p;
    Sleep(10);

    return;
};

// TODO: Find solution so PULSE will render only ONE frame
void Aegis::G1::Renderer::Pulse() {
    this->Pause(false);
    this->Pause(true);
    
    // Somehow wait untill new frame is rendered
//    WaitForSingleObject( this->hCopyEvent, INFINITE ); // do not work
    
    return;
};

// threads

DWORD WINAPI Aegis::G1::Renderer::ThreadEvaluator(LPVOID lpThis) {
    Aegis::G1::Renderer *This = (Aegis::G1::Renderer*) lpThis;
    
    WaitForSingleObject( This->hRendererInitialized, INFINITE );
    
    do {
        WaitForSingleObject( This->hEvaluatorPauseEvent, INFINITE );
        // lets compute next frame data
        This->Evaluate( This->time );
        
        WaitForSingleObject( This->hCopyEvent, INFINITE );
        EnterCriticalSection( &This->criticalSection );
        
        This->Swap(This->time);

        LeaveCriticalSection( &This->criticalSection );
    } while(!This->quit);    

    return 0;
};    

DWORD WINAPI Aegis::G1::Renderer::ThreadRenderer(LPVOID lpThis) {
    Aegis::G1::Renderer *This = (Aegis::G1::Renderer*) lpThis;

    This->ActivateThisThread();
    This->Ext       = new Aegis::G1::Extension;
    This->Frame     = new Aegis::G1::FrameControlObject( *This );
    This->Textures  = new Aegis::G1::TextureControlObject( *This );
    This->Materials = new Aegis::G1::MaterialControlObject( *This );
    This->Lights    = new Aegis::G1::LightControlObject( *This );
    This->Fog       = new Aegis::G1::FogControlObject( *This );
    This->Blend     = new Aegis::G1::BlendControlObject( *This );
    
    // get the best look
    glHint( GL_LINE_SMOOTH_HINT, GL_NICEST );
    glHint( GL_POINT_SMOOTH_HINT, GL_NICEST );
    glHint( GL_POLYGON_SMOOTH_HINT, GL_NICEST );

    glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
    glHint( GL_FOG_HINT, GL_NICEST );
    
    if (This->Ext->Is( Aegis::G1::TextureGenerateMipmap ))
        glHint( GL_GENERATE_MIPMAP_HINT_SGIS, GL_NICEST );
        
    if (This->Ext->Is( Aegis::G1::PointParameters )) {
        // set normal attenuation
        static float att[3] = { 0.0f, 1.0f, 0.0f };
        This->Ext->
            glPointParameterfvARB( GL_POINT_DISTANCE_ATTENUATION_ARB, att );
    };    
    
    glDepthFunc(GL_LEQUAL);

    glDisable(GL_DEPTH_TEST);
    glDisable(GL_ALPHA_TEST);
    glDisable(GL_BLEND);
    glDisable(GL_STENCIL_TEST);

    glEnable(GL_DITHER); // May slow down but result in better quality on 16bit
//    glEnable(GL_CULL_FACE); // TODO: This must get enabled in final
    glEnable(GL_NORMALIZE); // according to nVidia's OpenGL Performace FAQv2
    
    __int64 counter = 0, pfcounter = 0;
    
    char *tmp, *mem;

    tmp = (char*)glGetString(GL_RENDERER);
    if (tmp) {
        mem = (char*) malloc( strlen(tmp) + 1 );
        if (mem) This->szRenderer = (char*)memcpy( mem, tmp, strlen(tmp) + 1 );
    };    
    tmp = (char*)glGetString(GL_VENDOR);
    if (tmp) {
        mem = (char*) malloc( strlen(tmp) + 1 );
        if (mem) This->szVendor = (char*)memcpy( mem, tmp, strlen(tmp) + 1 );
    };    
    tmp = (char*)glGetString(GL_VERSION);
    if (tmp) {
        float ver = strtof(tmp, NULL);
        if (ver >= 1.0) This->glVersion = ver;
    };    
    tmp = (char*)gluGetString(GLU_VERSION);
    if (tmp) {
        mem = (char*) malloc( strlen(tmp) + 1 );
        if (mem) This->szUtility = (char*)memcpy( mem, tmp, strlen(tmp) + 1 );
    };    

    SetEvent( This->hRendererInitialized );
    Sleep(1);
    // must be here so the vtables can be intialized for this
    
    glFlush();
    This->Init(); // let application to initialize important things
    glFlush();
    
    do {
        Sleep(1);
        WaitForSingleObject( This->hRenderedPauseEvent, INFINITE );
        EnterCriticalSection( &This->criticalSection );
        
        // Start timming
        if (This->perf_timer) {
            QueryPerformanceCounter((LARGE_INTEGER *) &(counter) );
        } else {
//            counter = GetTickCount();
            timeBeginPeriod(1);
            counter = timeGetTime();
        };
        
        // reset OpenGL
        glDisable(GL_DITHER); // must be disabled
        glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
        glEnable(GL_DITHER);

        glLoadIdentity();
        
        // call rendering function
        This->Render(This->time);
        
        // finish
        glFlush();
        Sleep(0); // give other threads a little more time to finish
        glFinish(); // without this, the hell will come to ride
        
        SwapBuffers( This->hDC );
        SetEvent( This->hCopyEvent ); // tell evaluator that we need new data

        // End timming
        if (This->perf_timer) {
            QueryPerformanceCounter((LARGE_INTEGER *) &(pfcounter) );
            
            counter = pfcounter - counter;
            counter *= 1000ll;
            counter /= This->perf_timer_freq;
            
            if (!counter) counter = 1;
            This->time = counter;
        } else {
//            This->time = GetTickCount() - counter;
            This->time = timeGetTime() - counter;

            if (!counter) counter = 1;
            timeEndPeriod(1);
        };
        
        // compute FPS
        if (This->time)
            This->fps_immediate = 1000.0f / This->time;
        else
            This->fps_immediate = 1000.0f;
        
        // compute accurate fps
        ++This->fps_compute_array_ptr;
        if (This->fps_compute_array_ptr >= 32) This->fps_compute_array_ptr = 0;

        if (This->fps_immediate < 1000.0f)
            This->fps_compute_array[This->fps_compute_array_ptr]
                = This->fps_immediate;
        else
            This->fps_compute_array[This->fps_compute_array_ptr] = 100.0f;

        This->fps = 0.0f;
        for (int i=0; i<32; ++i) This->fps += This->fps_compute_array[i];
        This->fps /= 32.0f;
        
        LeaveCriticalSection( &This->criticalSection );
    } while(!This->quit);

    SetEvent( This->hCopyEvent ); // so evaluator can finish safely
    This->DeactivateThisThread();
    
    return 0;
};    

int Aegis::G1::Renderer::Init() { return 0; };
int Aegis::G1::Renderer::Evaluate(int t) { return 0; };
int Aegis::G1::Renderer::Render(int t) { return 0; };
int Aegis::G1::Renderer::Swap(int t) { return 0; };


void Aegis::G1::Renderer::
Orthogonal( int vpoleft, int vpobott, int screenx, int screeny,
            double left, double right, double bottom, double top ,
            double znear, double zfar ) {

    glLoadIdentity();
    glViewport( vpoleft, vpobott, screenx, screeny );

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho( left, right, bottom, top, znear, zfar );
    
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity(); 

    return;
};    

void Aegis::G1::Renderer::
Frustum( int vpoleft, int vpobott, int screenx, int screeny,
         double angle, double aspect,
         double znear, double zfar ) {

	const double pi = 3.1415926535897932384626433832795;

    glLoadIdentity();
    glViewport( vpoleft, vpobott, screenx, screeny );
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

	double fH = tan( angle / 180.0 * pi ) * znear / 2.0;
	double fW = fH * aspect;
	glFrustum( -fW, fW, -fH, fH, znear, zfar );

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity(); 
    
    glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
    glEnable(GL_DEPTH_TEST);
    
    return;
};    



