/*
  Name: aG1_Renderer.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G1 - Renderer Class
*/

#ifndef _AG1_RENDERER_HPP
#define _AG1_RENDERER_HPP

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>

namespace Aegis {
    namespace G1 {
        class Renderer;
    };
};

#include "aG1_Extension.hpp"
#include "aG1_FrameControl.hpp"
#include "aG1_TextureControl.hpp"
#include "aG1_MaterialControl.hpp"
#include "aG1_FogControl.hpp"
#include "aG1_BlendControl.hpp"
#include "aG1_LightControl.hpp"

namespace Aegis {
    namespace G1 {

        // declaration
        class Renderer {
            private:
                bool paused;
                bool quit;
                
                bool perf_timer; 
                __int64 perf_timer_freq;
                
                int  time; // rendering length of last frame

                float fps_compute_array[32];
                unsigned int fps_compute_array_ptr;
                
                HDC hDC;
                HGLRC hRC;
                
                HANDLE hThreadEvaluator, hThreadRenderer;
                HANDLE hCopyEvent;
                HANDLE hEvaluatorPauseEvent, hRenderedPauseEvent;
                HANDLE hRendererInitialized;
                CRITICAL_SECTION criticalSection;
                
                static DWORD WINAPI ThreadEvaluator(LPVOID);
                static DWORD WINAPI ThreadRenderer(LPVOID);
            
            public:
                // childrens may need these
                char *szRenderer, *szVendor, *szUtility;
                float glVersion;

                float fps;
                float fps_immediate;
                
            public:
                // control objects
                Aegis::G1::Extension *Ext;
                Aegis::G1::FrameControlObject *Frame;
                Aegis::G1::TextureControlObject *Textures;
                Aegis::G1::MaterialControlObject *Materials;
                Aegis::G1::FogControlObject *Fog;
                Aegis::G1::BlendControlObject *Blend;
                Aegis::G1::LightControlObject *Lights;
                
            protected:
                // this should implement childens
                // Init - called in rendering context before loop starts
                virtual int Init();
                // Evaluate - runs simultaneusly with Render
                //          - cannot execute opengl commands
                virtual int Evaluate(int t);
                // Render - runs simultaneusly with Evaluate
                //        - renders (through OGL) image to screen
                virtual int Render(int t);
                // Swap - called when data used by Evaluate should
                //        be copied to data used by Render
                //      - if Evaluate is not used, computing should
                //        be done here (no multithread advantage)
                virtual int Swap(int t);
                
            public:
                Renderer(HDC, bool accuratefps = true, bool gdisafe = false);
                ~Renderer();
                
                bool IsPaused() const { return paused; };
                void Pause(bool);
                void Pulse();

                void DeactivateThisThread() { wglMakeCurrent( NULL, NULL ); };
                void ActivateThisThread() { wglMakeCurrent( this->hDC, this->hRC ); };
                
                void Orthogonal( int vpoleft, int vpobott,
                                 int screenx, int screeny,
                                 double left, double right,
                                 double bottom, double top,
                                 double znear, double zfar );
                void Frustum( int vpoleft, int vpobott,
                              int screenx, int screeny,
                              double angle, double aspect,
                              double znear, double zfar );
                              
                const char *name() const { return this->szRenderer; };
                const char *vendor() const { return this->szVendor; };
                const float version() const { return this->glVersion; };
        };
        
    };
};

#endif

