/*
  Name: aG1_TextureControl.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G1 - Texture Cache
*/

#include "aG1_TextureControl.hpp"

__fastcall Aegis::G1::TextureControlObject::TextureControlObject(
                      const Aegis::G1::Renderer & rn )
    : renderer(rn) {
    
    this->current_unit = 0;
    if (rn.Ext->Is(Aegis::G1::Multitexture)) {
        glGetIntegerv( GL_MAX_TEXTURE_UNITS_ARB, &this->max_unit );
        renderer.Ext->glActiveTextureARB( GL_TEXTURE0_ARB );
        
        if (this->max_unit > AEGIS_G1_MULTITEXTURE)
            this->max_unit = AEGIS_G1_MULTITEXTURE;
    } else {
        this->max_unit = 1;
    };
    
//    this->current_stack.reserve(1024);
//    this->unit_stack.reserve(4096);
    
    // clear cache
    for (int i=0; i<this->max_unit; ++i) {
        // initialize all data
        this->unit[i].mode = 0;
        this->unit[i].texture = 0;
        this->unit[i].env_mode = 0;
        this->unit[i].env_color = Aegis::M2::Color(0.0f, 0.0f, 0.0f, 0.0f);
    };
    return;
};

__fastcall void Aegis::G1::TextureControlObject::Bind( unsigned int tx ) {
    // cannot bind when no texture mode is active
    if (!this->unit[this->current_unit].mode) return;
    
    // bind if not already cached
    if (this->unit[this->current_unit].texture != tx) {
        this->unit[this->current_unit].texture = tx;
        glBindTexture( this->unit[this->current_unit].mode, tx );
    };
    return;
};
    
__fastcall void Aegis::G1::TextureControlObject::Current( unsigned int unit ) {
    if (unit >= this->max_unit) return;
    
    // bind if not already cached
    if (this->current_unit != unit) {
        this->current_unit = unit;
        renderer.Ext->glActiveTextureARB( GL_TEXTURE0_ARB + unit );
    };
    return;
};

__fastcall void Aegis::G1::TextureControlObject::Enable( unsigned int unit,
                                                         unsigned int mode ) {
    if (unit == 0xFFFFFFFF) unit = this->Current(); 
    if (unit >= this->max_unit) return;
    this->Current(unit);

    // check if this mode is already set and then return
    if (this->unit[this->current_unit].mode == mode) return;

    // disable old mode
    if (this->unit[this->current_unit].mode)
        glDisable( this->unit[this->current_unit].mode );
    
    // set new mode
    if (mode)
        glEnable( mode );
    
    // cache
    this->unit[this->current_unit].mode = mode;
    
    return;
};

__fastcall void Aegis::G1::TextureControlObject::TexEnv( unsigned int env_mode ) {
    if (this->unit[this->current_unit].env_mode != env_mode) {
        this->unit[this->current_unit].env_mode = env_mode;
        glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, env_mode );
    };
    
    return;
};

__fastcall void Aegis::G1::TextureControlObject::EnvColor( 
                           const Aegis::M2::Color& env_color ) {
    if (this->unit[this->current_unit].env_color != env_color) {
        this->unit[this->current_unit].env_color = env_color;
        glTexEnvfv( GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, env_color.c );
    };
    
    return;
};

__fastcall void Aegis::G1::TextureControlObject::Push() {
    this->current_stack.push( this->current_unit );
    for (int i = 0; i < this->max_unit; ++i)
        this->unit_stack.push( this->unit[i] );
    
    return;
};    

__fastcall void Aegis::G1::TextureControlObject::Pop() {
    if (this->current_stack.empty()) return;

    int c;
    __unit u;
    
    c = this->current_stack.top(); this->current_stack.pop();

    for (int i = this->max_unit - 1; i >= 0; --i) {
        u = this->unit_stack.top(); this->unit_stack.pop();
        
        // and set it
        this->Current(i);
        this->Enable(i, u.mode );
        this->Bind(     u.texture );
        this->TexEnv(   u.env_mode );
        this->EnvColor( u.env_color );
    };
    
    this->Current(c);
    
    return;
};

