/*
  Name: aG1_TextureControl.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G1 - Texture Cache
*/

#ifndef _AG1_TEXTURE_CONTROL_HPP
#define _AG1_TEXTURE_CONTROL_HPP

#include <windows.h>
#include <gl/gl.h>
#include <stack>

namespace Aegis {
    namespace G1 {
        // definition
        class TextureControlObject;
    };
};

#include "aG1_Renderer.hpp"
#include "../../M/M2/aM2_Color.hpp"

#define AEGIS_G1_MULTITEXTURE   8
// No reason to support more native texture units, because of no support from
// side of nVidia or ATI. If more units are required, vertex and fragment programs
// should be used then.

namespace Aegis {
    namespace G1 {
        
        class TextureControlObject {
            private:
                const Aegis::G1::Renderer & renderer; // owner

                int current_unit;
                int max_unit; // multitexture maximum
                
                struct __unit {
                    unsigned int mode;
                    unsigned int texture;
                    
                    unsigned int env_mode;
                    Aegis::M2::Color env_color;
                    
                } unit[AEGIS_G1_MULTITEXTURE];
                
                std::stack<int> current_stack;
                std::stack<__unit> unit_stack;

            public:
                __fastcall TextureControlObject( const Aegis::G1::Renderer & );
                
                inline unsigned int Current() { return this->current_unit; };    
                inline unsigned int UnitsNumber() { return this->max_unit; };    

                inline unsigned int GetMode( int u = 0xFFFFFFFF) {
                    if (u == 0xFFFFFFFF)
                        return this->unit[ this->Current() ].mode;
                    else
                        if (u < this->max_unit)
                            return this->unit[ u ].mode;
                        else
                            return 0;
                };
                inline bool IsEnabled( int u = 0xFFFFFFFF)  {
                    return (bool)this->GetMode(u);
                };

                __fastcall void Current( unsigned int unit );
                __fastcall void Enable( unsigned int unit = 0xFFFFFFFF,
                                        unsigned int mode = GL_TEXTURE_2D );

                __fastcall void Bind( unsigned int tx );
                __fastcall void TexEnv( unsigned int env_mode );
                __fastcall void EnvColor( const Aegis::M2::Color& env_color );

                inline void Disable( unsigned int unit = 0xFFFFFFFF ) {
                    this->Enable( unit, 0 );
                    return;
                };
                inline void DisableAll() {
                    for (int i = this->max_unit-1; i >= 0; --i) this->Disable(i);
                    return;
                };
                inline void SetMode( unsigned int mode,
                                     unsigned int unit = 0xFFFFFFFF ) {
                    this->Enable( unit, mode );
                    return;
                };

                __fastcall void Push();
                __fastcall void Pop();
                                
        };
        
    };
};

#endif

        
