/*
  Name: aG2_Element.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G2 - Element Class
*/

#ifndef _AG2_ELEMENT_HPP
#define _AG2_ELEMENT_HPP

#include <vector>

#include "../../Attribs.hpp"
#include "../../M/aM.hpp"
#include "../G1/aG1_Renderer.hpp"
#include "aG2_TexUnit.hpp"
#include "aG2_Face.hpp"
#include "aG2_MultiFace.hpp"

namespace Aegis {
    namespace G2 {
        
        class __aligned Element {
            private:
                const Aegis::G1::Renderer & renderer;
                
                unsigned int object; // vbo or display list name
                bool uncached; // does it need to be recreated (vbo or dl)
                
                // required local memory copy
                std::vector<Aegis::G2::Face>        pure_faces;
                std::vector<Aegis::G2::MultiFace>   multifaces;
                std::vector<Aegis::G2::Face*>       otherfaces;// not in VBO nor DL
                // otherfaces must keep pointers to copies of added faces
                
                // VBO caches
                unsigned int    vbo_vertex_offset; // always 0, for completeness
                unsigned int    vbo_normal_offset;
                unsigned int    vbo_color_offset;
                unsigned int    vbo_coord_offset[AEGIS_G1_MULTITEXTURE];
                unsigned int    vbo_fog_offset;
                
                std::vector<unsigned int>       vbo_modify_texture_points;
                std::vector<unsigned int>       vbo_modify_multitexture_points;
                
            protected:
                // internal functions for fastdata recreation
                __fastcall void RebuildVBO();
                __fastcall void RebuildDL();
                
            public:
                __fastcall Element(const Aegis::G1::Renderer &);
                __fastcall ~Element();

                // clears contents of element                
                __fastcall void Reset();
                // Add Face (or MultiFace or any Face ...)
                // NOTE: Right now accepts only pure Faces without combiners !!!
                __fastcall bool Add(const Aegis::G2::Face &);
                
                
                // Special functions to create common elements
                __fastcall bool BuildSphere( float radius,
                                             unsigned int n1, unsigned int n2,
                                             const Aegis::G2::TexUnit *tu,
                                             unsigned int n_tu );

                // Draw function 
                //  - prepares the object if required
                //  - calls the DL or VBO
                __fastcall void Draw();


        };
        
    };    
};    


#endif

