/*
  Name: aG2_Face.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G2 - Triangle (Face) Class
*/

#include "aG2_Face.hpp"
#include "../../M/M1/aM1_CommonMath.hpp"

__fastcall void Aegis::G2::Face::Draw( const Aegis::G1::Renderer& renderer ) {
    // let's draw
//    renderer.Textures->DisableAll();
    this->texunit.Bind(renderer); // cached, so no call overhead should occur

    if (renderer.Fog->EmulateParametric()) {
        // double pass, emulate parametric fog
        Aegis::M2::Color fc = renderer.Fog->GetColor();
        Aegis::M2::Color c1(this->color[0]),
                         c2(this->color[1]),
                         c3(this->color[2]);
        Aegis::M2::Color f1(fc), f2(fc), f3(fc);
        f1.a = renderer.Fog->ComputeFakeParametricAlpha( this->fog[0] );
        f2.a = renderer.Fog->ComputeFakeParametricAlpha( this->fog[1] );
        f3.a = renderer.Fog->ComputeFakeParametricAlpha( this->fog[2] );

        // TODO: FIXME: Emulated Parametric Fog doesn't work correcly
        //              when color alpha value is less than 1.0f.
        // This is NOT correct!
//        f1.a *= c1.a;// c1.a -= f1.a;
//        f2.a *= c2.a;// c2.a -= f2.a;
//        f3.a *= c3.a;// c3.a -= f3.a;
        
        // render
        renderer.Frame->Push();
        renderer.Frame->DepthMask(0);
        
        glBegin(GL_TRIANGLES);
            glColor4fv   ( c1.c );
            glNormal3fv  ( this->normal[0].v );
            glTexCoord4fv( this->coord [0].c );
            glVertex4fv  ( this->vertex[0].v );
            
            glColor4fv   ( c2.c );
            glNormal3fv  ( this->normal[1].v );
            glTexCoord4fv( this->coord [1].c );
            glVertex4fv  ( this->vertex[1].v );
    
            glColor4fv   ( c3.c );
            glNormal3fv  ( this->normal[2].v );
            glTexCoord4fv( this->coord [2].c );
            glVertex4fv  ( this->vertex[2].v );
        glEnd();
        
        renderer.Frame->Pop();
        
        renderer.Blend->Push();
        renderer.Blend->Default(); // TODO: Consider applying this at the
                                   // beginning of Draw function.
                                   // We won't probably want to use different
                                   // blending and this may not work then.
        
        renderer.Textures->Push();
        renderer.Textures->Disable();

        glBegin(GL_TRIANGLES);
            glColor4fv   ( f1.c );
            glNormal3fv  ( this->normal[0].v );
            glVertex4fv  ( this->vertex[0].v );
            
            glColor4fv   ( f2.c );
            glNormal3fv  ( this->normal[1].v );
            glVertex4fv  ( this->vertex[1].v );
    
            glColor4fv   ( f3.c );
            glNormal3fv  ( this->normal[2].v );
            glVertex4fv  ( this->vertex[2].v );
        glEnd();
        
        renderer.Textures->Pop();
        renderer.Blend->Pop();
//*/
    } else {
        // single pass, only parametric fog
        glBegin(GL_TRIANGLES);
            glColor4fv   ( this->color [0].c );
            glNormal3fv  ( this->normal[0].v );
            glTexCoord4fv( this->coord [0].c );
            renderer.Ext->glFogCoordfEXT( this->fog[0] );
            
            glVertex4fv  ( this->vertex[0].v );
            
            glColor4fv   ( this->color [1].c );
            glNormal3fv  ( this->normal[1].v );
            glTexCoord4fv( this->coord [1].c );
            renderer.Ext->glFogCoordfEXT( this->fog[1] );
            
            glVertex4fv  ( this->vertex[1].v );
    
            glColor4fv   ( this->color [2].c );
            glNormal3fv  ( this->normal[2].v );
            glTexCoord4fv( this->coord [2].c );
            renderer.Ext->glFogCoordfEXT( this->fog[2] );
            
            glVertex4fv  ( this->vertex[2].v );
        glEnd();
    };

    return;
};    


