/*
  Name: aG2_Font.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G2 - Font Class
*/

#include "aG2_Font.hpp"
#include <new>
#include <ansidecl.h>

Aegis::G2::Font::Font( const Aegis::G1::Renderer &ren ):renderer(ren) {
    this->mode = 0;
    for (int i=0; i<256; ++i) {
        this->offsetx[i] = 0.0f;
        this->offsety[i] = 0.0f;
        this->width[i] = 0.0f;
    };
    return;
};

Aegis::G2::Font::Font( const Aegis::G1::Renderer &ren,
                       const Aegis::F::Bitmap & bm, bool fixed)
                :renderer(ren)  {
    this->mode = 0;
    this->Build( bm, fixed );
    return;
};

__fastcall void Aegis::G2::Font::Build( const Aegis::F::Bitmap & bm,
                                        bool fixedwidth ) {
    unsigned int sizex = bm.Width();
    unsigned int sizey = bm.Height();
    
    // if no powers of two, do not continue
    if (!(!(sizex&sizex-1) && !(sizey&sizey-1))) return;
    
    unsigned char *alpha = new(std::nothrow) unsigned char[sizex * sizey];
    if (!alpha) return;
    
    // create alph
    unsigned int sum = 0;
    for (int y = 0; y < sizey; ++y)
        for (int x = 0; x < sizex; ++x) {
            sum = bm.GetPixelRed(x,y)
                + bm.GetPixelGreen(x,y)
                + bm.GetPixelBlue(x,y);
            sum /= 3;
            sum *= bm.GetPixelAlpha(x,y);
            sum /= 255;
            
            alpha[y*sizex + x] = (unsigned char)sum;
        };
    
    // build texture
    this->renderer.Textures->Push();
    this->renderer.Textures->Enable(0,GL_TEXTURE_2D);

    this->tex.SetParams( this->renderer, 0, 1, 0, 0 );
    this->tex.SetPriority( this->renderer, Aegis::G2::TexturePriorityHigh );
    
    glTexImage2D( GL_TEXTURE_2D, 0, GL_ALPHA8, sizex, sizey,
                  0, GL_ALPHA, GL_UNSIGNED_BYTE, alpha );

    this->renderer.Textures->Pop();
    
    // compute offsets and widths
    // offsets
    for (int c = 0; c < 256; c++) {
        this->offsetx[c] = float(c%16) / 16.0f;
        this->offsety[c] = float(c/16) / 16.0f;
    };
        
    // widths
    if (fixedwidth) {
        // only 'X' letter width
        int cx = ('X' % 16) * (sizex/16);
        int cy = ('X' / 16) * (sizey/16);
        int w;

        for (w = (sizex/16) - 1; w >= 0; --w) {
            for (int row = 0; row < (sizey/16); ++row) {
                int py = cy + row;
                int px = cx + w;
                
                if (alpha[py * sizex + px]) goto _aegis_g2_font_build_l0;
            };
        };
        
        _aegis_g2_font_build_l0:

        ++w;
        for (int c = 0; c < 256; ++c)
            this->width[c] = ((float)w / (float)(sizex/16)) / 16.0f;
        
    } else {
        int chw = sizex/16;
        int chh = sizey/16;
        for (int c = 0; c < 256; ++c) {
            int cx = (c % 16) * chw;
            int cy = (c / 16) * chh;
            int w;
            
            for (w = chw-1; w >= 0; --w) {
                for (int row = 0; row < chh; ++row) {
                    int py = cy + row;
                    int px = cx + w;
                    
                    if (alpha[py * sizex + px]) goto _aegis_g2_font_build_l1;
                };
            };
            
            _aegis_g2_font_build_l1:
            
            ++w;
            this->width[c] = ((float)w / (float)chw) / 16.0f;
//            if (!(c%16)) printf("\n%02d ",w); else printf("%02d ",w);
        };
        
        // fix space characters
        this->width[' '] = this->width['_']; // space
        this->width[160] = this->width['_']; // non-breaking space
    };
 
    // get baseline by examining the '.' character
    int cx = ('.' % 16) * (sizex/16);
    int cy = ('.' / 16) * (sizey/16);
    int w;

    for (w = (sizey/16) - 1; w >= 0; --w) {
        for (int col = 0; col < (sizex/16); ++col) {
            int py = cy + w;
            int px = cx + col;
            
            if (alpha[py * sizex + px]) goto _aegis_g2_font_build_l9;
        };
    };
    
    _aegis_g2_font_build_l9:

    ++w;
    this->baseline = (float)w / (float)(sizey/16);
    
    // cleanup
    delete[] alpha;
        
    return;
};

__fastcall void Aegis::G2::Font::Assign( const Aegis::F::Bitmap & bm,
                                         bool fixedwidth ) {
    unsigned int sizex = bm.Width();
    unsigned int sizey = bm.Height();
    
    // if no powers of two, do not continue
    if (!(!(sizex&sizex-1) && !(sizey&sizey-1))) return;

    this->tex.SetParams( this->renderer, 0, 1, 0, 0 );
    this->tex.SetPriority( this->renderer, Aegis::G2::TexturePriorityAboveNormal );
    
    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA8, sizex, sizey,
                  0, GL_RGBA, GL_UNSIGNED_BYTE, bm.GetPointer() );

    // compute offsets
    
    
    // widths
    
    
    // baseline
    
    
    return;
};



__fastcall float Aegis::G2::Font::getwidth(const unsigned char c) {
    return 16.0f * width[c];
};

__fastcall float Aegis::G2::Font::getwidth(const char *sz) {
    float w = 0.0f;

    int len = strlen((const char*)sz);
    for (int i = 0; i < len; ++i) w += this->getwidth(sz[i]);
        
    return w;
};


__fastcall int Aegis::G2::Font::putc( float x, float y,
                                      float size, const unsigned char c ) {
    this->renderer.Textures->Enable(0,GL_TEXTURE_2D);
    this->renderer.Textures->Bind( this->tex.Name() );
    
    switch(this->mode) {
        default: case 0: y -= size*this->baseline; /*baseline mode*/ break;
        case 1: y = y; /*top mode*/ break;
        case 2: y -= size; /*bottom mode*/ break;
    };
    
    float w = this->getwidth(c);
    float x1 = x + size * w;
    float y1 = y + size;
    
    glNormal3f( 0.0f, 0.0f, 1.0f );
    glBegin(GL_QUADS);
        glTexCoord2f( this->offsetx[c], this->offsety[c] );
        glVertex2f( x, y );

        glTexCoord2f( this->offsetx[c] + this->width[c], this->offsety[c] );
        glVertex2f( x1, y );

        glTexCoord2f( this->offsetx[c] + this->width[c],
                      this->offsety[c] + 0.0625f );
        glVertex2f( x1, y1 );

        glTexCoord2f( this->offsetx[c], this->offsety[c] + 0.0625f );
        glVertex2f( x, y1 );
    glEnd();
    
    return 0;
};

__fastcall int Aegis::G2::Font::puts( float x, float y, 
                                      float size, const char *sz ){
    this->renderer.Textures->Push();
    this->renderer.Textures->Enable(0,GL_TEXTURE_2D);
    this->renderer.Textures->Bind( this->tex.Name() );
    this->renderer.Blend->Push();
    this->renderer.Blend->Enable();
    this->renderer.Blend->Default();
    
    int len = strlen(sz);
    for (int i = 0; i < len; ++i) {
        this->putc( x, y, size, sz[i] );
        x += size * this->getwidth(sz[i]);
    };

    this->renderer.Textures->Pop();
    this->renderer.Blend->Pop();
    
    return 0;
};

__cdecl int Aegis::G2::Font::Printf( float x, float y, float size,
                                     const char *fmt, ... ) {
    int zz = 0;
    if (fmt == NULL) return zz;

    char text[2048];

    VA_OPEN (ap, fmt);
        VA_FIXEDARG (ap, const char *, fmt);
        zz = vsnprintf (text, 2048, fmt, ap);
    VA_CLOSE (ap); 

    this->puts( x, y, size, text );
    
    return zz;
};

__fastcall void Aegis::G2::Font::SetMode(int mode) {
    this->mode = mode;
    return;
};
    


