/*
  Name: aG2_Material.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G2 - Material Class
*/

#ifndef _AG2_MATERIAL_HPP
#define _AG2_MATERIAL_HPP

#include <windows.h>
#include <gl/gl.h>

#include "../../Attribs.hpp"
#include "../G1/aG1_Renderer.hpp"
#include "../../M/M2/aM2_Color.hpp"

namespace Aegis {
    namespace G2 {
        
        struct __aligned Material {
            bool double_sided; // BACK materials are used
            
            // front
            Aegis::M2::Color ambient;
            Aegis::M2::Color diffuse;
            Aegis::M2::Color specular;
            Aegis::M2::Color emission;
            float            shininess; // 0.0f - 128.0f

            // back face (only used when double_sided)
            Aegis::M2::Color back_ambient;
            Aegis::M2::Color back_diffuse;
            Aegis::M2::Color back_specular;
            Aegis::M2::Color back_emission;
            float            back_shininess;
            
            // constructor
            inline Material() : double_sided(false),
                                ambient(0.2f, 0.2f, 0.2f, 1.0f),
                                diffuse(0.8f, 0.8f, 0.8f, 1.0f),
                                specular(0.0f, 0.0f, 0.0f, 1.0f),
                                emission(0.0f, 0.0f, 0.0f, 1.0f),
                                shininess(0.0f),
                                back_ambient(0.2f, 0.2f, 0.2f, 1.0f),
                                back_diffuse(0.8f, 0.8f, 0.8f, 1.0f),
                                back_specular(0.0f, 0.0f, 0.0f, 1.0f),
                                back_emission(0.0f, 0.0f, 0.0f, 1.0f),
                                back_shininess(0.0f) {
                 return;
            };
            
            // copyctor unneeded - binary copy is sufficient
            __fastcall void Correct();
            __fastcall void Apply( const Aegis::G1::Renderer& ren );
            inline void Apply( const Aegis::G1::Renderer * ren ) {
                this->Apply( *ren );
                return;
            };    

            
        };

        inline bool operator==( const Material& a, const Material& b ) {
            return ((a.double_sided == b.double_sided) &&
                    (a.ambient == b.ambient) &&
                    (a.diffuse == b.diffuse) &&
                    (a.specular == b.specular) &&
                    (a.emission == b.emission) &&
                    (a.back_ambient == b.back_ambient) &&
                    (a.back_diffuse == b.back_diffuse) &&
                    (a.back_specular == b.double_sided) &&
                    (a.back_emission == b.back_emission) &&
                    (a.shininess == b.shininess) &&
                    (a.back_shininess == b.back_shininess));
        };
        inline bool operator!=( const Material& a, const Material& b ) {
            return ((a.double_sided != b.double_sided) ||
                    (a.ambient != b.ambient) ||
                    (a.diffuse != b.diffuse) ||
                    (a.specular != b.specular) ||
                    (a.emission != b.emission) ||
                    (a.back_ambient != b.back_ambient) ||
                    (a.back_diffuse != b.back_diffuse) ||
                    (a.back_specular != b.double_sided) ||
                    (a.back_emission != b.back_emission) ||
                    (a.shininess != b.shininess) ||
                    (a.back_shininess != b.back_shininess));
        };
        
    };
};

#endif

