/*
  Name: aG2_MultiFace.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G2 - Multitextured Face
*/

#include "aG2_MultiFace.hpp"
#include <windows.h>

// DETAILS:
// - Only first 'renderer.Textures->UnitsNumber()' textures may have
//   GL_COMBINER_ARB environment mode, all these with such mode will
//   be ingored because there is no real-time way to simulate them.

__fastcall void Aegis::G2::MultiFace::Draw( const Aegis::G1::Renderer& renderer,
                                            bool ignorecombiners ) {
    int nmtx = renderer.Textures->UnitsNumber();
//    if (emulate) nmtx = 1;
    // nmtx is never more than AEGIS_G1_MULTITEXTURE which is 8 by now
    
    // colapse and optimize TexUnits into compact array - required
    Aegis::G2::TexUnit tu[AEGIS_G2_MULTIFACE_NTEX];

    int begin_with = AEGIS_G2_MULTIFACE_NTEX-1;
    for (;begin_with >= 0; --begin_with)
        if (this->texunit[begin_with].mode)
            if (this->texunit[begin_with].env_mode == GL_REPLACE)
                break; // we don't need to perform rendering that would be
                       // replaced after all

    if (begin_with < 0) begin_with = 0; // so no before-replace removals
    
    // perform colapsing here
    int maxcombiner = nmtx;
    if (!renderer.Ext->Is( Aegis::G1::TextureEnvCombine )) maxcombiner = 0;
    
    int number = 0; // number of textures
    for (int i = begin_with; i < AEGIS_G2_MULTIFACE_NTEX; ++i)
        if (this->texunit[i].mode) {
            if (this->texunit[i].env_mode == GL_COMBINE_ARB &&
                (number >= maxcombiner || ignorecombiners))
                continue; // Combiners here are unsupported or ignored
            else
                tu[ number++ ] = this->texunit[i]; // will work with this
        };
    
    // render all textures
    if (renderer.Fog->EmulateParametric() || number > nmtx) {
        renderer.Frame->Push();
        renderer.Frame->DepthMask(0);
    };    
    
    // first all-units single pass 
    if (nmtx == 1) {
        tu[0].Bind( renderer, 0 );
        glBegin(GL_TRIANGLES);
            glColor4fv   ( this->color [0].c );
            glNormal3fv  ( this->normal[0].v );
            glTexCoord4fv( this->coord [0][0].c );
            if (!renderer.Fog->EmulateParametric())
                renderer.Ext->glFogCoordfEXT( this->fog[0] );
            glVertex4fv  ( this->vertex[0].v );
            
            glColor4fv   ( this->color [1].c );
            glNormal3fv  ( this->normal[1].v );
            glTexCoord4fv( this->coord [0][1].c );
            if (!renderer.Fog->EmulateParametric())
                renderer.Ext->glFogCoordfEXT( this->fog[1] );
            glVertex4fv  ( this->vertex[1].v );
    
            glColor4fv   ( this->color [2].c );
            glNormal3fv  ( this->normal[2].v );
            glTexCoord4fv( this->coord [0][2].c );
            if (!renderer.Fog->EmulateParametric())
                renderer.Ext->glFogCoordfEXT( this->fog[2] );
            glVertex4fv  ( this->vertex[2].v );
        glEnd();
    } else {    
        int nt = nmtx <? number;
        // bind units
        for (int i = nt-1; i >= 0; --i) tu[i].Bind( renderer, i );
        
        glBegin( GL_TRIANGLES );
            glColor4fv   ( this->color[0].c );
            glNormal3fv  ( this->normal[0].v );
            for (int t = 0; t < nt; ++t)
                renderer.Ext->glMultiTexCoord4fvARB( GL_TEXTURE0_ARB + t,
                                                     this->coord[t][0].c );
            if (!renderer.Fog->EmulateParametric())
                renderer.Ext->glFogCoordfEXT( this->fog[0] );
            glVertex4fv  ( this->vertex[0].v );
            
            glColor4fv   ( this->color[1].c );
            glNormal3fv  ( this->normal[1].v );
            for (int t = 0; t < nt; ++t)
                renderer.Ext->glMultiTexCoord4fvARB( GL_TEXTURE0_ARB + t,
                                                     this->coord[t][1].c );
            if (!renderer.Fog->EmulateParametric())
                renderer.Ext->glFogCoordfEXT( this->fog[1] );
            glVertex4fv  ( this->vertex[1].v );
    
            glColor4fv   ( this->color[2].c );
            glNormal3fv  ( this->normal[2].v );
            for (int t = 0; t < nt; ++t)
                renderer.Ext->glMultiTexCoord4fvARB( GL_TEXTURE0_ARB + t,
                                                     this->coord[t][2].c );
            if (!renderer.Fog->EmulateParametric())
                renderer.Ext->glFogCoordfEXT( this->fog[0] );
            glVertex4fv  ( this->vertex[2].v );
        glEnd();
    };
    //*/
    renderer.Blend->Push();
    
    renderer.Textures->DisableAll();
    // overblend multipas
    for (int i = nmtx; i < number; ++i) {
        // determine what can be rendered in next single pass
        int first = i, n = 1;
        
        // TODO: todo :-]
        
        // restore depth mask if this is last pass
        if (first + n == number &&
            !renderer.Fog->EmulateParametric()) 
                renderer.Frame->Pop();
        
        // render
        if (n == 1) {
            // render one texture with glTexCoord
            tu[i].SetAccordingBlend( renderer );
            // and make it exact by setting "flat-splat" replace mode :-)))
            tu[i].env_mode = GL_REPLACE; // To make it Blend MP compatible.
            tu[i].Bind( renderer, 0 );

            glBegin(GL_TRIANGLES);
                glNormal3fv  ( this->normal[0].v );
                glTexCoord4fv( this->coord [i][0].c );
                glVertex4fv  ( this->vertex[0].v );
                
                glNormal3fv  ( this->normal[1].v );
                glTexCoord4fv( this->coord [i][1].c );
                glVertex4fv  ( this->vertex[1].v );
        
                glNormal3fv  ( this->normal[2].v );
                glTexCoord4fv( this->coord [i][2].c );
                glVertex4fv  ( this->vertex[2].v );
            glEnd();
            
        } else {
            // render N textures with glMultiTexCoord

            
        };
        
    };
    
    // render emulated parametric fog, if needed
    if (renderer.Fog->EmulateParametric()) {
        renderer.Frame->Pop();
        
        renderer.Blend->Default();
        renderer.Textures->Push();   // TODO: FIXME: Do we really need this?
        renderer.Textures->DisableAll();
        
        // render fog
        Aegis::M2::Color fc = renderer.Fog->GetColor();
        Aegis::M2::Color f1(fc), f2(fc), f3(fc);
        f1.a = renderer.Fog->ComputeFakeParametricAlpha( this->fog[0] );
        f2.a = renderer.Fog->ComputeFakeParametricAlpha( this->fog[1] );
        f3.a = renderer.Fog->ComputeFakeParametricAlpha( this->fog[2] );

        glBegin(GL_TRIANGLES);
            glColor4fv   ( f1.c );
            glNormal3fv  ( this->normal[0].v );
            glVertex4fv  ( this->vertex[0].v );
            
            glColor4fv   ( f2.c );
            glNormal3fv  ( this->normal[1].v );
            glVertex4fv  ( this->vertex[1].v );
    
            glColor4fv   ( f3.c );
            glNormal3fv  ( this->normal[2].v );
            glVertex4fv  ( this->vertex[2].v );
        glEnd();        
        
        renderer.Textures->Pop();
    };

    renderer.Blend->Pop();

    return;
};    

