/*
  Name: aG2_Sprite.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G2 - Sprite Class
*/

#include "aG2_Sprite.hpp"
#include "../G1/aG1_EffectiveGL.hpp"

__fastcall void Aegis::G2::Sprite::Draw(
                const Aegis::G1::Renderer& renderer ) const {
    Aegis::G1::PushMatrixObjectf PMO; // will POP at the end of the function
    
    glTranslatef( this->position.x, this->position.y, this->position.z );
    
    // compute billboard
    float modelview[16];
	glGetFloatv(GL_MODELVIEW_MATRIX, modelview);
    	for( int i=0; i<3; ++i )  for( int j=0; j<3; ++j )
    			if ( i==j ) modelview[(i<<2)+j] = 1.0f;
    			       else modelview[(i<<2)+j] = 0.0f;
//        modelview[12+0] += this->position.x;
//        modelview[12+1] += this->position.y;
//        modelview[12+2] += this->position.z;
	glLoadMatrixf(modelview);

    if (this->texture) this->texture->Bind( renderer );

    glColor4fv( this->color.c );
    glNormal3f( 0.0f, 0.0f, 1.0f );

    if (renderer.Fog->EmulateParametric()) {
        Aegis::M2::Color fogclr = renderer.Fog->GetColor();
        fogclr.a = renderer.Fog->ComputeFakeParametricAlpha( this->fog );

        renderer.Frame->Push();
        renderer.Frame->DepthMask(0);
        
        glBegin(GL_QUADS);
            glTexCoord2f( 0.0f, 0.0f ); glVertex2f( -this->size,  this->size );
            glTexCoord2f( 0.0f, 1.0f ); glVertex2f( -this->size, -this->size );
            glTexCoord2f( 1.0f, 1.0f ); glVertex2f(  this->size, -this->size );
            glTexCoord2f( 1.0f, 0.0f ); glVertex2f(  this->size,  this->size );
        glEnd();

        renderer.Frame->Pop();

        renderer.Blend->Push();
        renderer.Blend->Default();

        glColor4fv( fogclr.c );
        glBegin(GL_QUADS);
            glVertex2f( -this->size,  this->size );
            glVertex2f( -this->size, -this->size );
            glVertex2f(  this->size, -this->size );
            glVertex2f(  this->size,  this->size );
        glEnd();

        renderer.Blend->Pop();

    } else {
        renderer.Ext->glFogCoordfEXT( this->fog );
        
        glBegin(GL_QUADS);
            glTexCoord2f( 0.0f, 0.0f ); glVertex2f( -this->size,  this->size );
            glTexCoord2f( 0.0f, 1.0f ); glVertex2f( -this->size, -this->size );
            glTexCoord2f( 1.0f, 1.0f ); glVertex2f(  this->size, -this->size );
            glTexCoord2f( 1.0f, 0.0f ); glVertex2f(  this->size,  this->size );
        glEnd();
    };

    return;
};    

