/*
  Name: aG2_TexCombiner.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G2 - TexCombiner Class
*/

#ifndef _AG2_TEXCOMBINER_HPP
#define _AG2_TEXCOMBINER_HPP

#include <gl/gl.h>
#include "../../Attribs.hpp"
#include "../G1/aG1_Renderer.hpp"

namespace Aegis {
    namespace G2 {
        
        struct __aligned __packed TexCombiner {
            unsigned int mode[16];
            bool non_default;
            
            TexCombiner() {
                non_default = false;
                
                mode[ 0] = GL_MODULATE;
                mode[ 1] = GL_MODULATE;
                mode[ 2] = GL_TEXTURE;
                mode[ 3] = GL_PREVIOUS_ARB;
                mode[ 4] = GL_CONSTANT_ARB;
                mode[ 5] = GL_TEXTURE;
                mode[ 6] = GL_PREVIOUS_ARB;
                mode[ 7] = GL_CONSTANT_ARB;
                mode[ 8] = GL_SRC_COLOR;
                mode[ 9] = GL_SRC_COLOR;
                mode[10] = GL_SRC_ALPHA;
                mode[11] = GL_SRC_ALPHA;
                mode[12] = GL_SRC_ALPHA;
                mode[13] = GL_SRC_ALPHA;
                mode[14] = 1;
                mode[15] = 1;
                return;
            };
           
            // operators
            inline unsigned int& operator[](const int n) {
                non_default = true; // change requested
                switch(n) {
                    case GL_COMBINE_RGB_ARB:    return mode[0];
                    case GL_COMBINE_ALPHA_ARB:  return mode[1];
                    case GL_SOURCE0_RGB_ARB:    return mode[2];
                    case GL_SOURCE1_RGB_ARB:    return mode[3];
                    case GL_SOURCE2_RGB_ARB:    return mode[4];
                    case GL_SOURCE0_ALPHA_ARB:  return mode[5];
                    case GL_SOURCE1_ALPHA_ARB:  return mode[6];
                    case GL_SOURCE2_ALPHA_ARB:  return mode[7];
                    case GL_OPERAND0_RGB_ARB:   return mode[8];
                    case GL_OPERAND1_RGB_ARB:   return mode[9];
                    case GL_OPERAND2_RGB_ARB:   return mode[10];
                    case GL_OPERAND0_ALPHA_ARB: return mode[11];
                    case GL_OPERAND1_ALPHA_ARB: return mode[12];
                    case GL_OPERAND2_ALPHA_ARB: return mode[13];
                    case GL_RGB_SCALE_ARB:      return mode[14];
                    case GL_ALPHA_SCALE:        return mode[15];
                };
                return mode[n];
            };
            inline const unsigned int& operator[](const int n) const {
                switch(n) {
                    case GL_COMBINE_RGB_ARB:    return mode[0];
                    case GL_COMBINE_ALPHA_ARB:  return mode[1];
                    case GL_SOURCE0_RGB_ARB:    return mode[2];
                    case GL_SOURCE1_RGB_ARB:    return mode[3];
                    case GL_SOURCE2_RGB_ARB:    return mode[4];
                    case GL_SOURCE0_ALPHA_ARB:  return mode[5];
                    case GL_SOURCE1_ALPHA_ARB:  return mode[6];
                    case GL_SOURCE2_ALPHA_ARB:  return mode[7];
                    case GL_OPERAND0_RGB_ARB:   return mode[8];
                    case GL_OPERAND1_RGB_ARB:   return mode[9];
                    case GL_OPERAND2_RGB_ARB:   return mode[10];
                    case GL_OPERAND0_ALPHA_ARB: return mode[11];
                    case GL_OPERAND1_ALPHA_ARB: return mode[12];
                    case GL_OPERAND2_ALPHA_ARB: return mode[13];
                    case GL_RGB_SCALE_ARB:      return mode[14];
                    case GL_ALPHA_SCALE:        return mode[15];
                };
                return mode[n];
            };

            // some modification functions
            
            // glTexEnv application
            __fastcall void Apply(); // may result in GL_INVALID_ENUM !!!
                                     // but is a very little faster
            __fastcall void Apply( const Aegis::G1::Renderer& ren );
            // correction of non_default flag
            __fastcall void Correct();
        };
        
        // operators
        inline bool operator==( const TexCombiner& a, const TexCombiner& b ) {
            return ((a.mode[ 0] == b.mode[ 0]) &&
                    (a.mode[ 1] == b.mode[ 1]) &&
                    (a.mode[ 2] == b.mode[ 2]) &&
                    (a.mode[ 3] == b.mode[ 3]) &&
                    (a.mode[ 4] == b.mode[ 4]) &&
                    (a.mode[ 5] == b.mode[ 5]) &&
                    (a.mode[ 6] == b.mode[ 6]) &&
                    (a.mode[ 7] == b.mode[ 7]) &&
                    (a.mode[ 8] == b.mode[ 8]) &&
                    (a.mode[ 9] == b.mode[ 9]) &&
                    (a.mode[10] == b.mode[10]) &&
                    (a.mode[11] == b.mode[11]) &&
                    (a.mode[12] == b.mode[12]) &&
                    (a.mode[13] == b.mode[13]) &&
                    (a.mode[14] == b.mode[14]) &&
                    (a.mode[15] == b.mode[15]));
        };
        inline bool operator!=( const TexCombiner& a, const TexCombiner& b ) {
            return ((a.mode[ 0] != b.mode[ 0]) ||
                    (a.mode[ 1] != b.mode[ 1]) ||
                    (a.mode[ 2] != b.mode[ 2]) ||
                    (a.mode[ 3] != b.mode[ 3]) ||
                    (a.mode[ 4] != b.mode[ 4]) ||
                    (a.mode[ 5] != b.mode[ 5]) ||
                    (a.mode[ 6] != b.mode[ 6]) ||
                    (a.mode[ 7] != b.mode[ 7]) ||
                    (a.mode[ 8] != b.mode[ 8]) ||
                    (a.mode[ 9] != b.mode[ 9]) ||
                    (a.mode[10] != b.mode[10]) ||
                    (a.mode[11] != b.mode[11]) ||
                    (a.mode[12] != b.mode[12]) ||
                    (a.mode[13] != b.mode[13]) ||
                    (a.mode[14] != b.mode[14]) ||
                    (a.mode[15] != b.mode[15]));
        };
    };
};

#endif

