/*
  Name: aG2_TexUnit.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G2 - TexUnit Class
*/

#include "aG2_TexUnit.hpp"

__fastcall void Aegis::G2::TexUnit::Bind(const Aegis::G1::Renderer& rn,
                                         unsigned int unit) {
    if (!this->mode) {
        // structure invalid, means no texturing, disable it
        rn.Textures->Disable( unit );
        
    } else {
        rn.Textures->Current( unit );
        // bind texture
        if (this->texture) {
            // set according mode
            rn.Textures->Enable( unit, this->mode );
            
            // bind this textrure
            this->texture->Bind(rn);
        };
    
        // apply environment
        rn.Textures->TexEnv( this->env_mode );
        rn.Textures->EnvColor( this->env_color );
        
        // apply combiner
        if (this->env_combiner && this->env_mode == GL_COMBINE_ARB)
            this->env_combiner->Apply();
            
    };
    
    return;
};    

__fastcall void
Aegis::G2::TexUnit::SetAccordingBlend(const Aegis::G1::Renderer& rn) {
    switch( this->env_mode ) {
        default:
        case 0:
            rn.Blend->Disable();
//            rn.Blend->Void(); // TODO: FIXME: Wouldn't this be better?
                                // Or fall back to MODULATE?
                                // Is this block required at all?
        break;
        
        case GL_MODULATE:
            rn.Blend->Enable(); // TODO: CONSIDER: Is this required?
            rn.Blend->Modulate();
        break;
        case GL_DECAL:
            rn.Blend->Enable(); // TODO: CONSIDER: Is this required?
            rn.Blend->Decal();
        break;
        case GL_BLEND:
            rn.Blend->Enable(); // TODO: CONSIDER: Is this required?
            rn.Blend->Blend();
        break;
        case GL_REPLACE:
            rn.Blend->Enable(); // TODO: CONSIDER: Is this required?
            rn.Blend->Replace();
        break;
        case GL_ADD:
            rn.Blend->Enable(); // TODO: CONSIDER: Is this required?
            rn.Blend->Add();
        break;
/*      case GL_COMBINE_ARB:
            // Who do you think I am? Magician?
            // It would be nice if anybody could fake this.
        break;//*/
        
    };
    return;
};

