/*
  Name: aG2_TexUnit.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G2 - TexUnit Class
*/

#ifndef _AG2_TEXUNIT_HPP
#define _AG2_TEXUNIT_HPP

#include <windows.h>
#include <gl/gl.h>

namespace Aegis {
    namespace G2 {
        
        struct TexUnit;
        
    };
};

#include "../../Attribs.hpp"
#include "../G1/aG1_Renderer.hpp"
#include "aG2_Texture.hpp"
#include "aG2_TexCombiner.hpp"
#include "../../M/M2/aM2_Color.hpp"

namespace Aegis {
    namespace G2 {
        
        struct __aligned __packed TexUnit {
            unsigned int            mode;
            // GL_TEXTURE_1D, GL_TEXTURE_2D, GL_TEXTURE_3D, GL_TEXTURE_4D
            // or GL_TEXTURE_CUBE_MAP_ARB; 
            // 0 means textures is unused and thus no texturing to be used
            
            Aegis::G2::Texture      *texture;
            // If mode == 0 then unused, else it cannot be NULL
            // TexeLine and CubeMap (and Texture3D, Texture4D)
            // are just derived from Texture...
            
            unsigned int            env_mode;
            // GL_MODULATE, GL_DECAL, GL_BLEND, GL_REPLACE ???
            // GL_ADD, GL_COMBINE_ARB (combiner structure is then used)
            
            Aegis::M2::Color        env_color;
            // environment color

            Aegis::G2::TexCombiner  *env_combiner;
            // used when tex_env_mode is GL_COMBINE_ARB
            // shouldn't it be rather pointer???

            inline TexUnit() {
                this->mode = 0;
                this->texture = NULL;
                this->env_mode = GL_MODULATE;
                this->env_color.a = 0.0f; // needed by gl spec
                return;
            };
            inline TexUnit( const TexUnit& t ) : env_color(t.env_color),
                                                 env_combiner(t.env_combiner) {
                this->mode = t.mode;
                this->texture = t.texture;
                this->env_mode = t.env_mode;
                return;
            };

            inline ~TexUnit() { return; };
            
            __fastcall void Bind(const Aegis::G1::Renderer& rn, 
                                 unsigned int unit = 0xFFFFFFFF);
            // Set all values to OpenGL (BindTexture, TexEnv, ...) but only if
            // they differs from those already set (cached).
            // unit - target texture unit (0 - maximum)
            //      - maximum is GL_MAX_TEXTURE_UNITS_ARB
            
            __fastcall void SetAccordingBlend(const Aegis::G1::Renderer& rn);
        };
        
    };
};            

#endif

