/*
  Name: aG2_Texture.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - G (Graphics) Subsystem - G2 - Texture Class
*/

#ifndef _AG2_TEXTURE_HPP
#define _AG2_TEXTURE_HPP

#include <gl/gl.h>

namespace Aegis {
    namespace G2 {
        
        class Texture;
        
    };
};

#include "../G1/aG1_Renderer.hpp"
#include "../../F/aF_Bitmap.hpp"
//#include "../../F/aF_TexMap.hpp"
//#include "../../F/aF_Glyph.hpp"

namespace Aegis {
    namespace G2 {
        // these constants can be used when specifying texture priority
        extern const float TexturePriorityLowest;
        extern const float TexturePriorityLow;
        extern const float TexturePriorityBelowNormal;
        extern const float TexturePriorityNormal;
        extern const float TexturePriorityAboveNormal;
        extern const float TexturePriorityHigh;
        extern const float TexturePriorityCritical;
        extern const float TexturePriorityHighest;
        
        class Texture {
            private:
                unsigned int texture;
                
            public:
                __fastcall Texture();
                __fastcall ~Texture();
                
                inline unsigned int Name() { return texture; };
                
                // create texture from bitmap
                inline Texture( const Aegis::G1::Renderer & rndr,
                                const Aegis::F::Bitmap & b ) {
                    this->Build( rndr, b );
                    return;
                };

                // create bitmap from texture
                virtual __fastcall Aegis::F::Bitmap*
                MakeBitmap(const Aegis::G1::Renderer &, int mipmap = 0) const;
                
                virtual
                __fastcall void SetParams( const Aegis::G1::Renderer &,
                                           bool mipmap = true,
                                           int filter = 1, 
                                           float anisotropic = 1.0f, // none
                                           bool repeat = true );
                virtual
                __fastcall void SetPriority( const Aegis::G1::Renderer &,
                                             float priority );
                // priority: (0.0 - 1.0)
                // or you can use one of predefined values (TexturePriorityXXX)
                //  0.0 - probably won't be loaded into graphics memory
                //  1.0 - highly probable to be load into graphics memory
                //  0.4 - default Aegis::G2::Texture priority

                virtual 
                __fastcall int Build( const Aegis::G1::Renderer &,
                                      const Aegis::F::Bitmap &,
                                      bool mipmap = true, int filter = 1,
                                      float aniso = 1.0f, bool repeat = true );
                virtual 
                __fastcall int BuildAlphaMap( const Aegis::G1::Renderer &,
                                              const Aegis::F::Bitmap &,
                                              bool mipmap = true, int filter = 1,
                                              float aniso = 1.0f,
                                              bool repeat = true );
                virtual 
                __fastcall int BuildLuminanceAlphaMap( const Aegis::G1::Renderer &,
                                                       const Aegis::F::Bitmap &,
                                                       bool mipmap = true,
                                                       int filter = 1,
                                                       float aniso = 1.0f,
                                                       bool repeat = true );
                                           
                                           
                // filter:
                // 0 - mipmap ? nearest_nearest : nearest  , mag: nearest
                // 1 - mipmap ? linear_nearest  : linear   , mag: linear
                // 2 - mipmap ? linear_linear   : linear   , mag: linear
                
                inline void Bind( const Aegis::G1::Renderer & ren ) const {
                    ren.Textures->Bind( this->texture );
                    return;
                };    
        };    
    };
};
    
#endif
        
