/*
  Name: aM1_FastMath.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - M (Math) Subsystem - M1 - Set of fast math functions
*/

#ifndef _AM1_FASTMATH_HPP
#define _AM1_FASTMATH_HPP

#include "../../Attribs.hpp"

namespace Aegis {
    namespace M1 {
//        typedef float(* FF_Func)(float);
//        typedef int  (* IF_Func)(float);
        
        __fastcall bool FastMathEnable( bool enabled );
        
        // ret = sqrtf(f);
        // Very fast.
        __fastcall float sqrt( float f ) __pure;

        // NOTE sin and asin are NOT implemented because of need to call
        // cos and acos. Implementation of Aegis::M1 should recompute
        // values that vould be passed to sin so it can pass them to cos.

        // ret = cosf(f);
        // Not too fast (~50% faster).
        __fastcall float cos ( float f ) __pure;
        
        // ret = acosf(f);
        // Very fast (4 times faster than acosf)
        __fastcall float acos( float f ) __pure; 
        
        // ret = 1.0f/f;
        // WARNING: INV may be VERY inaccurate (+- 0.2)
        __fastcall float  inv( float f ) __pure;

        // ret = (int) floatnumber;
        __fastcall int   ftoi( float f ) __pure;
        
    };
};

#endif        

