/*
  Name: aM1_Random.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - M (Math) Subsystem - M1 - The Best Randomizer
*/

#include "aM1_Random.hpp"

Aegis::M1::Random random; // global randomizer

__fastcall Aegis::M1::Random::Random(int seed1, int seed2) {
    this->Randomize( (seed1 << 16) | seed2 );
   
    return;
};

__fastcall void Aegis::M1::Random::Randomize(int seed) {
    int seed1 = (seed & 0xFFFF0000) >> 16;
    int seed2 = (seed & 0x0000FFFF);

    seed1 %= 31329;
    seed2 %= 30082;

	int i, j, k, l, ii, jj, m;
	float s, t;

	i = (seed1/177)%177 + 2;
	j = seed1%177 + 2;
	k = (seed2/169)%178 + 1;
	l = seed2%169;
	
	for (ii=1; ii<=97; ++ii) {
		s = 0.0f;
		t = 0.5f;
		for (jj=1; jj<=24; ++jj) {
			m = (((i*j)%179)*k) % 179;
			i = j;
			j = k;
			k = m;
			l = (53*l + 1) % 169;
			if ((l*m)%64 >= 32) s += t;
			t *= 0.5f;
		}
		this->u[ii] = s;
	}
	
	this->c = 362436.0f / 16777216.0f;
	this->cd = 7654321.0f / 16777216.0f;
	this->cm = 16777213.0f / 16777216.0f;
	
	this->i97 = 97;
	this->j97 = 33;
    
    return;
};

__fastcall Aegis::M1::Random::operator float() {
	float uni;
	
	uni = this->u[i97] - this->u[j97];
	if (uni < 0.0f) uni += 1.0f;

	this->u[this->i97] = uni;

	--this->i97; if (!this->i97) this->i97 = 97;
	--this->j97; if (!this->j97) this->j97 = 97;

	this->c -= this->cd;
	if (this->c < 0.0f) this->c += this->cm;

	uni -= this->c;
	if (uni < 0.0f) uni += 1.0f;
    
    return uni;
};

