/*
  Name: aM2_Sense.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - M (Math) Subsystem - M2 - Sense (Position) Class
*/

#include "aM2_Sense.hpp"
#include <gl/gl.h>

__fastcall void Aegis::M2::Sense::RotateX( float angle ) {
    
    this->Direction.RotateX( angle );
    this->UpVector.RotateX ( angle );

    return;
};    

__fastcall void Aegis::M2::Sense::RotateY( float angle ) {

    this->Direction.RotateY( angle );
    this->UpVector.RotateY ( angle );

    return;
};    

__fastcall void Aegis::M2::Sense::RotateZ( float angle ) {
    
    this->Direction.RotateZ( angle );
    this->UpVector.RotateZ ( angle );

    return;
};    

__fastcall void Aegis::M2::Sense::Rotate( const Normal& n, float angle ) {
    
    this->Direction.Rotate( n, angle );
    this->UpVector.Rotate ( n, angle );

    return;
};    

__fastcall void Aegis::M2::Sense::ApplyMatrix() const {
    if (this->Direction == this->UpVector) return;

    Normal Z = this->Direction.Normalized();
    
    Normal X = Cross( Z, this->UpVector.Normalized() );
    Normal Y = Cross( X, Z );
    
    X.Normalize();
    Y.Normalize();
    
    float m[16] = {  X.x, X.y, X.z, 0.0,
                     Y.x, Y.y, Y.z, 0.0,
                    -Z.x,-Z.y,-Z.z, 0.0,
                    this->Position.x, this->Position.y, this->Position.z, 1.0,
                  };//*/

    glMultMatrixf( m );

    return;
};    

__fastcall void Aegis::M2::Sense::ApplyNegativeMatrix() const {
    if (this->Direction == this->UpVector) return;

    Normal Z = this->Direction.Normalized();
    
    Normal X = Cross( Z, this->UpVector.Normalized() );
    Normal Y = Cross( X, Z );
    
    X.Normalize();
    Y.Normalize();
    
    float m[16] = { -X.x,-X.y,-X.z, 0.0,
                    -Y.x,-Y.y,-Y.z, 0.0,
                     Z.x, Z.y, Z.z, 0.0,
                    this->Position.x, this->Position.y, this->Position.z, 1.0,
                  };//*/
    
    glMultMatrixf( m );

    return;
};

__fastcall float 
Aegis::M2::Sense::GetPointDistance(const Aegis::M2::Vertex & point) const {
    Aegis::M2::Normal nn = Aegis::M2::Normal(point)
                         - Aegis::M2::Normal(this->Position);
    
    return Dot( nn, this->Direction.Normalized() );    
};

__fastcall bool 
Aegis::M2::Sense::IsIntersectedByLine(const Aegis::M2::Vertex & v1,
                                      const Aegis::M2::Vertex & v2) const {
    float distance1 = this->GetPointDistance( v1 );
    float distance2 = this->GetPointDistance( v2 );
    
    return ((distance1 * distance2) <= 0.0f);
};


__fastcall Aegis::M2::Vertex 
Aegis::M2::Sense::GetLineIntersectionPoint(const Vertex& v1,
                                           const Vertex& v2) const {
    Aegis::M2::Normal lineDir( v1, v2 );
    lineDir.Normalize();
    
    float numerator = this->GetPointDistance( v1 );
    float denominator = Aegis::M2::Dot( this->Direction, lineDir );
    
    if (denominator == 0.0f) return v1;

    float dist = numerator / denominator;
    return Aegis::M2::Vertex( (float)(v1.x + (lineDir.x * dist)),
                              (float)(v1.y + (lineDir.y * dist)),
                              (float)(v1.z + (lineDir.z * dist)) );
};

