/*
  Name: aM2_Sense.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - M (Math) Subsystem - M2 - Sense (Position) Class
*/

#ifndef _AM2_SENSE_HPP
#define _AM2_SENSE_HPP

namespace Aegis {
    namespace M2 {
        
        struct Sense;
        
    };
};

#include "../../Attribs.hpp"
#include "aM2_Vertex.hpp"
#include "aM2_Normal.hpp"

namespace Aegis {
    namespace M2 {
        
        struct __aligned __packed Sense {
            // data
            Vertex Position;
            Normal Direction, UpVector;
            
            // methods
            inline Sense() : Position ( 0.0f, 0.0f, 0.0f ),
                             Direction( 0.0f, 0.0f,-1.0f ),
                             UpVector ( 0.0f, 1.0f, 0.0f ) { };
            inline Sense( const Vertex& pos, const Normal& dir, const Normal& up )
                        : Position (pos), Direction(dir), UpVector(up) { };
            inline Sense( const Sense& s ) : 
                          Position ( s.Position ),
                          Direction( s.Direction ),
                          UpVector ( s.UpVector ) { };
            
        	__fastcall void RotateX( float angle );
        	__fastcall void RotateY( float angle );
        	__fastcall void RotateZ( float angle );
        	__fastcall void Rotate( const Normal& axis, float angle );

        	__fastcall void ApplyMatrix() const; // OpenGL call
        	__fastcall void ApplyNegativeMatrix() const; // OpenGL call
        	
        	// sense as a plane functions
            __fastcall float GetPointDistance(const Aegis::M2::Vertex&) const;
            inline bool IsPointInFront(const Aegis::M2::Vertex & v) const {
                return this->GetPointDistance(v) > 0.0f;
            };

            // collision function
            __fastcall bool IsIntersectedByLine(const Aegis::M2::Vertex & v1,
                                         const Aegis::M2::Vertex & v2) const;
            __fastcall Vertex GetLineIntersectionPoint(const Vertex& v1,
                                                       const Vertex& v2) const;

        };
        
        typedef Sense Plane;
        
    };
};

#endif

