/*
  Name: aM2_TexCoord.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - M (Math) Subsystem - M2 - TexCoord Class
*/

#ifndef _AM2_TEXCOORD_HPP
#define _AM2_TEXCOORD_HPP

#include "../../Attribs.hpp"

namespace Aegis {
    namespace M2 {
        
        struct __aligned __packed TexCoord {
            union {
                struct {
                    float s;
                    float t;
                    float r;
                    float q;
                };
                float c[4];
            };
            
            TexCoord() : s(0.0f), t(0.0f), r(0.0f), q(1.0f) { };
            inline TexCoord( float is, float it = 0.0f,
                             float ir = 0.0f, float iq = 1.0f ) :
                             s(is), t(it), r(ir), q(iq) { };
            inline TexCoord( const TexCoord& v ) :
                             s(v.s), t(v.t), r(v.r), q(v.q) { };
            // no need for copyctor, binary copy is sufficient
            
            // access operators, non checked boundaries
            inline float& operator[](const int n) { return c[n]; };
            inline const float& operator[](const int n) const { return c[n]; };
            
            // some modification functions
        };
        
        
    };
};

#endif

