/*
  Name: aM_Vertex.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - M (Math) Subsystem - Vertex Class
*/

#include <math.h>

#include "aM2_Vertex.hpp"
#include "../M1/aM1_FastMath.hpp"
#include "aM2_Normal.hpp"

__fastcall void Aegis::M2::Vertex::Translate( float off_x, float off_y,
                                              float off_z, float off_w ) {
    this->x += off_x;
    this->y += off_y;
    this->z += off_z;
    this->w += off_w;
    
    return;
};

__fastcall void Aegis::M2::Vertex::AdjustWeight() {

    this->x *= this->w;
    this->y *= this->w;
    this->z *= this->w;
    this->w = 1.0;
    
    return;
};    

__fastcall void Aegis::M2::Vertex::ClosestPointOnLine( const Vertex & v1,
                                                       const Vertex & v2,
                                                       const Vertex & pt) {
	Aegis::M2::Normal vVector1 = (Aegis::M2::Normal)pt - (Aegis::M2::Normal)v1;
    Aegis::M2::Normal vVector2 = (Aegis::M2::Normal)v2 - (Aegis::M2::Normal)v1;
    vVector2.Normalize();

    float t = Aegis::M2::Dot(vVector2, vVector1);
    if (t <= 0.0f) {
        *this = v1;
        return;
    };

    float d = Aegis::M2::Distance(v2, v1);
    if (t >= d) {
        *this = v2;
        return;
    };

    vVector2 *= t;
    Aegis::M2::Normal vClosestPoint = (Aegis::M2::Normal)v1 + vVector2;
    *this = vClosestPoint;

	return;
};

// non member functions
__fastcall float Aegis::M2::Distance( const Vertex& v1, const Vertex& v2 ) {
    
	return Aegis::M1::sqrt( (v2.x - v1.x) * (v2.x - v1.x) +
						    (v2.y - v1.y) * (v2.y - v1.y) +
						    (v2.z - v1.z) * (v2.z - v1.z) );
};    

