/*
  Name: aM2_Vertex.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - M (Math) Subsystem - M2 - Vertex Class
*/

#ifndef _AM2_VERTEX_HPP
#define _AM2_VERTEX_HPP

namespace Aegis {
    namespace M2 {
        class Vertex;
    };
};

#include "../../Attribs.hpp"

namespace Aegis {
    namespace M2 {
        
        class __aligned __packed Vertex {
            public:
            
            union {
                struct {
                    float x;
                    float y;
                    float z;
                    float w; // weight, needs special treatment
                };
                float v[4];
            };
            
            Vertex() : x(0.0f), y(0.0f), z(0.0f), w(1.0f) { };
            inline Vertex( float ix, float iy, float iz = 0.0f, float iw = 1.0f ) :
                           x(ix), y(iy), z(iz), w(iw) { };
            inline Vertex( const Vertex& v ) :
                           x(v.x), y(v.y), z(v.z), w(v.w) { };

            // construction
            __fastcall void ClosestPointOnLine( const Vertex& l1, const Vertex& l2, const Vertex& pt);
            
            // access operators, non checked boundaries
            inline float& operator[](const int n) { return v[n]; };
            inline const float& operator[](const int n) const { return v[n]; };
            
            // some useful functions
            __fastcall void Translate( float off_x, float off_y,
                                       float off_z = 0.0f, float off_w = 0.0f );
            __fastcall void AdjustWeight();
        };
        
        inline Vertex operator+ ( const Vertex& a, const Vertex& b ) { return Vertex( a.x+b.x, a.y+b.y, a.z+b.z, a.w+b.w ); }
        inline Vertex operator- ( const Vertex& a, const Vertex& b ) { return Vertex( a.x-b.x, a.y-b.y, a.z-b.z, a.w-b.w ); }
        
        inline bool operator==( const Vertex& a, const Vertex& b ) { return ((a.x == b.x) && (a.y == b.y) && (a.z == b.z) && (a.w == b.w)); }
        inline bool operator!=( const Vertex& a, const Vertex& b ) { return ((a.x != b.x) || (a.y != b.y) || (a.z != b.z) || (a.w != b.w)); }

        __fastcall float Distance( const Vertex&, const Vertex& );
    };
};

#endif

