/*
  Name: aM3_isort.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - M (Math) Subsystem - M3 - Insertion Sort
*/

#include "aM3_isort.hpp"
#include <mem.h>

namespace Aegis {
    namespace M3 {
        
        void isort (void *array, size_t count, size_t size,
                    int(*compare)(const void*, const void*)) {
                        
            if (!array) return;
            if (!size) return;
            if (!compare) return;
            if (count < 2) return;
        
            char *ptr = (char*)array;
            
            // get last value (that should be inserted)
            char *item = (char*)malloc(size);
            if (!item) return;
            memcpy(item, &ptr[(count-1)*size], size);
            
            int offset = -1;
        
            // look for space to place
            for (int i = 0; i < count - 1; ++i) {
                int result = compare((void*) &ptr[size*i], (void*) item);
                
                if (result >= 0) {
                    offset = i;
                    break;
                }
            }
            
            if (offset == -1) // no sorting required
                return;
            
            memmove(&ptr[(offset+1)*size], &ptr[offset*size], size*(count-offset-1));
            memcpy(&ptr[offset*size], item, size);
            
            free(item);    
        
            return;
        };
    };
};        

