/*
  Name: aP1_Collision.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - P (Physics) Subsystem - P1 - Face collisions
*/

#include "aP1_Collision.hpp"

__fastcall bool Aegis::P1::IsInCollision( const Aegis::G2::Face & f,
                                          const Aegis::M2::Vertex & v ) {
    const double ACCURACY = 0.99;
    
    Aegis::M2::Normal pa( v, f.vertex[0] );
    Aegis::M2::Normal pb( v, f.vertex[1] );
    Aegis::M2::Normal pc( v, f.vertex[2] );

    float angle  = Angle( pa, pb);
          angle += Angle( pb, pc); // this way we can get collision
          angle += Angle( pc, pa); // with any convex polygon

    return (angle >= (ACCURACY * (2.0f * Aegis::M1::PI)) );
};

__fastcall bool Aegis::P1::IsInCollision( const Aegis::G2::Face & f,
                                          const Aegis::M2::Vertex & v1,
                                          const Aegis::M2::Vertex & v2 ) {
    Aegis::M2::Normal normal( f.vertex[0], f.vertex[2], f.vertex[1] );
    Aegis::M2::Sense plane( f.vertex[0], normal, normal ); // unused fake up-vector
    
    if (plane.IsIntersectedByLine( v1, v2 )) {
        Aegis::M2::Vertex vx = plane.GetLineIntersectionPoint( v1, v2 );
        if (Aegis::P1::IsInCollision( f, vx )) return true;
    };
    
    return false;
};

// TODO: FIMXE: Isn't there some better way to determine Face2Face collision?
__fastcall bool IsInCollision( const Aegis::G2::Face & f1,
                               const Aegis::G2::Face & f2 ) {
    // prepare data
    Aegis::M2::Normal n1( f1.vertex[0], f1.vertex[2], f1.vertex[1] );
    Aegis::M2::Normal n2( f2.vertex[0], f2.vertex[2], f2.vertex[1] );
    Aegis::M2::Sense p1( f1.vertex[0], n1, n1 ); // unused fake up-vector
    Aegis::M2::Sense p2( f2.vertex[0], n2, n2 ); // unused fake up-vector
    
    // check f1 against f2
    if (p2.IsIntersectedByLine( f1.vertex[0], f1.vertex[1] )) {
        Aegis::M2::Vertex vx =
            p2.GetLineIntersectionPoint( f1.vertex[0], f1.vertex[1] );
        if (Aegis::P1::IsInCollision( f2, vx )) return true;
    };
    if (p2.IsIntersectedByLine( f1.vertex[0], f1.vertex[2] )) {
        Aegis::M2::Vertex vx =
            p2.GetLineIntersectionPoint( f1.vertex[0], f1.vertex[2] );
        if (Aegis::P1::IsInCollision( f2, vx )) return true;
    };
    if (p2.IsIntersectedByLine( f1.vertex[1], f1.vertex[2] )) {
        Aegis::M2::Vertex vx =
            p2.GetLineIntersectionPoint( f1.vertex[1], f1.vertex[2] );
        if (Aegis::P1::IsInCollision( f2, vx )) return true;
    };
    
    // check f2 against f1
    if (p1.IsIntersectedByLine( f2.vertex[0], f2.vertex[1] )) {
        Aegis::M2::Vertex vx =
            p1.GetLineIntersectionPoint( f2.vertex[0], f2.vertex[1] );
        if (Aegis::P1::IsInCollision( f1, vx )) return true;
    };
    if (p1.IsIntersectedByLine( f2.vertex[0], f2.vertex[2] )) {
        Aegis::M2::Vertex vx =
            p1.GetLineIntersectionPoint( f2.vertex[0], f2.vertex[2] );
        if (Aegis::P1::IsInCollision( f1, vx )) return true;
    };
    if (p1.IsIntersectedByLine( f2.vertex[1], f2.vertex[2] )) {
        Aegis::M2::Vertex vx =
            p1.GetLineIntersectionPoint( f2.vertex[1], f2.vertex[2] );
        if (Aegis::P1::IsInCollision( f1, vx )) return true;
    };
    
    return false;
};    
                               
                               



