/*
  Name: aP1_Sphere.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - P (Physics) Subsystem - P1 - Bounding Sphere
*/

#include "aP1_Sphere.hpp"
#include "aP1_Collision.hpp"

__fastcall Aegis::P1::Sphere::Sphere( const Aegis::G2::Face &f ) {
    this->Position.x = (f.vertex[0].x + f.vertex[1].x + f.vertex[2].x) / 3.0f;
    this->Position.y = (f.vertex[0].y + f.vertex[1].y + f.vertex[2].y) / 3.0f;
    this->Position.z = (f.vertex[0].z + f.vertex[1].z + f.vertex[2].z) / 3.0f;
    
    this->Radius = fabsf( this->Position.x - f.vertex[0].x );
    
    return;
};

__fastcall bool Aegis::P1::Sphere::IsInCollision( const Sphere & s ) {
    float distance = Aegis::M2::Distance( this->Position, s.Position );
    float radiussum = this->Radius + s.Radius;
    
    if (distance <= radiussum) return true;
                          else return false;
};

__fastcall bool Aegis::P1::Sphere::IsInCollision( const Aegis::M2::Sense & s ) {
    return (this->Radius >= fabsf(s.GetPointDistance( this->Position )));
};

__fastcall bool Aegis::P1::Sphere::IsInCollision( const Aegis::M2::Vertex & v1,
                                                  const Aegis::M2::Vertex & v2 ) {
    Aegis::M2::Vertex pt;
    pt.ClosestPointOnLine( v1, v2, this->Position );
    
    float dist = Aegis::M2::Distance( pt, this->Position );
    return (dist <= this->Radius);
};

__fastcall bool Aegis::P1::Sphere::IsInCollision( const Aegis::G2::Face & f ) {
    Aegis::M2::Normal normal( f.vertex[0], f.vertex[1], f.vertex[2] );
    Aegis::M2::Sense plane( f.vertex[0], normal, normal ); // fake up-vector
    
    if (this->IsInCollision( plane )) {
        // ok, we are in collision with face's plane
        float dist = plane.GetPointDistance( this->Position );
        
        // center projected to plane
        Aegis::M2::Normal prj = Aegis::M2::Normal(this->Position)
                              - dist * normal;

		if (Aegis::P1::IsInCollision( f, Aegis::M2::Vertex(prj) )) return true;
		
		if (this->IsInCollision( f.vertex[0], f.vertex[1] )) return true;
		if (this->IsInCollision( f.vertex[0], f.vertex[2] )) return true;
		if (this->IsInCollision( f.vertex[1], f.vertex[2] )) return true;
    };
    
    return false;
};

__fastcall float Aegis::P1::Sphere::Distance( const Sphere & s ) {
    float distance = Aegis::M2::Distance( this->Position, s.Position );
    distance -= this->Radius + s.Radius;
    
    return distance;
};

__fastcall float Aegis::P1::Sphere::Distance( const Aegis::M2::Vertex & v ) {
    float distance = Aegis::M2::Distance( this->Position, v );
    distance -= this->Radius;
    
    return distance;
};





