/*
  Name: aP1_Sphere.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - P (Physics) Subsystem - P1 - Bounding Sphere
*/

#ifndef _AP1_SPHERE_HPP
#define _AP1_SPHERE_HPP

namespace Aegis {
    namespace P1 {
        
        struct Sphere;
        
    };
};

#include "../../Attribs.hpp"
#include "../../M/M1/aM1_FastMath.hpp"
#include "../../M/M2/aM2_Vertex.hpp"
#include "../../M/M2/aM2_Sense.hpp"

#include "../../G/G2/aG2_Face.hpp"
// TODO: Add elements and objects (object trees)

namespace Aegis {
    namespace P1 {
        
        struct __aligned __packed Sphere {
            // data
            Aegis::M2::Vertex Position;
            float Radius;
            
            // common functions
            Sphere() { Radius = 0.0f; };
            ~Sphere() { };
            
            // constructors
            // bounding sphere of single face
            __fastcall Sphere( const Aegis::G2::Face & );
            
            // collision function
            __fastcall bool IsInCollision( const Sphere & );
            // collision of sphere and plane (as sense)
            __fastcall bool IsInCollision( const Aegis::M2::Sense & );
            // here is collision of sphere with line
            __fastcall bool IsInCollision( const Aegis::M2::Vertex &,
                                           const Aegis::M2::Vertex & );
            // collision of sphere and face
            __fastcall bool IsInCollision( const Aegis::G2::Face & );

            
            // distance function
            __fastcall float Distance( const Sphere & );
            __fastcall float Distance( const Aegis::M2::Vertex & );
            
        };
        
    };
};

#endif

