/*
  Name: aW_Console.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - W (Window) Subsystem - Console Class
*/

#include "aW_Console.hpp"
#include "../C/aC_Keyboard.hpp"

#include <ansidecl.h>
#include <string.h>
#include <stdio.h>

Aegis::W::Console::Console(HWND hParent, char *szTitle) {
    if (!szTitle) szTitle = "Aegis Engine Console";
    
    WNDCLASSEXA wincl;
    HINSTANCE hInst = GetModuleHandle(NULL);

    this->hwnd = NULL;
    
    InitializeCriticalSection( &this->cs );
    if (!GetClassInfoExA( hInst, "Aegis::W::Console", &wincl )) {
        wincl.cbSize = sizeof (WNDCLASSEX);
        wincl.hInstance = hInst;
        wincl.lpszClassName = "Aegis::W::Console";
        wincl.lpfnWndProc = this->WndProc;

        wincl.style = 0;
        wincl.hIcon = LoadIcon (NULL, IDI_APPLICATION);
        wincl.hIconSm = LoadIcon (NULL, IDI_APPLICATION);
        wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
        wincl.lpszMenuName = NULL;

        wincl.cbClsExtra = 0;
        wincl.cbWndExtra = 0;
        wincl.hbrBackground = (HBRUSH)GetStockObject( BLACK_BRUSH );
    
        if (!RegisterClassExA (&wincl)) return;
    };
    
    DWORD style = WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_SYSMENU;
    DWORD exsty = WS_EX_CLIENTEDGE | WS_EX_TOOLWINDOW;
    RECT r = { 128, 128, 490, 260 };

    r.right += 2*GetSystemMetrics(SM_CXFIXEDFRAME)
             + 2*GetSystemMetrics(SM_CXEDGE);
    
    r.bottom += 2*GetSystemMetrics(SM_CYFIXEDFRAME)
              + 2*GetSystemMetrics(SM_CYEDGE)
              + GetSystemMetrics(SM_CYSMCAPTION);

    
    this->hwnd = CreateWindowExA( exsty, "Aegis::W::Console", szTitle,
                                  style, r.left, r.top, r.right, r.bottom,
                                  hParent, NULL, hInst, NULL );
    SetClassLong( this->hwnd, GCL_STYLE, 0x00020000 ); // shadow
                                  
    SetProp( this->hwnd, "Aegis::W::Console::This", this );
    
    LOGFONT lf = { 10,6,0,0,FW_NORMAL,0,0,0,DEFAULT_CHARSET,0,0,
                   PROOF_QUALITY/*NONANTIALIASED_QUALITY*/,
                   FF_MODERN, "Lucida Console" };
    this->hFont = CreateFontIndirect( &lf );

    for (int i = 0; i < 25; ++i) memset( this->data[i], ' ', 81 );
    memcpy( data[24], szTitle, strlen(szTitle)<?80 );
    this->putc('\f');
    
    this->cx = 0;
    this->cb = false;
    this->cr = 0;
    
    return;
};

Aegis::W::Console::~Console() {
    DeleteObject( this->hFont );
    DeleteCriticalSection( &this->cs );

    if (IsWindow(this->hwnd)) {
        RemoveProp( this->hwnd, "Aegis::W::Console::This" );
        DestroyWindow( this->hwnd );
    };
    
    return;
};

LRESULT CALLBACK
Aegis::W::Console::WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) {
    Aegis::W::Console::Console *This = (Aegis::W::Console::Console *)
        GetProp( hwnd, "Aegis::W::Console::This" );

    switch(msg) {
        case WM_CREATE:
//            SetTimer( hwnd, 1, 100, NULL );
        break;
        case WM_CLOSE:
            ShowWindow( hwnd, SW_HIDE );
        break;
        case WM_KEYDOWN: // the extra menu can do this too from main window
            if (Aegis::C::Keyboard::IsKeyDownNow( VK_CONTROL )
                && wParam == VK_F9) ShowWindow( hwnd, SW_HIDE );
        break;

        case WM_DESTROY:
            This->hwnd = NULL;
        break;
        
        case WM_SETFOCUS:
            CreateCaret(hwnd, NULL, 6, 1);
            SetCaretPos(5+6*This->cx, 255);
            ShowCaret(hwnd);
        break;
        case WM_KILLFOCUS:
            HideCaret(hwnd);
            DestroyCaret();
        break;
        
        case WM_PAINT: {
            if (!This) break;
            
            PAINTSTRUCT ps;
            HDC hDC = BeginPaint( hwnd, &ps );
            
            SelectObject(hDC, This->hFont);
            SetTextColor(hDC, 0x00FFFFFF);
            SetBkColor(hDC, 0x00000000);
            SetBkMode(hDC, OPAQUE);

            RECT dummy;
            for (int y = 0; y < 25; ++y) {
                RECT r = { 5, 5+10*y, 485, 5+10*(y+1) };
                if (IntersectRect( &dummy, &r, &ps.rcPaint )) {
//                    FillRect( hDC, &r, (HBRUSH)GetStockObject(BLACK_BRUSH));
                    TextOut( hDC, 5, 5+10*y, This->data[y], 80 );
                };
            };//*/
//*/
            
            EndPaint( hwnd, &ps );
        } break;    

        default:
            return DefWindowProc( hwnd, msg, wParam, lParam );
    };
    
    return 0;
};    

void Aegis::W::Console::Show() {
    if (this->hwnd)
        ShowWindow( this->hwnd, SW_SHOW );
    return;
};

void Aegis::W::Console::Hide() {
    if (this->hwnd)
        ShowWindow( this->hwnd, SW_HIDE );
    return;
};

__fastcall void Aegis::W::Console::clear() {
    EnterCriticalSection(&this->cs);
    memset( this->data, ' ', 25*81 );
    this->cx = 0;
    LeaveCriticalSection(&this->cs);

    if (this->hwnd)
        if (!this->hold)
            InvalidateRect( this->hwnd, NULL, FALSE );
    return;
};

__fastcall void Aegis::W::Console::clearline() {
    EnterCriticalSection(&this->cs);
    memset( this->data[24], ' ', 81 );
    this->cx = 0;
    LeaveCriticalSection(&this->cs);

    if (this->hwnd) {
        RECT r = { 5, 245, 485, 255 };
        if (!this->hold)
            InvalidateRect( this->hwnd, &r, FALSE );
    };
    return;
};

__fastcall void Aegis::W::Console::back() {
    this->cx = 0;

    if (this->hwnd) {
        RECT r = { 5, 245, 485, 255 };
        if (!this->hold)
            InvalidateRect( this->hwnd, &r, FALSE );
    };
    return;
};

__fastcall void Aegis::W::Console::putc(char c, bool seq) {
    EnterCriticalSection(&this->cs);
    
    if (c == '\t') { // horizontal tab
        int n = 4 - ( this->cx % 4 );
        for (int i = 0; i < n; ++i) this->putc(' ', seq);
    } else

    if (c == '\f') { // form feed
        this->putc('\n');
        for (int i = 0; i < 3; ++i) this->putc('-', true);
        this->putc('\n');
        this->putc('\n');
    } else

    if (c == '\v') { // vertical tab (like \r in cprintf)
        int oldcx = this->cx;
        this->putc('\n');
        this->cx = oldcx;
    } else

    if (c == '\a') { // alert
        this->putc('\n');
        for (int i = 0; i < 3; ++i) this->putc('-', true);
        this->printf(" Warning ");
        for (int i = 0; i < 3; ++i) this->putc('-', true);
        this->putc('\n');
    } else

    if (c == '\b') { // backspace
        if (this->cx) --this->cx;
    } else

/*    if (c == '\e') { // escape
        // TODO: Handle escaped console commands here
    } else //*/
        
    if (c == '\n') { // newline
        this->cx = 0;
        // roll text up (forget first line)
        for (int i = 0; i < 24; ++i)
            strncpy( this->data[i], this->data[i+1], 80 );

        // reset last line
        memset( this->data[24], ' ', 80 );
        
        // invalidate whole window
        if (this->hwnd) {
            if (!this->hold)
                InvalidateRect( this->hwnd, NULL, FALSE );
        };    
    } else
    
    if (c > 31) { // printable character
        if (this->hwnd) {
            RECT r = { 5 + 6*this->cx, 5 + 10*24,
                       11+ 6*this->cx, 15+ 10*24 };
            if (!seq && !this->hold)
                InvalidateRect( this->hwnd, &r, FALSE );
        };

        this->data[24][ this->cx ] = c;
        if (++this->cx == 80) this->putc('\n');
    
        if (this->hwnd) {
            RECT r1 = { 5 + 6*this->cx, 5 + 10*24,
                       11+ 6*this->cx, 15+ 10*24 };
            if (!seq && !this->hold)
                InvalidateRect( this->hwnd, &r1, FALSE );
        };
    };

    LeaveCriticalSection(&this->cs);
    return;
};

__cdecl int Aegis::W::Console::printf(const char *fmt, ...) {
    int zz = 0;
    if (fmt == NULL) return zz;

    char text[2048];

    VA_OPEN (ap, fmt);
        VA_FIXEDARG (ap, const char *, fmt);
        zz = vsnprintf (text, 2048, fmt, ap);
    VA_CLOSE (ap); 
    
    int len = strlen(text);
    
    EnterCriticalSection(&this->cs);
    for (int i = 0; i < len; ++i) this->putc( text[i], true );
    LeaveCriticalSection(&this->cs);
    
    return zz;
};

