/*
  Name: aW_ExtraMenu.cpp
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - W (Window) Subsystem - System Menu extension
*/

#include "aW_ExtraMenu.hpp"
#include "../C/aC_Keyboard.hpp"

__fastcall Aegis::W::ExtraMenu::ExtraMenu( Aegis::W::Window & _wnd )
                               : wnd(_wnd) {
    this->hMenu = GetSystemMenu( this->wnd, FALSE );
    
    // add menu items
    AppendMenu( this->hMenu, MF_SEPARATOR, 0, NULL );
    AppendMenu( this->hMenu, MF_STRING, 0x1A10,
                "Fullscreen mode\tCtrl+F12" );
    AppendMenu( this->hMenu, MF_STRING, 0x1A00,
                "Center window\tCtrl+F11" );
    AppendMenu( this->hMenu, MF_STRING, 0x1A20,
                "Debug console\tCtrl+F9" );
//    AppendMenu( this->hMenu, MF_STRING, 0x1AA0,
//                "FPS display\tCtrl+F1" );
    
    return;
};

__fastcall Aegis::W::ExtraMenu::~ExtraMenu() {
    GetSystemMenu( this->wnd, TRUE );
    return;
};

bool Aegis::W::ExtraMenu::ProcessMessage( HWND hwnd, UINT msg,
                                          WPARAM wParam, LPARAM lParam ) {
    switch(msg) {
        case WM_KEYDOWN: {
            if (Aegis::C::Keyboard::IsKeyDownNow( VK_CONTROL ))
            switch(wParam) {
                case VK_F9: {
                    // console
                    if (this->wnd.console->IsVisible())
                        this->wnd.console->Hide();
                    else
                        this->wnd.console->Show();
                } break;
                case VK_F11: {
                    this->wnd.Center();
                } break;
                case VK_F12: {
                    bool s = (bool) GetProp( this->wnd,
                                             "Aegis::W::Window::Fullscreen" );
                    this->wnd.Fullscreen( !s );
                } break;
            };
        } break;    
        
        // system menu commands
        case WM_SYSCOMMAND: {
            unsigned int cmd = 0xFFF0 & wParam;
            
            switch(cmd) {
                case 0x1A00: { // center
                    this->wnd.Center();
                } return true;
                
                case 0x1A10: { // full
                    this->wnd.Fullscreen(true);
                } return true;

                case 0x1A20: { // consolee
                    if (this->wnd.console->IsVisible())
                        this->wnd.console->Hide();
                    else
                        this->wnd.console->Show();
                } return true;
            };
        } break;    
    };
    
    return false;
};    

