/*
  Name: aW_Resolution.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - W (Window) Subsystem - Resolution Class
*/

#include "aW_Resolution.hpp"
#include <windows.h>
#include <stdlib.h>

#ifndef CDS_FULLSCREEN // Undefined in Mingw; PSDK: The mode is temporary in nature.
  #define CDS_FULLSCREEN      0x00000004
#endif


Aegis::W::Resolution::Resolution() { // light constructor
    Init();
    return;
};

Aegis::W::Resolution::Resolution(int x, int y, int bpp, int freq) { // heavy constructor
    Init();
    Set( x, y, bpp, freq );
    return;
};

Aegis::W::Resolution::~Resolution() { // destructor
    Restore();
    return;
};

void Aegis::W::Resolution::Init() {
    this->changed = false;

    return;
};

bool Aegis::W::Resolution::Set(int x, int y, int bpp, int freq) {
    DEVMODE dm;

    dm.dmSize = sizeof(DEVMODE);
    dm.dmPelsWidth = x;
    dm.dmPelsHeight = y;
    dm.dmBitsPerPel = bpp;
    dm.dmDisplayFrequency = freq;
    
    dm.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT;
    if (bpp) dm.dmFields |= DM_BITSPERPEL;
    if (freq) dm.dmFields |= DM_DISPLAYFREQUENCY;
    
    this->changed = ( ChangeDisplaySettings( &dm, CDS_FULLSCREEN )
                      == DISP_CHANGE_SUCCESSFUL );
    return this->changed;
};

bool Aegis::W::Resolution::Test(int x, int y, int bpp, int freq) {
    DEVMODE dm;

    dm.dmSize = sizeof(DEVMODE);
    dm.dmPelsWidth = x;
    dm.dmPelsHeight = y;
    dm.dmBitsPerPel = bpp;
    dm.dmDisplayFrequency = freq;
    
    dm.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT;
    if (bpp) dm.dmFields |= DM_BITSPERPEL;
    if (freq) dm.dmFields |= DM_DISPLAYFREQUENCY;
    
    return ( ChangeDisplaySettings( &dm, CDS_TEST | CDS_FULLSCREEN )
             == DISP_CHANGE_SUCCESSFUL );
};

bool Aegis::W::Resolution::Restore() {
    if (this->changed) {
        this->changed = false;
        return ( ChangeDisplaySettings( NULL, 0 ) == DISP_CHANGE_SUCCESSFUL );
    }
    return false;
};

bool Aegis::W::Resolution::Enum( int index, int &x, int &y, int &bpp, int &freq ) {
    DEVMODE dm;
    dm.dmSize = sizeof(DEVMODE);

    bool ret = EnumDisplaySettings( NULL, index, &dm );
    
    x = dm.dmPelsWidth;
    y = dm.dmPelsHeight;
    bpp = dm.dmBitsPerPel;
    freq = dm.dmDisplayFrequency;
    
    return ret;
};

