/*
  Name: aW_Window.CPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - W (Window) Subsystem - Window Class
*/

#include "aW_Window.hpp"
//#include <stdio.h>

Aegis::W::Window::Window() { // light constructor
    Create( "Aegis Engine Application", NULL, false, true );
    return;
};

Aegis::W::Window::Window( char *szAppName, char *szIcon,
                          bool fullscreen, bool single ) { 
    Create( szAppName, szIcon, fullscreen, single );
    return;
};

void Aegis::W::Window::Create( char *szAppName, char *szIcon,
                               bool fullscreen, bool single) { 
    // heavy constructor
    WNDCLASSEXA wincl;
    HINSTANCE hInst = GetModuleHandle(NULL);

    this->msgh_flag = false;
    this->hwnd = NULL;
    this->hDC = NULL;
    this->single = single;
    
    if (!GetClassInfoExA( hInst, "Aegis::W::Window", &wincl )) {
        wincl.cbSize = sizeof (WNDCLASSEX);
        wincl.hInstance = hInst;
        wincl.lpszClassName = "Aegis::W::Window";
        wincl.lpfnWndProc = this->WndProc;

        wincl.style = 0;
        
        if (!szIcon) {
            wincl.hIcon = LoadIcon (NULL, IDI_APPLICATION);
            wincl.hIconSm = LoadIcon (NULL, IDI_APPLICATION);
        } else {
            wincl.hIcon = (HICON)LoadImageA(hInst, szIcon, IMAGE_ICON, 32, 32, 0);
            wincl.hIconSm = (HICON)LoadImageA(hInst, szIcon, IMAGE_ICON, 16, 16, 0);
        };    
        
        wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
        wincl.lpszMenuName = NULL;

        wincl.cbClsExtra = 0;
        wincl.cbWndExtra = 0;
        wincl.hbrBackground = (HBRUSH)GetStockObject( BLACK_BRUSH );
//         GetSysColorBrush( COLOR_BACKGROUND );
    
        if (!RegisterClassExA (&wincl)) return;
    };
    
    this->hwnd = CreateWindowExA( 0, "Aegis::W::Window", szAppName,
                                  WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
                                  0, 0, 0, 0,
                                  HWND_DESKTOP, NULL, hInst, NULL );
    this->Fullscreen( fullscreen );
    this->hDC = GetDC( this->hwnd );

    SetProp( this->hwnd, "Aegis::W::Window::This", this );
    PostMessage( this->hwnd, 0x8000 - 16, 0, 0 );
    
    this->console = new Aegis::W::Console( this->hwnd );

    return;    
};

Aegis::W::Window::~Window() { // destructor
    if (IsWindow(this->hwnd)) {
        ReleaseDC( this->hwnd, this->hDC );
        SendMessage( this->hwnd, WM_CLOSE, 0, 0 ); // MUST be Send to synchro...
    };
    
    delete this->console; // console object must be available, even
                          // when its parent is already closed
    return;
};

int Aegis::W::Window::Init() { return 0; }
int Aegis::W::Window::Close() { return 0; }
int Aegis::W::Window::MessageHandler(HWND, UINT, WPARAM, LPARAM) { return 0; }
int Aegis::W::Window::FullMsgHandler(HWND, UINT, WPARAM, LPARAM) { return 0; }

LRESULT CALLBACK
Aegis::W::Window::WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) {
    Aegis::W::Window::Window *This
        = (Aegis::W::Window::Window *)GetProp( hwnd, "Aegis::W::Window::This" );

    if (This) {
        int ret1 = This->FullMsgHandler(hwnd, msg, wParam, lParam);
        if (ret1) return ret1;

        if (This->msgh_flag) {
            int ret2 = This->MessageHandler(hwnd, msg, wParam, lParam);
            if (ret2) return ret2;
        };
    };
    
    switch(msg) {
        case WM_CREATE:
            SetThreadPriority( GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL );
        break;
            
        case 0x8000-16:
            if (This) {
                This->Init();
                This->msgh_flag = true;
                
                RECT okno;
                GetClientRect( This->hwnd, &okno );
                PostMessage( This->hwnd, WM_SIZE, SIZE_RESTORED,
                             MAKELPARAM( okno.right, okno.bottom ));
            };
        break;

        case WM_CLOSE:
            SetThreadPriority( GetCurrentThread(), THREAD_PRIORITY_NORMAL );
            DestroyWindow(hwnd);
        break;
        
        case WM_DESTROY:
            if (This) {
                This->msgh_flag = false;
                This->Close();
                
                if (This->single) PostQuitMessage(0);
            }    
        break;

        default:
            return DefWindowProc( hwnd, msg, wParam, lParam );
    };
    
    return 0;
};    

bool Aegis::W::Window::Fullscreen(bool fs) {
    UINT style = GetWindowLong( this->hwnd, GWL_STYLE );
    
    if (fs) {
        style &= ~WS_OVERLAPPEDWINDOW;
        style |= WS_POPUP;
    } else {
        style &= ~WS_POPUP;
        style |= WS_OVERLAPPEDWINDOW;
    };
        
    SetWindowLong( this->hwnd, GWL_STYLE, style );
    SetProp( this->hwnd, "Aegis::W::Window::Fullscreen", (HANDLE)fs );
    
    this->Center( fs ? GetSystemMetrics( SM_CXSCREEN ) : 512,
                  fs ? GetSystemMetrics( SM_CYSCREEN ) : 384 );

    return 0;
};    
                
bool Aegis::W::Window::Center(int x, int y) {
    RECT okno;
    GetWindowRect( this->hwnd, &okno );
    
    int sirka = GetSystemMetrics(SM_CXSCREEN);
    int vyska = GetSystemMetrics(SM_CYSCREEN);
    int so = x ?: (okno.right - okno.left);
    int vo = y ?: (okno.bottom - okno.top);
    
    MoveWindow( this->hwnd, (sirka-so)>>1, (vyska-vo)>>1, so, vo, true);
    
    return 0;
};    

void Aegis::W::Window::CloseRequest() {
    PostMessage( this->hwnd, WM_CLOSE, 0, 0 );
    return;
};
