/*
  Name: aW_Window.HPP
  Author: Jan Ringo, Tringi@Mx-3.cz, http://Tringi.Mx-3.cz
  Description: Aegis Engine - W (Window) Subsystem - Window Class
*/

#ifndef _AW_WINDOW_HPP
#define _AW_WINDOW_HPP

#include <windows.h>
#include "aW_Console.hpp"

namespace Aegis {
    namespace W {
        class Window { // Window class
            private:
                HWND hwnd;
                HDC hDC;
                bool single;
                bool msgh_flag;
                
                void Create( char * szAppName, char * szIcon, // internal use
                             bool fullscreen, bool primary);
                static LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
                
            public:
                Aegis::W::Console *console;
                
            protected:
                virtual int Init(); // after window initialization
                virtual int Close(); // before window is destroyed
                
                // Called between Init and Close and only if FullMsgHandler
                // returned 0.
                virtual int MessageHandler(HWND, UINT, WPARAM, LPARAM);
                
                // Primary message handler, is called before Init
                // and may be called after Close.
                // If returns nonzero, MessageHandler is not called then.
                virtual int FullMsgHandler(HWND, UINT, WPARAM, LPARAM);
                
            public:
                Window();
                Window( char * szAppName, char * szIcon,
                        bool fullscreen = false, bool single = true);
                ~Window();

                bool Fullscreen(bool);
                bool Center(int sizex = 0, int sizey = 0);
                void CloseRequest();
                
                HWND GetHWND() const { return this->hwnd; };
                HDC GetHDC() const { return this->hDC; };

                operator HWND() const { return this->hwnd; };
                operator HDC() const { return this->hDC; };
        };
    };
};        

#endif

